@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* BoltSceneNode:
@@ -21,8 +21,6 @@
 #include "ShotSceneNode.h"
 #include "OpenGLLight.h"
 
-class OpenGLTexture;
-
 class BoltSceneNode : public ShotSceneNode {
   public:
 			BoltSceneNode(const GLfloat pos[3]);
@@ -30,10 +28,10 @@ class BoltSceneNode : public ShotSceneNode {
 
     void		setFlares(bool);
     void		setSize(float radius);
-    void		setColor(GLfloat r, GLfloat g, GLfloat b);
+    void		setColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a = 1.0f);
+    void		setTextureColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a = 1.0f);
     void		setColor(const GLfloat* rgb);
-    void		setTexture(const OpenGLTexture&);
-    void		setColorblindTexture(const OpenGLTexture&);
+    void		setTexture(const int);
     void		setTextureAnimation(int cu, int cv);
 
     bool		getColorblind() const;
@@ -42,7 +40,7 @@ class BoltSceneNode : public ShotSceneNode {
     void		move(const GLfloat pos[3], const GLfloat forward[3]);
     void		addLight(SceneRenderer&);
 
-    void		notifyStyleChange(const SceneRenderer&);
+    void		notifyStyleChange();
     void		addRenderNodes(SceneRenderer&);
 
   protected:
@@ -50,19 +48,21 @@ class BoltSceneNode : public ShotSceneNode {
       public:
 			BoltRenderNode(const BoltSceneNode*);
 			~BoltRenderNode();
-	void		setColor(const GLfloat* rgb);
+	void		setColor(const GLfloat* rgba);
+  void    setTextureColor(const GLfloat* rgba);
 	void		render();
-	const GLfloat*	getPosition() { return sceneNode->getSphere(); }
+	const GLfloat*	getPosition() const { return sceneNode->getSphere(); }
 	void		setAnimation(int cu, int cv);
       private:
 	const BoltSceneNode* sceneNode;
 	int		u, v, cu, cv;
 	GLfloat		du, dv;
-	GLfloat		mainColor[3];
-	GLfloat		innerColor[3];
+	GLfloat		mainColor[4];
+	GLfloat		innerColor[4];
 	GLfloat		outerColor[4];
 	GLfloat		coronaColor[4];
 	GLfloat		flareColor[4];
+  GLfloat		textureColor[4];
 	int		numFlares;
 	float		theta[6];
 	float		phi[6];
@@ -81,7 +81,7 @@ class BoltSceneNode : public ShotSceneNode {
     bool		texturing;
     bool		colorblind;
     float		size;
-    GLfloat		color[3];
+    GLfloat		color[4];
     OpenGLLight		light;
     OpenGLGState	gstate;
     OpenGLGState	colorblindGState;
