@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* FlagSceneNode:
@@ -28,8 +28,10 @@ class FlagSceneNode : public SceneNode {
 
     static void		waveFlag(float dt);
     static void		freeFlag();
+
     void		move(const GLfloat pos[3]);
-    void		turn(GLfloat angle);
+    void		setAngle(GLfloat angle);
+    void		setWind(const GLfloat wind[3], float dt);
     void		setBillboard(bool billboard);
 
     const GLfloat*	getColor() const { return color; }
@@ -42,6 +44,8 @@ class FlagSceneNode : public SceneNode {
     void		addRenderNodes(SceneRenderer&);
     void		addShadowNodes(SceneRenderer&);
 
+    bool		cullShadow(int planeCount,
+				   const float (*planes)[4]) const;
   protected:
     class FlagRenderNode : public RenderNode {
       public:
@@ -58,13 +62,13 @@ class FlagSceneNode : public SceneNode {
   private:
     bool		billboard;
     GLfloat		angle;
+    GLfloat		tilt;
+    GLfloat		hscl;
     GLfloat		color[4];
     bool		transparent;
     bool		texturing;
     OpenGLGState	gstate;
     FlagRenderNode	renderNode;
-    static const GLfloat Width;
-    static const GLfloat Height;
 };
 
 #endif // BZF_FLAG_SCENE_NODE_H
