@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* TetraBuilding:
@@ -27,6 +27,7 @@
 #include "MeshFace.h"
 #include "MeshTransform.h"
 
+class MeshDrawInfo;
 
 class MeshObstacle : public Obstacle {
   public:
@@ -44,7 +45,8 @@ class MeshObstacle : public Obstacle {
 		  const std::vector<int>& normals,
 		  const std::vector<int>& texcoords,
 		  const BzMaterial* bzMaterial, int physics,
-		  bool noclusters, bool bounce, bool drive, bool shoot);
+		  bool noclusters, bool bounce, bool drive, bool shoot,
+		  bool triangulate);
 
     ~MeshObstacle();
 
@@ -53,6 +55,9 @@ class MeshObstacle : public Obstacle {
     Obstacle* copyWithTransform(const MeshTransform&) const;
     void copyFace(int face, MeshObstacle* mesh) const;
 
+    void setName(const std::string& name);
+    const std::string&	getName() const;
+
     enum CheckType {
       CheckInside =  0,
       CheckOutside = 1,
@@ -99,16 +104,27 @@ class MeshObstacle : public Obstacle {
     bool useSmoothBounce() const;
     bool noClusters() const;
 
+    const MeshDrawInfo* getDrawInfo() const;
+    void setDrawInfo(MeshDrawInfo*);
+
     int packSize() const;
     void *pack(void*) const;
     void *unpack(void*);
 
     void print(std::ostream& out, const std::string& indent) const;
+    void printOBJ(std::ostream& out, const std::string& indent) const;
 
   private:
+    void makeFacePointers(const std::vector<int>& _vertices,
+			  const std::vector<int>& _normals,
+			  const std::vector<int>& _texcoords,
+			  float**& v, float**& n, float**& t);
 
+  private:
     static const char* typeName;
 
+    std::string name;
+
     int checkCount;
     char* checkTypes;
     fvec3* checkPoints;
@@ -125,6 +141,8 @@ class MeshObstacle : public Obstacle {
     bool inverted; // used during building. can be ditched if
 		   // edge tables are setup with bi-directional
 		   // ray-vs-face tests and parity counts.
+
+    MeshDrawInfo* drawInfo; // hidden data stored in extra texcoords
 };
 
 inline const char *MeshObstacle::getCheckTypes() const
@@ -187,6 +205,22 @@ inline bool MeshObstacle::noClusters() const
   return noclusters;
 }
 
+inline const MeshDrawInfo* MeshObstacle::getDrawInfo() const
+{
+  return drawInfo;
+}
+
+inline const std::string& MeshObstacle::getName() const
+{
+  return name;
+}
+
+inline void MeshObstacle::setName(const std::string& str)
+{
+  name = str;
+  return;
+}
+
 #endif // BZF_MESH_OBSTACLE_H
 
 // Local Variables: ***
