@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /*
@@ -66,18 +66,22 @@ const uint16_t		MsgAlive = 0x616c;			// 'al'
 const uint16_t		MsgAddPlayer = 0x6170;			// 'ap'
 const uint16_t		MsgAutoPilot = 0x6175;			// 'au'
 const uint16_t		MsgCaptureFlag = 0x6366;		// 'cf'
+const uint16_t		MsgCustomSound = 0x6373;		// 'cs'
 const uint16_t		MsgCacheURL = 0x6375;			// 'cu'
 const uint16_t		MsgDropFlag = 0x6466;			// 'df'
 const uint16_t		MsgEnter = 0x656e;			// 'en'
 const uint16_t		MsgExit = 0x6578;			// 'ex'
 const uint16_t		MsgFlagUpdate = 0x6675;			// 'fu'
+const uint16_t		MsgFetchResources = 0x6672;		// 'fr'
 const uint16_t		MsgGrabFlag = 0x6766;			// 'gf'
 const uint16_t		MsgGMUpdate = 0x676d;			// 'gm'
 const uint16_t		MsgGetWorld = 0x6777;			// 'gw'
 const uint16_t		MsgGameSettings = 0x6773;		// 'gs'
+const uint16_t		MsgGameTime = 0x6774;			// 'gt'
 const uint16_t		MsgKilled = 0x6b6c;			// 'kl'
 const uint16_t		MsgKrbPrincipal = 0x6b70;		// 'kp'
 const uint16_t		MsgKrbTicket    = 0x6b74;		// 'kt'
+const uint16_t		MsgLagState = 0x6c73;			// 'ls'
 const uint16_t		MsgMessage = 0x6d67;			// 'mg'
 const uint16_t		MsgNewRabbit = 0x6e52;			// 'nR'
 const uint16_t		MsgNegotiateFlags = 0x6e66;		// 'nf'
@@ -102,6 +106,9 @@ const uint16_t		MsgTransferFlag = 0x7466;		// 'tf'
 const uint16_t		MsgTeamUpdate = 0x7475;			// 'tu'
 const uint16_t		MsgWantWHash = 0x7768;			// 'wh'
 const uint16_t		MsgWantSettings = 0x7773;		// 'ws'
+const uint16_t		MsgPortalAdd = 0x5061;			// 'Pa'
+const uint16_t		MsgPortalRemove = 0x5072;		// 'Pr'
+const uint16_t		MsgPortalUpdate = 0x5075;		// 'Pu'
 
 // world database codes
 const uint16_t		WorldCodeHeader = 0x6865;		// 'he'
@@ -157,6 +164,11 @@ const uint16_t		RejectRepeatCallsign = 0x0007;
 const uint16_t		RejectRejoinWaitTime = 0x0008;
 const uint16_t		RejectIPBanned = 0x0009;
 const uint16_t		RejectHostBanned = 0x000A;
+const uint16_t		RejectIDBanned = 0x000B;
+
+// sound type codes
+const uint16_t		LocalCustomSound = 0x0001;
+
 
 // death by obstacle
 // FIXME: really a killed reason, NOT a message type.
