@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* SceneDatabase:
@@ -26,6 +26,7 @@ class ViewFrustum;
 class SceneNode;
 class SphereSceneNode;
 class SceneRenderer;
+class Extents;
 
 
 // NOTE -- SceneDatabase owns all static nodes added to it,
@@ -40,6 +41,7 @@ class SceneDatabase {
     virtual bool	addStaticNode(SceneNode*, bool dontFree) = 0;
     virtual void	addDynamicNode(SceneNode*) = 0;
     virtual void	addDynamicSphere(SphereSceneNode*) = 0;
+    virtual void	finalizeStatics() = 0;
     virtual void	removeDynamicNodes() = 0;
     virtual void	removeAllNodes() = 0;
     virtual bool	isOrdered() = 0;
@@ -52,6 +54,10 @@ class SceneDatabase {
 
     virtual void	drawCuller() = 0;
 
+    virtual void	setOccluderManager(int);
+
+    virtual const Extents* getVisualExtents() const { return NULL; }
+
   private:
 			SceneDatabase(const SceneDatabase&);
     SceneDatabase&	operator=(const SceneDatabase&);
