@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* SceneRenderer:
@@ -43,7 +43,7 @@ class SceneNode;
 class BackgroundRenderer;
 class HUDRenderer;
 class MainWindow;
-
+class Extents;
 
 class FlareLight {
 public:
@@ -100,6 +100,8 @@ public:
 
   void		setExposed();
 
+  void		clearRadar(float opacity);
+
   void		getGroundUV(const float p[2], float uv[2]) const;
 
   bool		getBlank() const;
@@ -126,17 +128,24 @@ public:
   void		enableSun(bool = true); // toggle light state
 
   void		setTimeOfDay(double julianDay);
+
   const GLfloat*	getSunColor() const;
   const GLfloat*	getSunScaledColor() const;
   GLfloat		getSunBrightness() const;
   const GLfloat*	getSunDirection() const;
+  const GLfloat*	getAmbientColor() const;
   const GLfloat*	getCelestialTransform() const;
 
   SceneDatabase*	getSceneDatabase() const;
-  void		setSceneDatabase(SceneDatabase*);
+  void			setSceneDatabase(SceneDatabase*);
+
+  const Extents*	getVisualExtents() const;
+  const float		getLengthPerPixel() const;
+
+  int			getFrameTriangleCount() const;
 
   BackgroundRenderer*	getBackground();
-  void		setBackground(BackgroundRenderer*);
+  void			setBackground(BackgroundRenderer*);
 
   const RenderNodeList& getShadowList() const;
 
@@ -172,27 +181,30 @@ private:
   void		renderPostDimming();
 
   MainWindow*		window;
-  bool		blank;
-  bool		invert;
-  bool		mirror;
-  bool		drawGround;
-  bool		clearZbuffer;
+  bool			blank;
+  bool			invert;
+  bool			mirror;
+  bool			drawGround;
+  bool			clearZbuffer;
   ViewFrustum		frustum;
-  GLint		maxLights;
-  GLint		reservedLights;
-  GLint		dynamicLights;
-  int		lightsSize;
-  int		lightsCount;
-  OpenGLLight**	lights;
+  float			lengthPerPixel;
+  GLint			maxLights;
+  GLint			reservedLights;
+  GLint			dynamicLights;
+  int			lightsSize;
+  int			lightsCount;
+  OpenGLLight**		lights;
   OpenGLLight		theSun;
-  bool		sunOrMoonUp;
+  bool			sunOrMoonUp;
   GLfloat		sunDirection[3];	// or moon
-  GLfloat		sunColor[3];
-  GLfloat		sunScaledColor[3];
+  GLfloat		sunColor[4];
+  GLfloat		sunScaledColor[4];
   GLfloat		celestialTransform[16];
   GLfloat		sunBrightness;
+  GLfloat		ambientColor[4];
   SceneDatabase*	scene;
   BackgroundRenderer*	background;
+  int			triangleCount;
   static const GLint	SunLight;
 
   static const float dimDensity;
@@ -268,6 +280,11 @@ inline ViewFrustum&		SceneRenderer::getViewFrustum()
   return frustum;
 }
 
+inline const float		SceneRenderer::getLengthPerPixel() const
+{
+  return lengthPerPixel;
+}
+
 inline const OpenGLLight&	SceneRenderer::getLight(int index) const
 {
   return *(lights[index]);
@@ -288,6 +305,11 @@ inline GLfloat			SceneRenderer::getSunBrightness() const
   return sunBrightness;
 }
 
+inline const GLfloat* SceneRenderer::getAmbientColor() const
+{
+  return ambientColor;
+}
+
 inline const GLfloat*		SceneRenderer::getCelestialTransform() const
 {
   return celestialTransform;
