@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /**
@@ -25,6 +25,7 @@
 
 #include "common.h"
 
+/* system interface headers */
 #include <string>
 
 
@@ -41,13 +42,23 @@ public:
   ~TimeKeeper();
   TimeKeeper&		operator=(const TimeKeeper&);
 
-  float		operator-(const TimeKeeper&) const;
-  bool		operator<=(const TimeKeeper&) const;
-  TimeKeeper&		operator+=(float);
-  TimeKeeper&		operator+=(const TimeKeeper&) ;
+  double		operator-(const TimeKeeper&) const;
+  bool			operator<=(const TimeKeeper&) const;
+  TimeKeeper&		operator+=(double);
+  TimeKeeper&		operator+=(const TimeKeeper&);
+
+	// make a TimeKeeper with seconds = NULL act like unset
+	// Fixme: must this be defined here? didn't work for me outside the class
+	inline operator void*()
+	{
+		if (seconds > 0.0)
+			return this;
+		else
+			return NULL;
+	}
 
   /** returns how many seconds have elapsed since epoch, Jan 1, 1970 */
-  float	       getSeconds(void) const;
+  double       getSeconds(void) const;
 
   /** returns a timekeeper representing the current time */
   static const TimeKeeper&	getCurrent(void);
@@ -67,19 +78,25 @@ public:
   /** returns a timekeeper representing an unset timekeeper */
   static const TimeKeeper&	getNullTime(void);
 
+
+  /** returns the local time */
+  static void localTime(int *year = NULL, int *month = NULL, int* day = NULL, int* hour = NULL, int* min = NULL, int* sec = NULL, bool* dst = NULL);
+
   /** returns a string of the local time */
   static const char		*timestamp(void);
 
-  /** converts a float time difference into an array of integers
+  static void localTime( int &day);
+
+  /** converts a time difference into an array of integers
       representing days, hours, minutes, seconds */
-  static const void		convertTime(float raw, int convertedTimes[]);
+  static const void		convertTime(double raw, long int convertedTimes[]);
   /** prints an integer-array time difference in human-readable form */
-  static const std::string	printTime(int timeValue[]);
+  static const std::string	printTime(long int timeValue[]);
   /** prints an float time difference in human-readable form */
-  static const std::string	printTime(float diff);
+  static const std::string	printTime(double diff);
 
   /** sleep for a given number of floating point seconds */
-  static void			sleep(float secs); //const
+  static void			sleep(double secs); //const
 
 private:
   double		seconds;
@@ -117,19 +134,19 @@ inline TimeKeeper&	TimeKeeper::operator=(const TimeKeeper& t)
   return *this;
 }
 
-inline float		TimeKeeper::operator-(const TimeKeeper& t) const
+inline double		TimeKeeper::operator-(const TimeKeeper& t) const
 {
-  return (float)(seconds - t.seconds);
+  return seconds - t.seconds;
 }
 
-inline TimeKeeper&	TimeKeeper::operator+=(float dt)
+inline TimeKeeper&	TimeKeeper::operator+=(double dt)
 {
-  seconds += double(dt);
+  seconds += dt;
   return *this;
 }
 inline TimeKeeper&	TimeKeeper::operator+=(const TimeKeeper& t)
 {
-  seconds += double(t.seconds);
+  seconds += t.seconds;
   return *this;
 }
 
@@ -138,9 +155,9 @@ inline bool		TimeKeeper::operator<=(const TimeKeeper& t) const
   return seconds <= t.seconds;
 }
 
-inline float		TimeKeeper::getSeconds(void) const
+inline double		TimeKeeper::getSeconds(void) const
 {
-  return (float)seconds;
+  return seconds;
 }
 
 
