@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* WallSceneNode:
@@ -40,8 +40,6 @@ class WallSceneNode : public SceneNode {
     GLfloat		getDistance(const GLfloat*) const;
     virtual bool	inAxisBox (const Extents& exts) const;
 
-    bool		isTranslucent() const;
-
     void		setColor(GLfloat r, GLfloat g,
 				GLfloat b, GLfloat a = 1.0f);
     void		setColor(const GLfloat* rgba);
@@ -56,7 +54,7 @@ class WallSceneNode : public SceneNode {
     void		setLightedModulateColor(const GLfloat* rgba);
     void		setMaterial(const OpenGLMaterial&);
     void		setTexture(const int);
-    void		setTextureMatrix(const int);
+    void		setTextureMatrix(const GLfloat* texmat);
     void		setDynamicColor(const float* color);
     void		setBlending(bool);
     void		setSphereMap(bool);
