@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __VECTOR_MATH_H__
@@ -65,6 +65,13 @@ class cfvec4 {
 // FIXME - some these should go into a common file
 // Some handy geometry functions
 
+static inline void vec3add (float *result, const float* v1, const float* v2)
+{
+  result[0] = v1[0] + v2[0];
+  result[1] = v1[1] + v2[1];
+  result[2] = v1[2] + v2[2];
+  return;
+}
 static inline void vec3sub (float *result, const float* v1, const float* v2)
 {
   result[0] = v1[0] - v2[0];
