@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 #
 # bzflag
-# Copyright (c) 1993 - 2005 Tim Riker
+# Copyright (c) 1993 - 2006 Tim Riker
 #
 # This package is free software;  you can redistribute it and/or
 # modify it under the terms of the license found in the file
@@ -9,7 +9,7 @@
 #
 # THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 # IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
-# WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
 =pod
 
@@ -75,8 +75,15 @@ die "rejected by server" if ($id == 255);
 # send game request
 print S pack("n2", 0, 0x7167);
 
-# get reply
-die $! unless sysread(S, $buffer, 46) == 46;
+my $nbytes = sysread(S, $buffer, 46);
+if ($nbytes == 12) {  # if MsgGameTime rxed ... ignore it
+  my ($len, $code) = unpack("n2", $buffer);
+  $nbytes = sysread(S, $buffer, 46);
+}
+if ($nbytes != 46) {
+  die "Error: $nbytes bytes, expecting 46: $^E\n";
+}
+
 ($len,$code,$style,$maxPlayers,$maxShots,
 	$rogueSize,$redSize,$greenSize,$blueSize,$purpleSize,$obsSize,
 	$rogueMax,$redMax,$greenMax,$blueMax,$purpleMax,$obsMax,
