@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /** @file
@@ -77,45 +77,78 @@ inline int cr_waddstr(WINDOW* w, const char* str) {
 #  define HAVE_PROTO
 #endif
 
+// a complete and total hack, but its needed since windows.h ALSO has this
+#ifdef MOUSE_MOVED
+#undef MOUSE_MOVED
+#endif
+
+// protect from curses.h defining a min/max macro
+#ifndef min
+#  define no_min_def
+#endif
+#ifndef max
+#  define no_max_def
+#endif
+
 #ifdef HAVE_XCURSES_H
 #  define XCURSES
 #  include <xcurses.h>
 #else
+#  define NOMACROS 1
 #  include <curses.h>
 #endif
 
+// protect from curses.h defining a min/max macro
+#ifdef no_min_def
+#  ifdef min
+#    undef min
+#  endif
+#endif
+#ifdef no_max_def
+#  ifdef max
+#    undef max
+#  endif
+#endif
+
+
 #define COLOR_BGDEFAULT COLOR_BLACK
 #define COLOR_FGDEFAULT COLOR_WHITE
 
-// stop ugly macros from polluting our namespace (pdcurses doesn't use
-// the NOMACROS preprocessor variable)
-#undef erase
-inline int erase() {
-  return werase(stdscr);
-}
+// old pdcurses requires some workarounds
+#if (!defined(PDC_BUILD) || PDC_BUILD < 2800)
+  // stop ugly macros from polluting our namespace (pdcurses doesn't use
+  // the NOMACROS preprocessor variable)
+  #undef erase
+  inline int erase() {
+    return werase(stdscr);
+  }
 
-#undef clear
-inline int clear() {
-  return wclear(stdscr);
-}
+  #undef clear
+  inline int clear() {
+    return wclear(stdscr);
+  }
 
-#undef move
-inline int move(int y, int x) {
-  return wmove(stdscr, y, x);
-}
+  #undef move
+  inline int move(int y, int x) {
+    return wmove(stdscr, y, x);
+  }
 
-#undef nonl
-inline int nonl() {
-  return OK;
-}
+  #undef nonl
+  inline int nonl() {
+    return OK;
+  }
 
-// wrap some functions to make it compatible with ncurses
-inline int pd_waddstr(WINDOW* w, const char* str) {
-  char* newStr = new char[strlen(str) + 1];
-  strcpy(newStr, str);
-  return waddstr(w, newStr);
-}
-#define waddstr(W, C) pd_waddstr(W, C)
+  // wrap some functions to make it compatible with ncurses
+  inline int pd_waddstr(WINDOW* w, const char* str) {
+    char* newStr = new char[strlen(str) + 1];
+    strcpy(newStr, str);
+    return waddstr(w, newStr);
+  }
+  #define waddstr(W, C) pd_waddstr(W, C)
+
+  // wresize needs to preserve the pointer to the window
+  #define wresize(w, l, c) ((w = resize_window(w, l, c)) ? OK : ERR)
+#endif // old pdcurses
 
 #ifdef XCURSES
 inline int pd_endwin() {
@@ -126,10 +159,6 @@ inline int pd_endwin() {
 #define endwin pd_endwin
 #endif // XCURSES
 
-inline int wresize(WINDOW* w, int lines, int cols) {
-  return (resize_window(w, lines, cols) != NULL);
-}
-
 inline int resizeterm(int lines, int cols) {
   return resize_term(lines, cols);
 }
@@ -142,7 +171,7 @@ inline void use_default_colors() {
 
 
 
-#endif 
+#endif
 // Local Variables: ***
 // mode: C++ ***
 // tab-width: 8 ***
