@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,20 +7,19 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
-#include "common.h"
 #include "BaseLocalPlayer.h"
 
 /* common implementation headers */
 #include "BZDBCache.h"
 
 
-BaseLocalPlayer::BaseLocalPlayer(const PlayerId& id,
-				 const char* name, const char* email) :
-  Player(id, RogueTeam, name, email, TankPlayer),
+BaseLocalPlayer::BaseLocalPlayer(const PlayerId& _id,
+				 const char* name, const char* _email) :
+  Player(_id, RogueTeam, name, _email, TankPlayer),
   lastTime(TimeKeeper::getTick()),
   salt(0)
 {
@@ -52,7 +51,7 @@ void BaseLocalPlayer::update()
   lastPosition[2] = oldPosition[2];
 
   // update by time step
-  float dt = TimeKeeper::getTick() - lastTime;
+  float dt = float(TimeKeeper::getTick() - lastTime);
   lastTime = TimeKeeper::getTick();
   if (dt < 0.001f) dt = 0.001f;
   doUpdateMotion(dt);
