@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
@@ -15,10 +15,8 @@
 
 /* common interface headers */
 #include "BzfJoystick.h"
-#include "StateDatabase.h"
 
 /* local implementation headers */
-#include "MainMenu.h"
 #include "LocalPlayer.h"
 #include "playing.h"
 
@@ -61,15 +59,19 @@ namespace ForceFeedback {
   void death()
   {
     /* Nice long hard rumble for death */
-    if (useForceFeedback())
+    if (useForceFeedback("Rumble"))
       getJoystick()->ffRumble(1, 0.0f, 1.5f, 1.0f, 0.0f);
+    else if (useForceFeedback("Directional"))
+      getJoystick()->ffDirectionalPeriodic(1, 0.0f, 1.5f, 1.0f, 0.0f, 1.0f, 0.15f, BzfJoystick::FF_SawtoothDown);
   }
 
   void shotFired()
   {
     /* Tiny little kick for a normal shot being fired */
-    if (useForceFeedback())
+    if (useForceFeedback("Rumble"))
       getJoystick()->ffRumble(1, 0.0f, 0.1f, 0.0f, 1.0f);
+    else if (useForceFeedback("Directional"))
+      getJoystick()->ffDirectionalConstant(1, 0.0f, 0.1f, 0.0f, -1.0f, 0.5f);
   }
 
   void laserFired()
@@ -79,10 +81,36 @@ namespace ForceFeedback {
      *  some quirks in its driver may mean it's feeling a little different
      *  than it should)
      */
-    if (useForceFeedback())
+    if (useForceFeedback("Rumble"))
       getJoystick()->ffRumble(4, 0.01f, 0.02f, 1.0f, 1.0f);
+    else if (useForceFeedback("Directional"))
+      getJoystick()->ffDirectionalPeriodic(4, 0.1f, 0.1f, 0.0f, -1.0f, 0.5f, 0.05f, BzfJoystick::FF_Sine);
   }
 
+  void shockwaveFired()
+  {
+    /* try to 'match' the shockwave sound */
+    if (useForceFeedback("Rumble"))
+      getJoystick()->ffRumble(1, 0.0f, 0.5f, 0.0f, 1.0f);
+    else if (useForceFeedback("Directional"))
+      getJoystick()->ffDirectionalPeriodic(1, 0.0f, 1.0f, 0.0f, -1.0f, 0.5f, 0.1f, BzfJoystick::FF_Sine);
+  }
+
+  /* Burrowed, oscillating, etc, tanks get a special resistance force
+   * when moving through solid matter.  We use half-second increments
+   * of force-on time.
+   */
+  static TimeKeeper friction_timer = TimeKeeper::getSunGenesisTime();
+  void solidMatterFriction()
+  {
+    /* There is no way to simulate this with a rumble effect */
+    if (useForceFeedback("Directional")) {
+      if ((TimeKeeper::getCurrent() - friction_timer) >= 0.5f) {
+	getJoystick()->ffDirectionalResistance(0.5f, 1.0f, 0.5f, BzfJoystick::FF_Position);
+	friction_timer = TimeKeeper::getCurrent();
+      }
+    }
+  }
 }
 
 // Local Variables: ***
