@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,28 +7,21 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
 #include "GuidedMissleStrategy.h"
 
 /* common implementation headers */
-#include "StateDatabase.h"
 #include "BZDBCache.h"
 #include "TextureManager.h"
 #include "Intersect.h"
-#include "ObstacleMgr.h"
 
 /* local implementation headers */
 #include "LocalPlayer.h"
 #include "World.h"
-
-/* FIXME - from player.h */
-void addShotExplosion(const float* pos);
-Player* lookupPlayer(PlayerId id);
-void addShotPuff(const float* pos);
-
+#include "playing.h"
 
 static float limitAngle(float a)
 {
@@ -99,6 +92,9 @@ GuidedMissileStrategy::GuidedMissileStrategy(ShotPath* _path) :
 
   // no last target
   lastTarget = NoPlayer;
+
+  lastPuff = currentTime;
+  puffTime = BZDB.eval("gmPuffTime");
 }
 
 GuidedMissileStrategy::~GuidedMissileStrategy()
@@ -203,11 +199,11 @@ void GuidedMissileStrategy::update(float dt)
   newDirection[2] = sinf(elevation);
   Ray ray = Ray(nextPos, newDirection);
 
-  // Changed: GM leave smoke trail, call add puff every 3 updates
-  if (!BZDB.isTrue ("SupressGMSmoke")) {
-    if ((++renderTimes % 3) == 0) {
-      addShotPuff (nextPos);
-    }
+  renderTimes++;
+  // Changed: GM smoke trail, leave it every seconds, none of this per frame crap
+  if (currentTime.getSeconds() - lastPuff.getSeconds() > puffTime ) {
+    lastPuff = currentTime;
+    addShotPuff(nextPos,azimuth,elevation);
   }
 
   // get next position
@@ -251,7 +247,7 @@ void GuidedMissileStrategy::update(float dt)
 float GuidedMissileStrategy::checkBuildings(const Ray& ray)
 {
   float shotSpeed = BZDB.eval(StateDatabase::BZDB_SHOTSPEED);
-  float t = (currentTime - prevTime) * shotSpeed;
+  float t = float((currentTime - prevTime) * shotSpeed);
   int face;
   const Obstacle* building = getFirstBuilding(ray, Epsilon, t);
   const Teleporter* teleporter = getFirstTeleporter(ray, Epsilon, t, face);
