@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,24 +7,16 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 // interface headers
 #include "HUDuiTypeIn.h"
-#include "HUDuiControl.h"
 
-// system headers
-#include <math.h>
+// system implementation headers
 #include <ctype.h>
-#include <string>
 
 // common implementation headers
-#include "common.h"
-#include "bzfgl.h"
-#include "BzfEvent.h"
-#include "BundleMgr.h"
-#include "Bundle.h"
 #include "FontManager.h"
 
 // local implementation headers
@@ -37,7 +29,7 @@
 HUDuiTypeIn::HUDuiTypeIn()
 : HUDuiControl(), maxLength(0), cursorPos(0)
 {
-  allowEdit = true; //by default allow editing
+  allowEdit = true; // allow editing by default
   obfuscate = false;
 }
 
@@ -47,7 +39,7 @@ HUDuiTypeIn::~HUDuiTypeIn()
 
 void		HUDuiTypeIn::setObfuscation(bool on)
 {
-	obfuscate = on;
+  obfuscate = on;
 }
 
 int			HUDuiTypeIn::getMaxLength() const
@@ -124,9 +116,9 @@ bool			HUDuiTypeIn::doKeyPress(const BzfKeyEvent& key)
       if (cursorPos < (int)string.length()) {
 	cursorPos++;
 	c = backspace;
-      }
-      else
+      } else {
 	return true;
+      }
       break;
 
     default:
@@ -180,10 +172,10 @@ void			HUDuiTypeIn::doRender()
   FontManager &fm = FontManager::instance();
   std::string renderStr;
   if (obfuscate) {
-	renderStr.append(string.size(), '*');
-  }
-  else
+    renderStr.append(string.size(), '*');
+  } else {
     renderStr = string;
+  }
   fm.drawString(getX(), getY(), 0, getFontFace(), getFontSize(), renderStr);
 
   // find the position of where to draw the input cursor
