@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,34 +7,23 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /*
  *
  */
 
-//#define SHOOTING_FIX
-
-// bzflag common header
-#include "common.h"
-
-// system headers
-#include <stdlib.h>
-#include <math.h>
-#include <vector>
+// interface header
+#include "RobotPlayer.h"
 
 // common implementation headers
 #include "BZDBCache.h"
 
 // local implementation headers
 #include "World.h"
-#include "RobotPlayer.h"
-#include "RemotePlayer.h"
 #include "Intersect.h"
-#include "ShotStrategy.h" // need this for rayAtDistanceFromOrigin
 #include "TargetingUtils.h"
-#include "ServerLink.h"
 
 std::vector<BzfRegion*>* RobotPlayer::obstacleList = NULL;
 
@@ -71,10 +60,10 @@ void RobotPlayer::projectPosition(const Player *targ,const float t,float &x,floa
   else
   {
     double hisspeed = hypotf(hisvx, hisvy);
-    double alpha = omega * t;
+    double alfa = omega * t;
     double r = hisspeed / fabs(omega);
-    double dx = r * sin(alpha);
-    double dy2 = r * (1 - cos(alpha));
+    double dx = r * sin(alfa);
+    double dy2 = r * (1 - cos(alfa));
     double beta = atan2(dy2, dx) * (targ->getAngularVelocity() > 0 ? 1 : -1);
     double gamma = atan2(hisvy, hisvx);
     double rho = gamma+beta;
@@ -253,7 +242,7 @@ void			RobotPlayer::doUpdateMotion(float dt)
 	if (dist < 150.0f) {
 	  const float *shotVel = shot->getVelocity();
 	  float shotAngle = atan2f(shotVel[1], shotVel[0]);
-	  float shotUnitVec[2] = {cos(shotAngle), sin(shotAngle)};
+	  float shotUnitVec[2] = {cosf(shotAngle), sinf(shotAngle)};
 
 	  float trueVec[2] = {(position[0]-shotPos[0])/dist,(position[1]-shotPos[1])/dist};
 	  float dotProd = trueVec[0]*shotUnitVec[0]+trueVec[1]*shotUnitVec[1];
@@ -387,7 +376,7 @@ void		    RobotPlayer::setObstacleList(std::vector<BzfRegion*>*
 						     _obstacleList)
 {
   obstacleList = _obstacleList;
-};
+}
 
 const Player*		RobotPlayer::getTarget() const
 {
