@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* FIXME -- ugh.. the class header for this file is listed as a public header
@@ -16,39 +16,33 @@
  * may be moved elsewhere.
  */
 
-#include "common.h"
-
-/* system implementation headers */
-#include <stdlib.h>
-#include <string.h>
+// interface header
+#include "SceneRenderer.h"
 
 /* common implementation headers */
-#include "bzfgl.h"
-#include "SceneRenderer.h"
 #include "SceneDatabase.h"
-#include "SceneNode.h"
 #include "MainWindow.h"
-#include "OpenGLGState.h"
-#include "RenderNode.h"
 #include "DynamicColor.h"
 #include "TextureMatrix.h"
-#include "BzfWindow.h"
 #include "TankSceneNode.h"
 #include "StateDatabase.h"
 #include "TextUtils.h"
 #include "ParseColor.h"
 #include "BZDBCache.h"
-#include "TankGeometryMgr.h"
+#include "MeshSceneNode.h"
 
 /* FIXME - local implementation dependancies */
 #include "BackgroundRenderer.h"
-#include "HUDRenderer.h"
 #include "LocalPlayer.h"
 #include "daylight.h"
 #include "World.h"
 #include "TrackMarks.h"
 
 
+static bool mapFog;
+static bool setupMapFog();
+
+
 #ifdef GL_ABGR_EXT
 static int		strrncmp(const char* s1, const char* s2, int num)
 {
@@ -142,8 +136,8 @@ void SceneRenderer::setWindow(MainWindow* _window) {
   window->getWindow()->makeCurrent();
   GLint bits;
   if (!BZDB.isSet("zbuffer")) {
-    glGetIntegerv(GL_DEPTH_BITS, &bits);
-    BZDB.set("zbuffer", (bits > 0) ? "1" : "0");
+//	  glGetIntegerv(GL_DEPTH_BITS, &bits);
+    BZDB.set("zbuffer", "1");
   }
   glGetIntegerv(GL_STENCIL_BITS, &bits);
   useStencilOn = (bits > 0);
@@ -278,7 +272,9 @@ void SceneRenderer::setQuality(int value)
   if (useQualityValue >= 2) {
     glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
     glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
-    glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
+    // GL_NICEST for polygon smoothing seems to make some drivers
+    // cause massive slowdowns and "spikes" when drawing the radar
+    glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
   } else {
     glHint(GL_LINE_SMOOTH_HINT, GL_FASTEST);
     glHint(GL_POINT_SMOOTH_HINT, GL_FASTEST);
@@ -306,10 +302,33 @@ void SceneRenderer::setQuality(int value)
   else
     BZDB.set("moonSegments","12");
 
-  if (useQualityValue > 0)
+  if (useQualityValue > 0) {
+    // this can be modified by OpenGLMaterial
     glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
-  else
+  } else {
+    // OpenGLMaterial will not modify if (quality <= 0)
     glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
+  }
+
+  // this setting helps keep those specular highlights
+  // highlighting when applied to a dark textured surface.
+  // It was mainlined in OpenGL Version 1.2
+  // (there's also the GL_EXT_separate_specular_color extension)
+#ifdef GL_LIGHT_MODEL_COLOR_CONTROL
+  if (useQualityValue >= 2)
+    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
+  else
+    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR);
+#  else // in case someone includes <GL/glext.h> at some point
+#  ifdef GL_LIGHT_MODEL_COLOR_CONTROL_EXT
+  if (useQualityValue >= 2)
+    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL_EXT,
+		  GL_SEPARATE_SPECULAR_COLOR_EXT);
+  else
+    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL_EXT,
+		  GL_SINGLE_COLOR_EXT);
+#  endif
+#endif
 
   BZDB.set("useQuality", TextUtils::format("%d", value));
 }
@@ -341,6 +360,7 @@ void SceneRenderer::setRebuildTanks()
 void SceneRenderer::setupBackgroundMaterials()
 {
   if (background) {
+    background->setupSkybox();
     background->setupGroundMaterials();
   }
   return;
@@ -415,6 +435,8 @@ int SceneRenderer::getRadarSize() const
 
 void SceneRenderer::setMaxMotionFactor(int factor)
 {
+  if (factor < -11)
+    factor = -11;
   maxMotionFactor = factor;
   notifyStyleChange();
   if (window) {
@@ -447,6 +469,16 @@ void SceneRenderer::setExposed()
 }
 
 
+void SceneRenderer::clearRadar(float opacity)
+{
+  int size = window->getHeight() - window->getViewHeight();
+  float op = (opacity > 1.0f) ? 1.0f : (opacity < 0.0f) ? 0.0f : opacity;
+  glScissor(window->getOriginX(), 0, size, size);
+  glClearColor(0.0f, 0.0f, 0.0f, op);
+  glClear(GL_COLOR_BUFFER_BIT);
+}
+
+
 void SceneRenderer::setSceneDatabase(SceneDatabase* db)
 {
   // update the styles
@@ -506,7 +538,7 @@ void SceneRenderer::enableSun(bool on)
 void SceneRenderer::setupSun()
 {
   if (BZDBCache::lighting && sunOrMoonUp) {
-    theSun.execute(SunLight);
+    theSun.execute(SunLight, BZDB.isTrue("lightLists"));
   }
 }
 
@@ -592,7 +624,6 @@ void SceneRenderer::setTimeOfDay(double julianDay)
   }
 
   // set sun and ambient colors
-  GLfloat ambientColor[4];
   ::getSunColor(sunDir, sunColor, ambientColor, sunBrightness);
   theSun.setColor(sunColor);
   GLfloat maxComponent = sunColor[0];
@@ -659,6 +690,12 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
   lastFrame = _lastFrame;
   sameFrame = _sameFrame;
 
+  triangleCount = 0;
+  RenderNode::resetTriangleCount();
+  if (background) {
+    background->resetTriangleCount();
+  }
+
   // update the SceneNode, Background, and TrackMark styles
   if (needStyleUpdate) {
     if (scene) {
@@ -688,6 +725,20 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
     return;
   }
 
+  // setup the viewport LOD scale
+  MeshSceneNode::setLodScale(window->getWidth(), frustum.getFOVx(),
+			     window->getViewHeight(), frustum.getFOVy());
+  {
+    const int pixelsX = window->getWidth();
+    const int pixelsY = window->getViewHeight();
+    const float fovx = frustum.getFOVx();
+    const float fovy = frustum.getFOVy();
+    const float lppx = 2.0f * sinf(fovx * 0.5f) / (float)pixelsX;
+    const float lppy = 2.0f * sinf(fovy * 0.5f) / (float)pixelsY;
+    const float lpp = (lppx < lppy) ? lppx : lppy;
+    lengthPerPixel = lpp * BZDB.eval("lodScale");
+  }
+
   // get the track mark sceneNodes (only for BSP)
   if (scene) {
     TrackMarks::addSceneNodes(scene);
@@ -700,6 +751,14 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
     teleporterProximity = LocalPlayer::getMyTank()->getTeleporterProximity();
   }
 
+  // fog setup
+  mapFog = setupMapFog();
+  const bool reallyUseFogHack = !mapFog && useFogHack &&
+				(useQualityValue >= 2);
+  if (reallyUseFogHack) {
+    renderPreDimming();
+  }
+
 
   mirror = (BZDB.get(StateDatabase::BZDB_MIRROR) != "none")
 	   && BZDB.isTrue("userMirror");
@@ -714,19 +773,23 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
     frustum.flipVertical();
     glFrontFace(GL_CW);
 
+    // different occluders for the mirror
+    if (scene) {
+      scene->setOccluderManager(1);
+    }
+
     // the reflected scene
     renderScene(_lastFrame, _sameFrame, fullWindow);
 
+    // different occluders for the mirror
+    if (scene) {
+      scene->setOccluderManager(0);
+    }
+
     // flip back
     frustum.flipVertical();
     glFrontFace(GL_CCW);
 
-    // darken the reflection
-    glMatrixMode(GL_PROJECTION);
-    glLoadIdentity();
-    glMatrixMode(GL_MODELVIEW);
-    glLoadIdentity();
-
     float mirrorColor[4];
     if (!parseColorString(BZDB.get(StateDatabase::BZDB_MIRROR), mirrorColor)) {
       mirrorColor[0] = mirrorColor[1] = mirrorColor[2] = 0.0f;
@@ -741,19 +804,51 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
       mirrorColor[3] = 0.2f;
     }
 
-    // if low quality then use stipple -- it's probably much faster
-    if (BZDBCache::blend && (useQualityValue >= 2)) {
-      glColor4fv(mirrorColor);
-      glEnable(GL_BLEND);
-      glRectf(-1.0f, -1.0f, +1.0f, +1.0f);
-      glDisable(GL_BLEND);
+    // darken the reflection
+    if (!mapFog) {
+      glMatrixMode(GL_PROJECTION);
+      glLoadIdentity();
+      glMatrixMode(GL_MODELVIEW);
+      glLoadIdentity();
+      // if low quality then use stipple -- it's probably much faster
+      if (BZDBCache::blend && (useQualityValue >= 2)) {
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	glEnable(GL_BLEND);
+	glColor4fv(mirrorColor);
+	glRectf(-1.0f, -1.0f, +1.0f, +1.0f);
+	glDisable(GL_BLEND);
+      } else {
+	float stipple = mirrorColor[3];
+	glColor3fv(mirrorColor);
+	OpenGLGState::setStipple(stipple);
+	glEnable(GL_POLYGON_STIPPLE);
+	glRectf(-1.0f, -1.0f, +1.0f, +1.0f);
+	glDisable(GL_POLYGON_STIPPLE);
+      }
     } else {
-      float stipple = mirrorColor[3];
-      glColor3fv(mirrorColor);
-      OpenGLGState::setStipple(stipple);
-      glEnable(GL_POLYGON_STIPPLE);
-      glRectf(-1.0f, -1.0f, +1.0f, +1.0f);
-      glDisable(GL_POLYGON_STIPPLE);
+      // need the proper matrices for fog generation
+      // if low quality then use stipple -- it's probably much faster
+      frustum.executeView();
+      frustum.executeProjection();
+      const float extent = BZDBCache::worldSize * 10.0f;
+      if (BZDBCache::blend && (useQualityValue >= 2)) {
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	glEnable(GL_BLEND);
+	glColor4fv(mirrorColor);
+	glRectf(-extent, -extent, +extent, +extent);
+	glDisable(GL_BLEND);
+      } else {
+	float stipple = mirrorColor[3];
+	glColor3fv(mirrorColor);
+	OpenGLGState::setStipple(stipple);
+	glEnable(GL_POLYGON_STIPPLE);
+	glRectf(-extent, -extent, +extent, +extent);
+	glDisable(GL_POLYGON_STIPPLE);
+      }
+      glMatrixMode(GL_PROJECTION);
+      glLoadIdentity();
+      glMatrixMode(GL_MODELVIEW);
+      glLoadIdentity();
     }
 
     clearZbuffer = false;
@@ -762,6 +857,23 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
   // the real scene
   renderScene(_lastFrame, _sameFrame, fullWindow);
 
+  // finalize dimming
+  if (mapFog) {
+    glDisable(GL_FOG);
+  }
+  if (reallyUseFogHack) {
+    if ((teleporterProximity > 0.0f) || useDimming) {
+      glDisable(GL_FOG);
+    }
+  } else {
+    renderPostDimming();
+  }
+
+  triangleCount = RenderNode::getTriangleCount();
+  if (background) {
+    triangleCount += background->getTriangleCount();
+  }
+
   return;
 }
 
@@ -770,6 +882,7 @@ void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
 				bool fullWindow)
 {
   int i;
+  const bool lightLists = BZDB.isTrue("lightLists");
 
   // avoid OpenGL calls as long as possible -- there's a good
   // chance we're waiting on the vertical retrace.
@@ -788,22 +901,18 @@ void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
 
   // turn sunlight on -- the ground needs it
   if (BZDBCache::lighting && sunOrMoonUp) {
-    theSun.execute(SunLight);
+    theSun.execute(SunLight, lightLists);
     theSun.enableLight(SunLight, true);
   }
 
-  bool reallyUseFogHack = useFogHack && (useQualityValue >= 2);
-
-  if (reallyUseFogHack) {
-    renderPreDimming();
-  }
-
   // set scissor
-
   glScissor(window->getOriginX(), window->getOriginY() + window->getHeight() - window->getViewHeight(),
       window->getWidth(), window->getViewHeight());
 
   if (useDepthComplexityOn) {
+    if (BZDBCache::stencilShadows) {
+      BZDB.set("stencilShadows", "0");
+    }
     glEnable(GL_STENCIL_TEST);
     if (!mirror || (clearZbuffer)) {
       glClear(GL_STENCIL_BUFFER_BIT);
@@ -854,7 +963,16 @@ void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
   if (background) {
     background->setBlank(blank);
     background->setInvert(invert);
-    background->renderSky(*this, fullWindow, mirror);
+
+    const bool avoidSkyFog = (mapFog && BZDB.isTrue("_fogNoSky"));
+    if (avoidSkyFog) {
+      glDisable(GL_FOG);
+      background->renderSky(*this, fullWindow, mirror);
+      glEnable(GL_FOG);
+    } else {
+      background->renderSky(*this, fullWindow, mirror);
+    }
+
     if (drawGround) {
       background->renderGround(*this, fullWindow);
     }
@@ -864,7 +982,7 @@ void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
   // we may need to turn them on when drawing the background.
   if (BZDBCache::lighting) {
     for (i = 0; i < dynamicLights; i++) {
-      lights[i]->execute(i + reservedLights);
+      lights[i]->execute(i + reservedLights, lightLists);
     }
   }
 
@@ -942,15 +1060,6 @@ void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
   }
   glPopMatrix();
 
-  // finalize dimming
-  if (reallyUseFogHack) {
-    if ((teleporterProximity > 0.0f) || useDimming) {
-      glDisable(GL_FOG);
-    }
-  } else {
-    renderPostDimming();
-  }
-
   // do depth complexity
   if (useDepthComplexityOn) {
     renderDepthComplexity();
@@ -1005,8 +1114,9 @@ void SceneRenderer::renderPreDimming()
     glFogf(GL_FOG_END, (1.0f - density) * 1000.0f * worldSize);
     glFogfv(GL_FOG_COLOR, dimnessColor);
     glEnable(GL_FOG);
+    glHint(GL_FOG_HINT, GL_FASTEST);
   }
-  else if (teleporterProximity > 0.0f && useFogHack) {
+  else if (teleporterProximity > 0.0f) {
     const float density = (teleporterProximity > 0.75f) ? 1.0f
 			  : (teleporterProximity / 0.75f);
     glFogi(GL_FOG_MODE, GL_LINEAR);
@@ -1014,11 +1124,65 @@ void SceneRenderer::renderPreDimming()
     glFogf(GL_FOG_END, (1.0f - density) * 1000.0f * worldSize);
     glFogfv(GL_FOG_COLOR, blindnessColor);
     glEnable(GL_FOG);
+    glHint(GL_FOG_HINT, GL_FASTEST);
   }
+
   return;
 }
 
 
+static bool setupMapFog()
+{
+  std::string fogModeStr;
+  if ((BZDB.get(StateDatabase::BZDB_FOGMODE) == "none") ||
+      !BZDB.isTrue("fogEffect")) {
+    glDisable(GL_FOG);
+    glHint(GL_FOG_HINT, GL_FASTEST);
+    return false;
+  }
+
+  GLenum fogMode = GL_EXP;
+  GLfloat fogDensity = 0.001f;
+  GLfloat fogStart = 0.5f * BZDBCache::worldSize;
+  GLfloat fogEnd = BZDBCache::worldSize;
+  GLfloat fogColor[4] = {0.25f, 0.25f, 0.25f, 0.25f};
+
+  // parse the values;
+  const std::string modeStr = BZDB.get("_fogMode");
+  if (modeStr == "linear") {
+    fogMode = GL_LINEAR;
+  } else if (modeStr == "exp") {
+    fogMode = GL_EXP;
+  } else if (modeStr == "exp2") {
+    fogMode = GL_EXP2;
+  } else {
+    fogMode = GL_EXP;
+  }
+  fogDensity = BZDB.eval(StateDatabase::BZDB_FOGDENSITY);
+  fogStart = BZDB.eval(StateDatabase::BZDB_FOGSTART);
+  fogEnd = BZDB.eval(StateDatabase::BZDB_FOGEND);
+  if (!parseColorString(BZDB.get(StateDatabase::BZDB_FOGCOLOR), fogColor)) {
+    fogColor[0] = fogColor[1] = fogColor[2] = 0.1f;
+    fogColor[3] = 0.0f; // has no effect
+  }
+  if (BZDB.evalInt("fogEffect") >= 2) {
+    glHint(GL_FOG_HINT, GL_NICEST);
+  } else {
+    glHint(GL_FOG_HINT, GL_FASTEST);
+  }
+
+  // setup GL fog
+  glFogi(GL_FOG_MODE, fogMode);
+  glFogf(GL_FOG_DENSITY, fogDensity);
+  glFogf(GL_FOG_START, fogStart);
+  glFogf(GL_FOG_END, fogEnd);
+  glFogfv(GL_FOG_COLOR, fogColor);
+  glEnable(GL_FOG);
+
+  return true;
+}
+
+
 void SceneRenderer::renderPostDimming()
 {
   float density = 0.0f;
@@ -1210,6 +1374,22 @@ const GLfloat* SceneRenderer::getSunDirection() const
 }
 
 
+const Extents* SceneRenderer::getVisualExtents() const
+{
+  if (scene) {
+    return scene->getVisualExtents();
+  } else {
+    return NULL;
+  }
+}
+
+
+int SceneRenderer::getFrameTriangleCount() const
+{
+  return triangleCount;
+}
+
+
 // Local Variables: ***
 // mode: C++ ***
 // tab-width: 8 ***
