@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
@@ -17,8 +17,11 @@
 #include <sys/types.h>
 #include <vector>
 #include <string>
+#include <string.h>
+#include <errno.h>
 #ifndef _WIN32
 #  include <unistd.h>
+#  include <signal.h>
 #  include <dirent.h>
 #  include <pwd.h>
 #  ifndef GUSI_20
@@ -37,19 +40,17 @@
 
 /* common implementation headers */
 #include "DirectoryNames.h"
+#include "StateDatabase.h"
 #include "FontManager.h"
-#include "StartupInfo.h"
 #include "TextUtils.h"
 
 /* local implementation headers */
 #include "MenuDefaultKey.h"
 #include "MainMenu.h"
-#include "HUDuiControl.h"
-#include "HUDuiLabel.h"
 #include "HUDuiList.h"
-
-/* from playing.h */
-StartupInfo* getStartupInfo();
+#include "playing.h"
+#include "bzflag.h"
+#include "HUDui.h"
 
 char ServerStartMenu::settings[] = "bfaaaaabaaaaa";
 
@@ -57,38 +58,38 @@ ServerStartMenu::ServerStartMenu()
 {
   // add controls
   std::vector<HUDuiControl*>& controls = getControls();
-  HUDuiList* list;
+  HUDuiList* listHUD;
   std::vector<std::string>* items;
 
   // 0
   controls.push_back(createLabel("Start Server"));
 
   // 1
-  list = createList("Style:");
-  items = &list->getList();
+  listHUD = createList("Style:");
+  items = &listHUD->getList();
   items->push_back("Capture the Flag");
   items->push_back("Free for All");
   items->push_back("Rabbit Hunt (Random Selection)");
   items->push_back("Rabbit Hunt (Score-based Selection)");
   items->push_back("Rabbit Hunt (Killer Selection)");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 2
-  list = createList("Max Players:");
-  items = &list->getList();
+  listHUD = createList("Max Players:");
+  items = &listHUD->getList();
   items->push_back("2");
   items->push_back("3");
   items->push_back("4");
   items->push_back("8");
   items->push_back("20");
   items->push_back("40");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 3
-  list = createList("Max Shots:");
-  items = &list->getList();
+  listHUD = createList("Max Shots:");
+  items = &listHUD->getList();
   items->push_back("1");
   items->push_back("2");
   items->push_back("3");
@@ -98,92 +99,92 @@ ServerStartMenu::ServerStartMenu()
   items->push_back("10");
   items->push_back("15");
   items->push_back("20");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 4
-  list = createList("Teleporters:");
-  items = &list->getList();
+  listHUD = createList("Teleporters:");
+  items = &listHUD->getList();
   items->push_back("no");
   items->push_back("yes");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 5
-  list = createList("Ricochet:");
-  items = &list->getList();
+  listHUD = createList("Ricochet:");
+  items = &listHUD->getList();
   items->push_back("no");
   items->push_back("yes");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 6
-  list = createList("Jumping:");
-  items = &list->getList();
+  listHUD = createList("Jumping:");
+  items = &listHUD->getList();
   items->push_back("no");
   items->push_back("yes");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 7
-  list = createList("Handicap:");
-  items = &list->getList();
+  listHUD = createList("Handicap:");
+  items = &listHUD->getList();
   items->push_back("no");
   items->push_back("yes");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 8
-  list = createList("Superflags:");
-  items = &list->getList();
+  listHUD = createList("Superflags:");
+  items = &listHUD->getList();
   items->push_back("no");
   items->push_back("good flags only");
   items->push_back("all flags");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 9
-  list = createList("Max Superflags:");
-  items = &list->getList();
+  listHUD = createList("Max Superflags:");
+  items = &listHUD->getList();
   items->push_back("10");
   items->push_back("20");
   items->push_back("30");
   items->push_back("40");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 10
-  list = createList("Bad Flag Antidote:");
-  items = &list->getList();
+  listHUD = createList("Bad Flag Antidote:");
+  items = &listHUD->getList();
   items->push_back("no");
   items->push_back("yes");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 11
-  list = createList("Bad Flag Time Limit:");
-  items = &list->getList();
+  listHUD = createList("Bad Flag Time Limit:");
+  items = &listHUD->getList();
   items->push_back("no limit");
   items->push_back("15 seconds");
   items->push_back("30 seconds");
   items->push_back("60 seconds");
   items->push_back("180 seconds");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 12
-  list = createList("Bad Flag Win Limit:");
-  items = &list->getList();
+  listHUD = createList("Bad Flag Win Limit:");
+  items = &listHUD->getList();
   items->push_back("no limit");
   items->push_back("drop after 1 win");
   items->push_back("drop after 2 wins");
   items->push_back("drop after 3 wins");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 13
-  list = createList("Game Over:");
-  items = &list->getList();
+  listHUD = createList("Game Over:");
+  items = &listHUD->getList();
   items->push_back("never");
   items->push_back("after 5 minutes");
   items->push_back("after 15 minutes");
@@ -196,20 +197,20 @@ ServerStartMenu::ServerStartMenu()
   items->push_back("when a team gets +10");
   items->push_back("when a team gets +25");
   items->push_back("when a team gets +100");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 14
-  list = createList("Server Reset:");
-  items = &list->getList();
+  listHUD = createList("Server Reset:");
+  items = &listHUD->getList();
   items->push_back("no, quit after game");
   items->push_back("yes, reset for more games");
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 15
-  list = createList("World Map:");
-  items = &list->getList();
+  listHUD = createList("World Map:");
+  items = &listHUD->getList();
   items->push_back("random map");
 
   // add a list of .bzw files found in the world file dir
@@ -244,8 +245,8 @@ ServerStartMenu::ServerStartMenu()
 #endif // _WIN32
   scanWorldFiles (searchDir, items);
 
-  list->update();
-  controls.push_back(list);
+  listHUD->update();
+  controls.push_back(listHUD);
 
   // 16
   start = createLabel("Start");
@@ -356,9 +357,9 @@ void ServerStartMenu::execute()
   static const char*	serverApp = "bzfs";
   bool success = false;
 
-  std::vector<HUDuiControl*>& list = getControls();
-  HUDuiControl* focus = HUDui::getFocus();
-  if (focus == start) {
+  std::vector<HUDuiControl*>& listHUD = getControls();
+  HUDuiControl* _focus = HUDui::getFocus();
+  if (_focus == start) {
     // start it up:
     //   without: -p, -i, -q, -a, +f, -synctime
     //   -b if -c
@@ -369,7 +370,6 @@ void ServerStartMenu::execute()
     // other options as they were set
 
     // get path to server from path to client
-    extern const char* argv0;			// from bzflag.cxx
     // add 256 for flags room
     char serverCmd[PATH_MAX + 256];
     strcpy(serverCmd, argv0);
@@ -388,60 +388,64 @@ void ServerStartMenu::execute()
     int arg = 0;
     args[arg++] = serverApp;
 
+#if defined(_WIN32)
     // always try a fallback port if default port is busy
+    // (unix-like OSes will instead try to kill off the old process)
     args[arg++] = "-pf";
+#endif
 
     // load the world map first, so that later arguments can
     // override any that are present in a map's "options" block
-    if (((HUDuiList*)list[15])->getIndex() != 0) { // not random
+    if (((HUDuiList*)listHUD[15])->getIndex() != 0) { // not random
       args[arg++] = "-world";
-      std::vector<std::string> fileList = ((HUDuiList*)list[15])->getList();
-      std::string filename = fileList[((HUDuiList*)list[15])->getIndex()].c_str();
+      std::vector<std::string> fileList = ((HUDuiList*)listHUD[15])->getList();
+      std::string filename
+	= fileList[((HUDuiList*)listHUD[15])->getIndex()].c_str();
       args[arg++] = worldFiles[filename].c_str();
     }
 
     // game style
-    if (((HUDuiList*)list[1])->getIndex() == 0) {
+    if (((HUDuiList*)listHUD[1])->getIndex() == 0) {
       args[arg++] = "-c";
       args[arg++] = "-b";
     }
-    else if (((HUDuiList*)list[1])->getIndex() == 1){
+    else if (((HUDuiList*)listHUD[1])->getIndex() == 1){
       args[arg++] = "-h";
     }
     else {
 	static const char* rabbitStyles[] = { "random", "score", "killer" };
 	args[arg++] = "-rabbit";
-	args[arg++] = rabbitStyles[(((HUDuiList*)list[1])->getIndex()) - 2];
+	args[arg++] = rabbitStyles[(((HUDuiList*)listHUD[1])->getIndex()) - 2];
     }
 
     // max players
     static const char* numPlayers[] = { "2", "3", "4", "8", "20", "40" };
     args[arg++] = "-mp";
-    args[arg++] = numPlayers[((HUDuiList*)list[2])->getIndex()];
+    args[arg++] = numPlayers[((HUDuiList*)listHUD[2])->getIndex()];
 
     // max shots
     static const char* numShots[] = { "1", "2", "3", "4", "5", "8", "10", "15", "20" };
     args[arg++] = "-ms";
-    args[arg++] = numShots[((HUDuiList*)list[3])->getIndex()];
+    args[arg++] = numShots[((HUDuiList*)listHUD[3])->getIndex()];
 
     // teleporters
-    if (((HUDuiList*)list[4])->getIndex() == 1)
+    if (((HUDuiList*)listHUD[4])->getIndex() == 1)
       args[arg++] = "-t";
 
     // ricochet
-    if (((HUDuiList*)list[5])->getIndex() == 1)
+    if (((HUDuiList*)listHUD[5])->getIndex() == 1)
       args[arg++] = "+r";
 
     // jumping
-    if (((HUDuiList*)list[6])->getIndex() == 1)
+    if (((HUDuiList*)listHUD[6])->getIndex() == 1)
       args[arg++] = "-j";
 
     // handicap
-    if (((HUDuiList*)list[7])->getIndex() == 1)
+    if (((HUDuiList*)listHUD[7])->getIndex() == 1)
       args[arg++] = "-handicap";
 
     // superflags
-    const int superflagOption = ((HUDuiList*)list[8])->getIndex();
+    const int superflagOption = ((HUDuiList*)listHUD[8])->getIndex();
     if (superflagOption != 0) {
       if (superflagOption == 1) {
 	args[arg++] = "-f";
@@ -451,21 +455,21 @@ void ServerStartMenu::execute()
       args[arg++] = "+s";
 
       // max superflags
-      static const char* numFlags[] = { "10", "20", "30", "40" };
-      args[arg++] = numFlags[((HUDuiList*)list[9])->getIndex()];
+      static const char* numFlagsList[] = { "10", "20", "30", "40" };
+      args[arg++] = numFlagsList[((HUDuiList*)listHUD[9])->getIndex()];
     }
 
     // shaking
     static const char* shakingTime[] = { "", "15", "30", "60", "180" };
     static const char* shakingWins[] = { "", "1", "2", "3" };
-    if (((HUDuiList*)list[10])->getIndex() == 1) args[arg++] = "-sa";
-    if (((HUDuiList*)list[11])->getIndex() != 0) {
+    if (((HUDuiList*)listHUD[10])->getIndex() == 1) args[arg++] = "-sa";
+    if (((HUDuiList*)listHUD[11])->getIndex() != 0) {
       args[arg++] = "-st";
-      args[arg++] = shakingTime[((HUDuiList*)list[11])->getIndex()];
+      args[arg++] = shakingTime[((HUDuiList*)listHUD[11])->getIndex()];
     }
-    if (((HUDuiList*)list[12])->getIndex() != 0) {
+    if (((HUDuiList*)listHUD[12])->getIndex() != 0) {
       args[arg++] = "-sw";
-      args[arg++] = shakingWins[((HUDuiList*)list[12])->getIndex()];
+      args[arg++] = shakingWins[((HUDuiList*)listHUD[12])->getIndex()];
     }
 
     // game over
@@ -477,13 +481,13 @@ void ServerStartMenu::execute()
 					   "300", "900", "3600", "10800",
 					   "3", "10", "25",
 					   "3", "10", "25", "100" };
-    if (((HUDuiList*)list[13])->getIndex() != 0) {
-      args[arg++] = gameOverArg[((HUDuiList*)list[13])->getIndex()];
-      args[arg++] = gameOverValue[((HUDuiList*)list[13])->getIndex()];
+    if (((HUDuiList*)listHUD[13])->getIndex() != 0) {
+      args[arg++] = gameOverArg[((HUDuiList*)listHUD[13])->getIndex()];
+      args[arg++] = gameOverValue[((HUDuiList*)listHUD[13])->getIndex()];
     }
 
     // server reset
-    if (((HUDuiList*)list[14])->getIndex() == 0)
+    if (((HUDuiList*)listHUD[14])->getIndex() == 0)
       args[arg++] = "-g";
 
     // no more arguments
@@ -513,8 +517,31 @@ void ServerStartMenu::execute()
 #else /* defined(_WIN32) */
 
     // UNIX
-    pid_t pid = fork();
-    if (pid == -1) setStatus("Failed... cannot fork.");
+    static pid_t pid = -1;
+
+    // try to kill off the old process
+    if (pid != -1) {
+      int i;
+      for (i = 0; i < 12; i++) {
+        if (kill(pid, SIGTERM) == 0) {
+          TimeKeeper::sleep(0.25); // be gracious for max 3 seconds
+        } else {
+          if (errno == ESRCH) {
+            break; // the pid doesn't exist
+          }
+        }
+      }
+      if (i == 12) {
+        kill(pid, SIGKILL); // be brutal
+      }
+    }
+
+    // fork
+    pid = fork();
+    
+    if (pid == -1) {
+      setStatus("Failed... cannot fork.");
+    }
     else if (pid == 0) {
       // child process.  close down stdio.
       close(0);
@@ -531,14 +558,23 @@ void ServerStartMenu::execute()
     }
     else if (pid != 0) {
       // parent process.  wait a bit and check if child died.
-      TimeKeeper::sleep(1.0f);
-      if (waitpid(pid, NULL, WNOHANG) != 0) {
-	setStatus("Failed.");
+      TimeKeeper::sleep(1.0);
+      int pStatus;
+      if (waitpid(pid, &pStatus, WNOHANG) != 0) {
+	pid = -1;
+	char failBuf[64];
+        if (WIFEXITED(pStatus)) {
+          snprintf(failBuf, 64, "Failed (exit = %i).", WEXITSTATUS(pStatus));
+        } else if (WIFSIGNALED(pStatus)) {
+          snprintf(failBuf, 64, "Failed (signal = %i).", WTERMSIG(pStatus));
+        } else {
+          strcpy(failBuf, "Failed.");
+        }
+	setStatus(failBuf);
       } else {
 	setStatus("Server started.");
 	success = true;
       }
-
     }
 
 #endif /* defined(_WIN32) */
@@ -556,43 +592,45 @@ void ServerStartMenu::setStatus(const char* msg, const std::vector<std::string>
 {
   status->setString(msg, parms);
   FontManager &fm = FontManager::instance();
-  const float width = fm.getStrLength(status->getFontFace(), status->getFontSize(), status->getString());
-  status->setPosition(center - 0.5f * width, status->getY());
+  const float widt = fm.getStrLength(status->getFontFace(),
+				     status->getFontSize(),
+				     status->getString());
+  status->setPosition(center - 0.5f * widt, status->getY());
 }
 
-void ServerStartMenu::resize(int width, int height)
+void ServerStartMenu::resize(int _width, int _height)
 {
-  HUDDialog::resize(width, height);
-  center = 0.5f * (float)width;
+  HUDDialog::resize(_width, _height);
+  center = 0.5f * (float)_width;
 
   // use a big font for title, smaller font for the rest
-  const float titleFontSize = (float)height / 15.0f;
-  const float fontSize = (float)height / 54.0f;
+  const float titleFontSize = (float)_height / 15.0f;
+  const float fontSize = (float)_height / 54.0f;
   FontManager &fm = FontManager::instance();
 
   // reposition title
-  std::vector<HUDuiControl*>& list = getControls();
-  HUDuiLabel* title = (HUDuiLabel*)list[0];
+  std::vector<HUDuiControl*>& listHUD = getControls();
+  HUDuiLabel* title = (HUDuiLabel*)listHUD[0];
   title->setFontSize(titleFontSize);
   const float titleWidth = fm.getStrLength(title->getFontFace(), titleFontSize, title->getString());
   const float titleHeight = fm.getStrHeight(title->getFontFace(), titleFontSize, " ");
-  float x = 0.5f * ((float)width - titleWidth);
-  float y = (float)height - titleHeight;
+  float x = 0.5f * ((float)_width - titleWidth);
+  float y = (float)_height - titleHeight;
   title->setPosition(x, y);
 
   // reposition options
-  x = 0.5f * (float)width;
+  x = 0.5f * (float)_width;
   y -= 0.6f * titleHeight;
-  const float h = fm.getStrHeight(list[1]->getFontFace(), fontSize, " ");
-  const int count = list.size();
+  const float h = fm.getStrHeight(listHUD[1]->getFontFace(), fontSize, " ");
+  const int count = listHUD.size();
   for (int i = 1; i < count; i++) {
-    if (list[i] == start) {
+    if (listHUD[i] == start) {
       y -= 1.5f * h;
-      list[i]->setFontSize(1.5f * fontSize);
-      list[i]->setPosition(x, y);
+      listHUD[i]->setFontSize(1.5f * fontSize);
+      listHUD[i]->setPosition(x, y);
     } else {
-      list[i]->setFontSize(fontSize);
-      list[i]->setPosition(x, y);
+      listHUD[i]->setFontSize(fontSize);
+      listHUD[i]->setPosition(x, y);
     }
     y -= 1.0f * h;
   }
@@ -600,10 +638,10 @@ void ServerStartMenu::resize(int width, int height)
 
 HUDuiList* ServerStartMenu::createList(const char* str)
 {
-  HUDuiList* list = new HUDuiList;
-  list->setFontFace(MainMenu::getFontFace());
-  if (str) list->setLabel(str);
-  return list;
+  HUDuiList* listHUD = new HUDuiList;
+  listHUD->setFontFace(MainMenu::getFontFace());
+  if (str) listHUD->setLabel(str);
+  return listHUD;
 }
 
 HUDuiLabel* ServerStartMenu::createLabel(const char* str)
