@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,31 +7,19 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
 #include "ShotStrategy.h"
 
-/* system implementation headers */
-#include <vector>
-
 /* common implementation headers */
-#include "StateDatabase.h"
 #include "CollisionManager.h"
-#include "Intersect.h"
 #include "Obstacle.h"
+#include "ObstacleList.h"
 #include "WallObstacle.h"
-#include "BoxBuilding.h"
 #include "ObstacleMgr.h"
 
-/* local implementation headers */
-#include "World.h"
-
-
-/* local implementation headers */
-
-
 ShotStrategy::ShotStrategy(ShotPath* _path) :
   path(_path)
 {
