@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,14 +7,21 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
-#include "common.h"
-#include <math.h>
+
+// interface header
 #include "TargetingUtils.h"
-#include "ShotStrategy.h"
+
+// system headers
+#include <math.h>
+
+// common headers
 #include "Ray.h"
 
+// local headers
+#include "ShotStrategy.h"
+
 // These routines are 2 dimensional
 
 float TargetingUtils::normalizeAngle(float ang)
@@ -90,8 +97,8 @@ float TargetingUtils::getTargetAngleDifference( const float *src, float srcAzimu
 
   getUnitVector(src, target, targetUnitVector);
 
-  srcUnitVector[0] = cos(srcAzimuth);
-  srcUnitVector[1] = sin(srcAzimuth);
+  srcUnitVector[0] = cosf(srcAzimuth);
+  srcUnitVector[1] = sinf(srcAzimuth);
   srcUnitVector[2] = 0.0f;
 
   return acos( targetUnitVector[0]*srcUnitVector[0] + targetUnitVector[1]*srcUnitVector[1] );
