@@ -1,5 +1,5 @@
 /* bzflag
-* Copyright (c) 1993 - 2005 Tim Riker
+* Copyright (c) 1993 - 2006 Tim Riker
 *
 * This package is free software;  you can redistribute it and/or
 * modify it under the terms of the license found in the file
@@ -7,43 +7,32 @@
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
-* WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
 
-// bzflag global common header
-#include "common.h"
-#include "global.h"
-
 // interface header
 #include "WeatherRenderer.h"
 
-// system headers
-#include <string>
-#include <vector>
-
 // common impl headers
-#include "bzfgl.h"
-#include "OpenGLGState.h"
-#include "OpenGLMaterial.h"
 #include "TextureManager.h"
 #include "StateDatabase.h"
 #include "BZDBCache.h"
 #include "TimeKeeper.h"
 #include "TextUtils.h"
 #include "ParseColor.h"
+#include "Intersect.h"
+#include "Extents.h"
 
 // local impl headers
-#include "SceneRenderer.h"
-#include "Intersect.h"
 #include "RoofTops.h"
 
 // for debug
 #define _CULLING_RAIN false
 
 
-void bzdbCallBack(const std::string& /* name */ , void *userData)
+static void bzdbCallBack(const std::string& /* name */ , void *userData)
 {
-  ((WeatherRenderer*)userData)->set();
+  static_cast<WeatherRenderer*>(userData)->set();
 }
 
 
@@ -398,7 +387,7 @@ void WeatherRenderer::set(void)
 	  raindrops.push_back (drop);
 	}
       }
-      lastRainTime = TimeKeeper::getCurrent().getSeconds();
+      lastRainTime = float(TimeKeeper::getCurrent().getSeconds());
     }
     // recompute the drops based on the posible new size
     buildDropList();
@@ -425,8 +414,8 @@ void WeatherRenderer::set(void)
 void WeatherRenderer::update(void)
 {
   // update the time
-  float frameTime = TimeKeeper::getCurrent().getSeconds() - lastRainTime;
-  lastRainTime = TimeKeeper::getCurrent().getSeconds();
+  float frameTime = float(TimeKeeper::getCurrent().getSeconds() - lastRainTime);
+  lastRainTime = float(TimeKeeper::getCurrent().getSeconds());
 
   std::vector<rain> dropsToAdd;
 
@@ -579,9 +568,9 @@ void WeatherRenderer::rebuildContext(void)
 }
 
 
-void WeatherRenderer::buildDropList(bool draw)
+void WeatherRenderer::buildDropList(bool _draw)
 {
-  if (!draw) {
+  if (!_draw) {
     if (dropList != INVALID_GL_LIST_ID) {
       glDeleteLists(dropList, 1);
       dropList = INVALID_GL_LIST_ID;
@@ -654,16 +643,16 @@ void WeatherRenderer::buildDropList(bool draw)
     glPopMatrix();
   }
 
-  if (!draw) {
+  if (!_draw) {
     glEndList();
   }
 }
 
 
-void WeatherRenderer::buildPuddleList(bool draw)
+void WeatherRenderer::buildPuddleList(bool _draw)
 {
   float scale = 1;
-  if (!draw) {
+  if (!_draw) {
     if (puddleList != INVALID_GL_LIST_ID) {
       glDeleteLists(puddleList, 1);
       puddleList = INVALID_GL_LIST_ID;
@@ -686,7 +675,7 @@ void WeatherRenderer::buildPuddleList(bool draw)
   glVertex3f(-scale, scale, 0);
   glEnd();
 
-  if (!draw) {
+  if (!_draw) {
     glEndList();
   }
 }
@@ -735,8 +724,16 @@ bool WeatherRenderer::updateDrop(std::vector<rain>::iterator& drop,
 	if (cullRoofTops) {
 	  drop->roofTop =
 	    RoofTops::getTopHeight (drop->pos[0], drop->pos[1], drop->pos[2]);
+	  // clamp the rain to the valid rain range.
+	  if ( rainSpeed > 0)
+	  {
+		  if ( drop->roofTop < rainEndZ )
+			  drop->roofTop = rainEndZ;
+		  else if ( drop->roofTop > rainEndZ )
+			  drop->roofTop = rainEndZ;
+	  }
 	} else {
-	  drop->roofTop = 0.0f;
+	  drop->roofTop = rainEndZ;
 	}
 
 	return true;
@@ -757,8 +754,16 @@ bool WeatherRenderer::updateDrop(std::vector<rain>::iterator& drop,
 	if (cullRoofTops) {
 	  newDrop.roofTop =
 	    RoofTops::getTopHeight (newDrop.pos[0], newDrop.pos[1], newDrop.pos[2]);
+	  // clamp the rain to the valid rain range.
+	  if ( rainSpeed > 0)
+	  {
+		  if ( newDrop.roofTop < rainEndZ )
+			newDrop.roofTop = rainEndZ;
+		  else if ( newDrop.roofTop > rainEndZ )
+			  newDrop.roofTop = rainEndZ;
+	  }
 	} else {
-	  newDrop.roofTop = 0.0f;
+	  newDrop.roofTop = rainEndZ;
 	}
 
 	toAdd.push_back(newDrop);
