@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,21 +7,18 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
 #include "commands.h"
 
 /* system implementation headers */
-#include <fstream>
-#include <string>
 #include "../zlib/zconf.h"
 #include "../zlib/zlib.h"
 
 /* common implementation headers */
-#include "global.h"
-#include "StateDatabase.h"
+#include "BZDBCache.h"
 #include "TextUtils.h"
 #include "FileManager.h"
 #include "DirectoryNames.h"
@@ -31,55 +28,145 @@
 #include "LocalPlayer.h"
 #include "sound.h"
 #include "ComposeDefaultKey.h"
-#include "Roster.h"
 #include "SilenceDefaultKey.h"
 #include "ServerCommandKey.h"
-
-/* FIXME -- from playing.cxx */
-extern LocalPlayer* myTank;
-extern bool fireButton;
-#include "ServerLink.h"
-extern ServerLink* serverLink;
-void handleFlagDropped(Player* tank);
-void setTarget();
-extern bool gameOver;
-extern float destructCountdown;
+#include "Roaming.h"
+#include "playing.h"
 #include "HUDRenderer.h"
-extern HUDRenderer* hud;
-extern bool pausedByUnmap;
-#include "ControlPanel.h"
-extern ControlPanel* controlPanel;
-extern int savedVolume;
-bool shouldGrabMouse();
-#include "MainWindow.h"
-extern MainWindow* mainWindow;
-extern float pauseCountdown;
-#include "Address.h"
-extern char messageMessage[PlayerIdPLen + MessageLen];
-extern float clockAdjust;
-extern bool roaming;
-extern enum roamingView {
-  roamViewFree = 0,
-  roamViewTrack,
-  roamViewFollow,
-  roamViewFP,
-  roamViewFlag,
-  roamViewCount
-} roamView;
-extern int roamTrackTank, roamTrackWinner, roamTrackFlag;
-extern float roamPos[3], roamDPos[3];
-extern float roamTheta, roamDTheta;
-extern float roamPhi, roamDPhi;
-extern float roamZoom, roamDZoom;
-extern bool roamButton;
-#include "World.h"
-extern World* world;
-void setRoamingLabel();
-extern void warnAboutMainFlags();
-extern void warnAboutRadarFlags();
-extern void warnAboutRadar();
-extern void warnAboutConsole();
+#include "HUDui.h"
 
+/** jump
+ */
+static std::string cmdJump(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** fire weapon
+ */
+static std::string cmdFire(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** drop a flag
+ */
+static std::string cmdDrop(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** toggle radar visibility
+ */
+static std::string cmdToggleRadar(const std::string&,
+				  const CommandManager::ArgList& args, bool*);
+
+/** toggle console visibility
+ */
+static std::string cmdToggleConsole(const std::string&,
+				    const CommandManager::ArgList& args, bool*);
+
+/** toggle flags
+ */
+static std::string cmdToggleFlags(const std::string&,
+				  const CommandManager::ArgList& args, bool*);
+
+/** identify to a server
+ */
+static std::string cmdIdentify(const std::string&,
+			       const CommandManager::ArgList& args, bool*);
+
+/** restart/respawn
+ */
+static std::string cmdRestart(const std::string&,
+			      const CommandManager::ArgList& args, bool*);
+
+/** self-destruct
+ */
+static std::string cmdDestruct(const std::string&,
+			       const CommandManager::ArgList& args, bool*);
+
+/** pause
+ */
+static std::string cmdPause(const std::string&,
+			    const CommandManager::ArgList& args, bool*);
+
+/** select tabbed message
+ */
+static std::string cmdMessagePanel(const std::string&,
+				   const CommandManager::ArgList& args, bool*);
+
+/** toggle auto-pilot
+ */
+static std::string cmdAutoPilot(const std::string&,
+				const CommandManager::ArgList& args, bool*);
+
+/** change radar scale
+ */
+static std::string cmdRadarZoom(const std::string&,
+				const CommandManager::ArgList& args, bool*);
+
+/** change view angle
+ */
+static std::string cmdViewZoom(const std::string&,
+			       const CommandManager::ArgList& args, bool*);
+
+/** send
+ */
+static std::string cmdSend(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+#ifdef SNAPPING
+/** capture a screenshot
+ */
+static std::string cmdScreenshot(const std::string&,
+				 const CommandManager::ArgList& args, bool*);
+#endif
+
+/** time
+ */
+static std::string cmdTime(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** roam
+ */
+static std::string cmdRoam(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** silence another player
+ */
+static std::string cmdSilence(const std::string&,
+			      const CommandManager::ArgList& args, bool*);
+
+/** perform a server command
+ */
+static std::string cmdServerCommand(const std::string&,
+				    const CommandManager::ArgList& args,
+				    bool*);
+
+/** scroll the chat panel
+ */
+static std::string cmdScrollPanel(const std::string&,
+				  const CommandManager::ArgList& args, bool*);
+
+/** hunt a player
+ */
+static std::string cmdHunt(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** hunt another player
+ */
+static std::string cmdAddHunt(const std::string&,
+			      const CommandManager::ArgList& args, bool*);
+
+/** iconify window
+ */
+static std::string cmdIconify(const std::string&,
+			      const CommandManager::ArgList& args, bool*);
+
+/** toggle mouse capture
+ */
+static std::string cmdMouseGrab(const std::string&,
+				const CommandManager::ArgList& args, bool*);
+
+/** toggle Full Screen
+ */
+static std::string cmdToggleFS(const std::string&,
+			       const CommandManager::ArgList& args, bool*);
 
 const struct CommandListItem commandList[] = {
   { "fire",	&cmdFire,	"fire:  fire a shot" },
@@ -99,7 +186,9 @@ const struct CommandListItem commandList[] = {
   { "servercommand",	&cmdServerCommand,	"servercommand:  quick admin" },
   { "scrollpanel",	&cmdScrollPanel,	"scrollpanel {up|down}:  scroll message panel" },
   { "hunt",	&cmdHunt,	"hunt:  hunt a specific player" },
+  { "addhunt",	&cmdAddHunt,	"addhunt:  add/modify hunted player(s)" },
   { "iconify",  &cmdIconify,	"iconify: iconify & pause bzflag" },
+  { "mousegrab", &cmdMouseGrab, "mousegrab: toggle exclusive mouse mode" },
   { "fullscreen", &cmdToggleFS, "fullscreen: toggle fullscreen mode" },
   { "autopilot",&cmdAutoPilot,	"autopilot:  set/unset autopilot bot code" },
   { "radarZoom", &cmdRadarZoom, "radarZoom {in/out}: change maxRadar range"},
@@ -112,8 +201,8 @@ const struct CommandListItem commandList[] = {
 };
 
 
-std::string cmdToggleFS(const std::string&,
-			const CommandManager::ArgList& args)
+static std::string cmdToggleFS(const std::string&,
+			       const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: fullscreen";
@@ -122,7 +211,19 @@ std::string cmdToggleFS(const std::string&,
   return std::string();
 }
 
-std::string cmdIconify(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdMouseGrab(const std::string&,
+				const CommandManager::ArgList& args, bool*)
+{
+  if (args.size() != 0)
+    return "usage: mousegrab";
+  const bool grabbing = !(BZDB.isTrue("mousegrab"));
+  BZDB.set("mousegrab", grabbing ? "true" : "false");
+  mainWindow->enableGrabMouse(grabbing);
+  return std::string();
+}
+
+static std::string cmdIconify(const std::string&,
+			      const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: iconify";
@@ -130,17 +231,19 @@ std::string cmdIconify(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdJump(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdJump(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: jump";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank != NULL)
     myTank->setJump();
   return std::string();
 }
 
-std::string cmdToggleFlags (const std::string&, const
-CommandManager::ArgList& args)
+static std::string cmdToggleFlags(const std::string&, const
+				  CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 1)
     return "usage: main|radar";
@@ -157,8 +260,8 @@ CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdToggleRadar (const std::string&, const
-CommandManager::ArgList& args)
+static std::string cmdToggleRadar(const std::string&,
+				  const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0) {
     return "usage: toggleRadar";
@@ -171,8 +274,8 @@ CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdToggleConsole (const std::string&, const
-CommandManager::ArgList& args)
+static std::string cmdToggleConsole(const std::string&,
+				    const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0) {
     return "usage: toggleConsole";
@@ -186,20 +289,24 @@ CommandManager::ArgList& args)
 }
 
 
-std::string cmdFire(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdFire(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: fire";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (fireButton && myTank != NULL && myTank->isAlive()
       && myTank->getTeam() != ObserverTeam)
     myTank->fireShot();
   return std::string();
 }
 
-std::string cmdDrop(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdDrop(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: drop";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank != NULL) {
     FlagType* flag = myTank->getFlag();
     if ((flag != Flags::Null) && !myTank->isPaused() &&
@@ -215,37 +322,43 @@ std::string cmdDrop(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdIdentify(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdIdentify(const std::string&,
+			       const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: identify";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank != NULL)
     if (myTank->isAlive() && !myTank->isPaused())
       setTarget();
   return std::string();
 }
 
-std::string cmdRestart(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdRestart(const std::string&,
+			      const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: restart";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank != NULL)
     if (!gameOver && !myTank->isSpawning() && (myTank->getTeam() != ObserverTeam) && !myTank->isAlive() && !myTank->isExploding()) {
       serverLink->sendAlive();
       myTank->setSpawning(true);
-	  CommandManager::ArgList zoomArgs;
-	  std::string resetArg = "reset";
-	  zoomArgs.push_back(resetArg);
-	  cmdViewZoom("", zoomArgs);
+      CommandManager::ArgList zoomArgs;
+      std::string resetArg = "reset";
+      zoomArgs.push_back(resetArg);
+      cmdViewZoom("", zoomArgs,NULL);
     }
 
   return std::string();
 }
 
-std::string cmdDestruct(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdDestruct(const std::string&,
+			       const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: destruct";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank != NULL) {
     if (destructCountdown > 0.0f) {
       destructCountdown = 0.0f;
@@ -260,11 +373,13 @@ std::string cmdDestruct(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdPause(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdPause(const std::string&,
+			    const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: pause";
 
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (!pausedByUnmap && myTank && myTank->isAlive() && !myTank->isAutoPilot()) {
     if (myTank->isPaused()) {
       // already paused, so unpause
@@ -311,7 +426,8 @@ std::string cmdPause(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdAutoPilot(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdAutoPilot(const std::string&,
+				const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: autopilot";
@@ -321,6 +437,7 @@ std::string cmdAutoPilot(const std::string&, const CommandManager::ArgList& args
     return std::string();
   }
 
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank != NULL && myTank->getTeam() != ObserverTeam) {
     if (myTank->isAutoPilot()) {
 
@@ -355,8 +472,8 @@ std::string cmdAutoPilot(const std::string&, const CommandManager::ArgList& args
   return std::string();
 }
 
-std::string cmdRadarZoom(const std::string&,
-			 const CommandManager::ArgList& args)
+static std::string cmdRadarZoom(const std::string&,
+				const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 1)
     return "usage: radarZoom {in|out}";
@@ -380,8 +497,8 @@ std::string cmdRadarZoom(const std::string&,
   return std::string();
 }
 
-std::string cmdViewZoom(const std::string&,
-			const CommandManager::ArgList& args)
+static std::string cmdViewZoom(const std::string&,
+			       const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 1)
     return "usage: viewZoom {in|out|toggle}";
@@ -399,13 +516,21 @@ std::string cmdViewZoom(const std::string&,
       fov = 15.0f;
     BZDB.setFloat("displayFOV", fov);
   } else if (args[0] == "toggle") {
-    if (fov < 15.5f)
+    if (fov < 15.5f) {
       fov = 60.0f;
-    else
+    } else {
       fov = 15.0f;
+    }
     BZDB.setFloat("displayFOV", fov);
+    // also toggle the observer fov
+    if (ROAM.getZoom() != 60.0f) {
+      ROAM.setZoom(60.0f);
+    } else {
+      ROAM.setZoom(15.0f);
+    }
   } else if (args[0] == "reset") {
-	fov = 60.0f;
+    fov = 60.0f;
+    ROAM.setZoom(60.0f);
     BZDB.setFloat("displayFOV", fov);
   } else {
     return "usage: viewZoom {in|out|toggle|reset}";
@@ -414,8 +539,8 @@ std::string cmdViewZoom(const std::string&,
   return std::string();
 }
 
-std::string cmdMessagePanel(const std::string&,
-			    const CommandManager::ArgList& args)
+static std::string cmdMessagePanel(const std::string&,
+				   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 1)
     return "usage: messagepanel {all|chat|server|misc}";
@@ -440,11 +565,13 @@ std::string cmdMessagePanel(const std::string&,
   return std::string();
 }
 
-std::string cmdSend(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdSend(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   static ComposeDefaultKey composeKeyHandler;
   if (args.size() != 1)
     return "usage: send {all|team|nemesis|recipient|admin}";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (!myTank)
     return "use send only when connected";
   std::string composePrompt;
@@ -498,14 +625,19 @@ std::string cmdSend(const std::string&, const CommandManager::ArgList& args)
 }
 
 #ifdef SNAPPING
-std::string cmdScreenshot(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdScreenshot(const std::string&,
+				 const CommandManager::ArgList& args, bool*)
 {
-  static int snap = 0;
+  int snap = atoi(BZDB.get(std::string("lastScreenshot")).c_str());
+  snap++;
+
   if (args.size() != 0)
     return "usage: screenshot";
 
   std::string filename = getScreenShotDirName();
-  filename += TextUtils::format("bzfi%04d.png", snap++);
+  filename += TextUtils::format("bzfi%04d.png", snap);
+
+  BZDB.setInt(std::string("lastScreenshot"),snap);
 
   std::ostream* f = FILEMGR.createDataOutStream (filename.c_str(), true, true);
 
@@ -514,6 +646,14 @@ std::string cmdScreenshot(const std::string&, const CommandManager::ArgList& arg
     const unsigned long blength = h * w * 3 + h;    //size of b[] and br[]
     unsigned char* b = new unsigned char[blength];  //screen of pixels + column for filter type required by PNG - filtered data
 
+    // pause to prevent dt from accumulating until done screenshotting
+    LocalPlayer *myTank = LocalPlayer::getMyTank();
+    bool temp_paused = false;
+    if (myTank && !myTank->isPaused()) {
+      myTank->setPause(true);
+      temp_paused = true;
+    }
+
     //Prepare gamma table
     const bool gammaAdjust = BZDB.isSet("gamma");
     unsigned char gammaTable[256];
@@ -578,17 +718,23 @@ std::string cmdScreenshot(const std::string&, const CommandManager::ArgList& arg
       // apply gamma correction if necessary
       if (gammaAdjust) {
 	unsigned char *ptr = b + line + 1;
-	for (int i = 1; i < w * 3 + 1; i++) {
+	for (int j = 1; j < w * 3 + 1; j++) {
 	  *ptr = gammaTable[*ptr];
 	  ptr++;
 	}
       }
     }
 
+    // let the user know we're paused while saving the file
+    if (temp_paused) {
+      drawFrame(0.0f);
+    }
+
     unsigned long zlength = blength + 15;	    //length of bz[], will be changed by zlib to the length of the compressed string contained therein
     unsigned char* bz = new unsigned char[zlength]; //just like b, but compressed; might get bigger, so give it room
     // compress b into bz
     compress2(bz, &zlength, b, blength, 5);
+
     temp = htonl(zlength);			  //(length) IDAT length after compression
     f->write((char*) &temp, 4);
     temp = htonl(PNGTAG("IDAT"));		   //(tag) IDAT
@@ -629,12 +775,21 @@ std::string cmdScreenshot(const std::string&, const CommandManager::ArgList& arg
     char notify[128];
     snprintf(notify, 128, "%s: %dx%d", filename.c_str(), w, h);
     controlPanel->addMessage(notify);
+
+    // unpause to prevent dt from accumulating until done screenshotting
+    if (temp_paused) {
+      myTank->setPause(false);
+    }
+    // update the display regardless of pausing
+    drawFrame(0.0f);
+
   }
   return std::string();
 }
 #endif
 
-std::string cmdTime(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdTime(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   // FIXME - time should be moved into BZDB
   if (args.size() != 1)
@@ -649,11 +804,12 @@ std::string cmdTime(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdRoam(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdRoam(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() == 0)
     return "usage: roam {zoom|cycle} <args>";
-  if (!roaming)
+  if (!ROAM.isRoaming())
     return std::string();
   if (args[0] == "zoom") {
     if (args.size() != 2)
@@ -661,11 +817,11 @@ std::string cmdRoam(const std::string&, const CommandManager::ArgList& args)
     if (!roamButton || args[1] == "stop") {
       roamDZoom = 0.0f;
     } else if (args[1] == "out") {
-      roamDZoom = 50.0f;
+      roamDZoom = +2.0f * BZDBCache::tankSpeed;
     } else if (args[1] == "in") {
-      roamDZoom = -50.0f;
+      roamDZoom = -2.0f * BZDBCache::tankSpeed;
     } else if (args[1] == "normal") {
-      roamZoom = 60.0f;
+      ROAM.setZoom(60.0f);
     } else {
       return "usage: roam zoom {in|out|normal|stop}";
     }
@@ -674,91 +830,19 @@ std::string cmdRoam(const std::string&, const CommandManager::ArgList& args)
       return "usage: roam cycle {type|subject} {forward|backward}";
     if (args[1] == "type") {
       if (args[2] == "forward") {
-	roamView = roamingView((roamView + 1) % roamViewCount);
-	if (roamView == roamViewFlag) {
-	  const int maxFlags = world->getMaxFlags();
-	  bool found = false;
-	  for (int i = 0; i < maxFlags; i++) {
-	    const Flag& flag = world->getFlag(i);
-	    if (flag.type->flagTeam != NoTeam) {
-	      roamTrackFlag = i;
-	      found = true;
-	      break;
-	    }
-	  }
-	  if (!found)
-	    roamView = roamViewFree;
-	} else if ((roamTrackTank != -1) && (roamView == roamViewTrack || roamView == roamViewFollow || roamView == roamViewFP)) {
-	  if ((player[roamTrackTank] != NULL) && (!player[roamTrackTank]->isAlive())) {
-	    bool found = false;
-	    for (int i = 0; i < curMaxPlayers; i++) {
-	      if (player[i] && player[i]->isAlive()) {
-		roamTrackTank = roamTrackWinner = i;
-		found = true;
-		break;
-	      }
-	    }
-	    if (!found)
-	      roamTrackTank = -1;
-	  }
-	}
-	setRoamingLabel();
+	ROAM.setMode(Roaming::RoamingView((ROAM.getMode() + 1) % Roaming::roamViewCount));
       } else if (args[2] == "backward") {
-	// FIXME
+	int setto = (ROAM.getMode() - 1) % Roaming::roamViewCount;
+	if (setto < 0) setto += Roaming::roamViewCount;
+	ROAM.setMode(Roaming::RoamingView(setto));
       } else {
 	return "usage: roam cycle {type|subject} {forward|backward}";
       }
     } else if (args[1] == "subject") {
       if (args[2] == "forward") {
-	if (roamView == roamViewFree) {
-	  // do nothing
-	} else if (roamView == roamViewFlag) {
-	  const int maxFlags = world->getMaxFlags();
-	  for (int i = 1; i < maxFlags; i++) {
-	    int j = (roamTrackFlag + i) % maxFlags;
-	    const Flag& flag = world->getFlag(j);
-	    if (flag.type->flagTeam != NoTeam) {
-	      roamTrackFlag = j;
-	      break;
-	    }
-	  }
-	} else {
-	  int i, j;
-	  for (i = 0; i < curMaxPlayers; i++) {
-	    j = (roamTrackTank + i + 2) % (curMaxPlayers + 1) - 1;
-	    if ((j == -1)
-		|| (player[j] && player[j]->getTeam() != ObserverTeam)) {
-	      roamTrackTank = roamTrackWinner = j;
-	      break;
-	    }
-	  }
-	}
-	setRoamingLabel();
+	ROAM.changeTarget(Roaming::next);
       } else if (args[2] == "backward") {
-	if (roamView == roamViewFree) {
-	  // do nothing
-	} else if (roamView == roamViewFlag) {
-	  const int maxFlags = world->getMaxFlags();
-	  for (int i = 1; i < maxFlags; i++) {
-	    int j = (roamTrackFlag - i + maxFlags) % maxFlags;
-	    const Flag& flag = world->getFlag(j);
-	    if (flag.type->flagTeam != NoTeam) {
-	      roamTrackFlag = j;
-	      break;
-	    }
-	  }
-	} else {
-	  for (int i = 0; i < curMaxPlayers; i++) {
-	    int j = (roamTrackTank - i + curMaxPlayers + 1)
-	      % (curMaxPlayers + 1) - 1;
-	    if ((j == -1)
-		|| (player[j] && player[j]->getTeam() != ObserverTeam)) {
-	      roamTrackTank = roamTrackWinner = j;
-	      break;
-	    }
-	  }
-	}
-	setRoamingLabel();
+	ROAM.changeTarget(Roaming::previous);
       } else {
 	return "usage: roam cycle {type|subject} {forward|backward}";
       }
@@ -771,7 +855,8 @@ std::string cmdRoam(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdSilence(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdSilence(const std::string&,
+			      const CommandManager::ArgList& args, bool*)
 {
   static SilenceDefaultKey silenceKeyHandler;
   if (args.size() != 0)
@@ -782,11 +867,13 @@ std::string cmdSilence(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdServerCommand(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdServerCommand(const std::string&,
+				    const CommandManager::ArgList& args, bool*)
 {
   static ServerCommandKey serverCommandKeyHandler;
   if (args.size() != 0)
     return "usage: servercommand";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (!myTank)
     return "use only when connected";
   static bool prevAdmin = myTank->isAdmin();
@@ -800,33 +887,51 @@ std::string cmdServerCommand(const std::string&, const CommandManager::ArgList&
   return std::string();
 }
 
-std::string cmdScrollPanel(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdScrollPanel(const std::string&,
+				  const CommandManager::ArgList& args, bool*)
 {
-  if (args.size() != 1)
-    return "usage: scrollpanel {up|down}\n";
+  if ((args.size() < 1) || (args.size() > 2)) {
+    return "usage: scrollpanel {up|up_page|down|down_page|top|bottom} [count]\n";
+  }
+  int count = 1;
+  int linecount = 2;
+  if (args.size() == 2) {
+    count = atoi(args[1].c_str());
+    linecount = count;
+  }
+  // whence - (0 = set, 1 = cur, 2 = end)
   if (args[0] == "up") {
-    controlPanel->setMessagesOffset(2,1);
+    controlPanel->setMessagesOffset(+linecount, 1 /* current */, false);
   } else if (args[0] == "down") {
-    controlPanel->setMessagesOffset(-2,1);
-  } else {
-    return "usage: scrollpanel {up|down}\n";
+    controlPanel->setMessagesOffset(-linecount, 1 /* current */, false);
+  } else if (args[0] == "up_page") {
+    controlPanel->setMessagesOffset(+count, 1 /* current */, true);
+  } else if (args[0] == "down_page") {
+    controlPanel->setMessagesOffset(-count, 1 /* current */, true);
+  } else if (args[0] == "top") {
+    controlPanel->setMessagesOffset(123456789, 0 /* set */, false);
+  } else if (args[0] == "bottom") {
+    controlPanel->setMessagesOffset(0, 0 /* set */, false);
   }
   return std::string();
 }
 
-std::string cmdHunt(const std::string&, const CommandManager::ArgList& args)
+
+static std::string cmdHunt(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: hunt";
-  if (hud->getHunting()) {
-    hud->setHunting(false);
-  } else {
-    playLocalSound(SFX_HUNT);
-    hud->setHunt(!hud->getHunt());
-    hud->setHuntPosition(0);
-    if (!BZDB.isTrue("displayScore"))
-      BZDB.set("displayScore", "1");
-  }
+   hud->getScoreboard()->huntKeyEvent (false);
+  return std::string();
+}
+
+static std::string cmdAddHunt(const std::string&,
+			      const CommandManager::ArgList& args, bool*)
+{
+  if (args.size() != 0)
+    return "usage: addhunt";
+   hud->getScoreboard()->huntKeyEvent (true);
   return std::string();
 }
 
