@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /*
@@ -24,14 +24,13 @@
 #include <vector>
 
 /* common headers */
-#include "BzfDisplay.h"
-#include "Player.h"
-#include "SceneRenderer.h"
 #include "StartupInfo.h"
+#include "AutoCompleter.h"
 
 /* local headers */
 #include "MainWindow.h"
-
+#include "ControlPanel.h"
+#include "HUDRenderer.h"
 
 #define MAX_MESSAGE_HISTORY (20)
 
@@ -65,13 +64,16 @@ bool			addExplosion(const float* pos,
 				float size, float duration);
 void			addTankExplosion(const float* pos);
 void			addShotExplosion(const float* pos);
-void			addShotPuff(const float* pos);
+void			addShotPuff(const float* pos, float azimuth, float elevation);
 void			warnAboutMainFlags();
 void			warnAboutRadarFlags();
+void		    warnAboutRadar();
+void		    warnAboutConsole();
 void			addPlayingCallback(PlayingCallback, void* data);
 void			removePlayingCallback(PlayingCallback, void* data);
 
 void			joinGame(JoinGameCallback, void* userData);
+void		    leaveGame();
 std::vector<std::string>& getSilenceList();
 void			updateEvents();
 void			addMessage(const Player* player,
@@ -91,12 +93,25 @@ bool shouldGrabMouse();
 void setRoamingLabel();
 void drawFrame(const float dt);
 
+extern void joinGame();
+
 extern HUDRenderer	*hud;
 extern char		messageMessage[PlayerIdPLen + MessageLen];
 extern ServerLink*	serverLink;
 extern int		numFlags;
 extern StartupInfo	startupInfo;
-
+extern DefaultCompleter completer;
+extern bool	     gameOver;
+extern ControlPanel    *controlPanel;
+extern bool	     fireButton;
+extern float	    destructCountdown;
+extern bool	     pausedByUnmap;
+extern int	      savedVolume;
+extern MainWindow      *mainWindow;
+extern float	    pauseCountdown;
+extern float	    clockAdjust;
+extern float	    roamDZoom;
+extern bool	     roamButton;
 
 #endif // BZF_PLAYING_H
 
