@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /*
@@ -19,6 +19,8 @@
 
 #include "common.h"
 
+#include <string>
+
 #define SFX_FIRE	0		/* shell fired */
 #define SFX_EXPLOSION	1		/* something other than me blew up */
 #define SFX_RICOCHET	2		/* shot bounced off building */
@@ -62,12 +64,17 @@ void			moveSoundReceiver(float x, float y, float z, float t,
 							int discontinuity);
 void			speedSoundReceiver(float vx, float vy, float vz);
 
+/* sound effect event at given position in world, or possible locally */
+void			playSound(int soundCode, const float pos[3],
+				  bool important, bool localSound);
+
 /* sound effect event at given position in world */
 void			playWorldSound(int soundCode, const float pos[3],
 				       bool important = false);
 
 /* sound effect positioned at receiver */
 void			playLocalSound(int soundCode);
+void			playLocalSound(std::string sound);
 
 /* start playing a sound effect repeatedly at world position */
 void			playFixedSound(int soundCode,
