@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __ACCESSCONTROLLIST_H__
@@ -28,10 +28,11 @@ struct BanInfo
   /** This constructor creates a new BanInfo with the address @c banAddr,
       the ban performer @c bannedBy, and the expiration time @c period
       minutes from now. */
-  BanInfo( in_addr &banAddr, const char *bannedBy = NULL, int period = 0, bool isFromMaster = false ) {
+  BanInfo(in_addr &banAddr, const char *_bannedBy = NULL, int period = 0,
+	  bool isFromMaster = false ) {
     memcpy( &addr, &banAddr, sizeof( in_addr ));
-    if (bannedBy)
-      this->bannedBy = bannedBy;
+    if (_bannedBy)
+      bannedBy = _bannedBy;
     if (period == 0) {
       banEnd = TimeKeeper::getSunExplodeTime();
     } else {
@@ -66,10 +67,11 @@ struct HostBanInfo
   /** This constructor creates a new HostBanInfo with the host pattern
       @c hostpat, the ban performer @c bannedBy, and the expiration time
       @c period minutes from now. */
-  HostBanInfo(std::string hostpat, const char *bannedBy = NULL, int period = 0, bool isFromMaster = false ) {
-    this->hostpat = hostpat;
-    if (bannedBy)
-      this->bannedBy = bannedBy;
+  HostBanInfo(std::string _hostpat, const char *_bannedBy = NULL,
+	      int period = 0, bool isFromMaster = false ) {
+    hostpat = _hostpat;
+    if (_bannedBy)
+      bannedBy = _bannedBy;
     if (period == 0) {
       banEnd = TimeKeeper::getSunExplodeTime();
     } else {
@@ -83,7 +85,7 @@ struct HostBanInfo
     return hostpat == rhs.hostpat;
   }
   /** Only HostBanInfos with same host pattern are identical. */
-  bool operator != (const HostBanInfo& rhs) const {
+  bool operator!=(const HostBanInfo& rhs) const {
     return hostpat != rhs.hostpat;
   }
 
@@ -94,6 +96,46 @@ struct HostBanInfo
   bool fromMaster;	// where the ban came from, local or master list.
 };
 
+
+/** This struct contains information about an idban - the databaseid pattern
+    that was banned, the time the ban will expire, the callsign of the player
+    who performed the ban, and the reason. It also has operators defined for
+    checking if two bans are equal. */
+struct IdBanInfo
+{
+  /** This constructor creates a new IdBanInfo with the id pattern
+      @c idpat, the ban performer @c bannedBy, and the expiration time
+      @c period minutes from now. */
+  IdBanInfo(std::string _idpat, const char *_bannedBy = NULL,
+	    int period = 0, bool isFromMaster = false ) {
+    idpat = _idpat;
+    if (_bannedBy)
+      bannedBy = _bannedBy;
+    if (period == 0) {
+      banEnd = TimeKeeper::getSunExplodeTime();
+    } else {
+      banEnd = TimeKeeper::getCurrent();
+      banEnd += period * 60.0f;
+    }
+    fromMaster = isFromMaster;
+  }
+  /** IdBanInfos with same id pattern are identical. */
+  bool operator==(const IdBanInfo &rhs) const {
+    return idpat == rhs.idpat;
+  }
+  /** Only IdBanInfos with same id pattern are identical. */
+  bool operator!=(const IdBanInfo& rhs) const {
+    return idpat != rhs.idpat;
+  }
+
+  std::string idpat;
+  TimeKeeper banEnd;
+  std::string bannedBy;
+  std::string reason;
+  bool fromMaster;	// where the ban came from, local or master list.
+};
+
+
 /* FIXME the AccessControlList assumes that 255 is a wildcard. it "should"
  * include a cidr mask with each address. it's still useful as is, though
  * see wildcard conversion occurs in convert().
@@ -110,22 +152,32 @@ public:
   /** This function will add a ban for the address @c ipAddr with the given
       parameters. If that address already is banned the old ban will be
       replaced. */
-  void ban(in_addr &ipAddr, const char *bannedBy, int period = 0, const char *reason=NULL, bool fromMaster = false);
+  void ban(in_addr &ipAddr, const char *bannedBy, int period = 0,
+	   const char *reason=NULL, bool fromMaster = false);
 
   /** This function takes a list of addresses as a string and tries to ban them
       using the given parameters. The string should be comma separated,
       like this: "1.2.3.4,5.6.7.8,9.10.11.12". */
-  bool ban(std::string &ipList, const char *bannedBy=NULL, int period = 0, const char *reason=NULL, bool fromMaster = false);
+  bool ban(std::string &ipList, const char *bannedBy=NULL, int period = 0,
+	   const char *reason=NULL, bool fromMaster = false);
 
   /** This function takes a list of addresses as a <code>const char*</code>
       and tries to ban them using the given parameters. The string should be
       comma separated, like this: "1.2.3.4,5.6.7.8,9.10.11.12". */
-  bool ban(const char *ipList, const char *bannedBy=NULL, int period = 0, const char *reason=NULL, bool fromMaster = false);
+  bool ban(const char *ipList, const char *bannedBy=NULL, int period = 0,
+	   const char *reason=NULL, bool fromMaster = false);
 
   /** This function adds a hostban for the host pattern @c hostpat with the
       given parameters. If the host pattern already is banned the old ban will
       be replaced. */
-  void hostBan(std::string hostpat, const char *bannedBy, int period = 0, const char *reason = NULL, bool fromMaster = false);
+  void hostBan(std::string hostpat, const char *bannedBy, int period = 0,
+	       const char *reason = NULL, bool fromMaster = false);
+
+  /** This function adds an idban for the id pattern @c idpat with the
+      given parameters. If the idban pattern already is banned the old ban will
+      be replaced. */
+  void idBan(std::string hostpat, const char *bannedBy, int period = 0,
+	     const char *reason = NULL, bool fromMaster = false);
 
   /** This function removes any ban for the address @c ipAddr.
       @returns @c true if there was a ban for that address, @c false if there
@@ -149,6 +201,11 @@ public:
       otherwise. */
   bool hostUnban(std::string hostpat);
 
+  /** This function removes any ban for the database id @c idpat.
+      @returns @c true if there was a ban for the database id, @c false
+      otherwise. */
+  bool idUnban(std::string idpat);
+
   /** This function checks if an address is "valid" or not. Valid in this case
       means that it has not been banned.
       @returns @c true if the address is valid, @c false if not. */
@@ -163,11 +220,22 @@ public:
       @returns @c true if the hostname is valid, @c false if it isn't. */
   bool hostValidate(const char *hostname, HostBanInfo *info = NULL);
 
+  /** This function checks that an id is "valid". In this case valid means
+      "not banned".
+      @returns @c true if the id is valid, @c false if it isn't. */
+  bool idValidate(const char *idname, IdBanInfo *info = NULL);
+
+  /** This function sends a textual list of the given IP ban to a player. */
+  void sendBan(PlayerId id, const BanInfo&);
+
   /** This function sends a textual list of all IP bans to a player. */
-  void sendBans(PlayerId id);
+  void sendBans(PlayerId id, const char* pattern);
 
   /** This function sends a textual list of all host bans to a player. */
-  void sendHostBans(PlayerId id);
+  void sendHostBans(PlayerId id, const char* pattern);
+
+  /** This function sends a textual list of all id bans to a player. */
+  void sendIdBans(PlayerId id, const char* pattern);
 
   /** This function tells this object where to save the banlist, and where
       to load it from. */
@@ -184,7 +252,7 @@ public:
   /** This function merges in a banlist from the master ban list ban file,
       if it has been set. It only returns @c false if the file exist but is not
       in the correct format, otherwise @c true is returned. */
-  bool merge(const std::string& banData);
+  int  merge(const std::string& banData);
 
   /** This function purges any bans that are flagged as from the master ban list,
       presumably so it can be remerged */
@@ -207,6 +275,9 @@ private:
   typedef std::vector<HostBanInfo> hostBanList_t;
   hostBanList_t hostBanList;
 
+  typedef std::vector<IdBanInfo> idBanList_t;
+  idBanList_t idBanList;
+
   std::string banFile;
 };
 
