@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 // interface header
@@ -16,9 +16,9 @@
 // implementation-specific system headers
 #include <fstream>
 #include <sstream>
+#include <ctype.h>
 
 // implementation-specific bzflag headers
-#include "URLManager.h"
 #include "BZDBCache.h"
 
 // implementation-specific bzfs-specific headers
@@ -49,12 +49,14 @@
 #include "ObstacleMgr.h"
 #include "BaseBuilding.h"
 #include "TextUtils.h"
+#include "StateDatabase.h"
 
-// FIXME - external dependancies (from bzfs.cxx)
-extern BasesList bases;
+// bzfs specific headers
+#include "bzfs.h"
 
 
-BZWReader::BZWReader(std::string filename) : location(filename), input(NULL)
+BZWReader::BZWReader(std::string filename) : cURLManager(), location(filename),
+					     input(NULL)
 {
   static const std::string httpProtocol("http://");
   static const std::string ftpProtocol("ftp://");
@@ -65,7 +67,8 @@ BZWReader::BZWReader(std::string filename) : location(filename), input(NULL)
   if ((filename.substr(0, httpProtocol.size()) == httpProtocol)
       || (filename.substr(0, ftpProtocol.size()) == ftpProtocol)
       || (filename.substr(0, fileProtocol.size()) == fileProtocol)) {
-    URLManager::instance().getURL(location, httpData);
+    setURL(location);
+    performWait();
     input = new std::istringstream(httpData);
   } else {
     input = new std::ifstream(filename.c_str(), std::ios::in);
@@ -93,6 +96,14 @@ BZWReader::~BZWReader()
 }
 
 
+void BZWReader::finalization(char *data, unsigned int length, bool good)
+{
+  if (good)
+    httpData = std::string(data, length);
+  else
+    httpData = "";
+}
+
 void BZWReader::readToken(char *buffer, int n)
 {
   int c = -1;
@@ -170,7 +181,6 @@ static bool parseNormalObject(const char* token, WorldFileObject** object)
   }
 }
 
-
 bool BZWReader::readWorldStream(std::vector<WorldFileObject*>& wlist,
 				GroupDefinition* groupDef)
 {
@@ -188,6 +198,9 @@ bool BZWReader::readWorldStream(std::vector<WorldFileObject*>& wlist,
   GroupDefinition* const worldDef = (GroupDefinition*)OBSTACLEMGR.getWorld();
   GroupDefinition* const startDef = groupDef;
 
+  std::string customObject;
+  std::vector<std::string>	customLines;
+
   bool gotWorld = false;
 
   while (!input->eof() && !input->fail() && input->good())
@@ -200,6 +213,11 @@ bool BZWReader::readWorldStream(std::vector<WorldFileObject*>& wlist,
 	if (object != fakeObject) {
 	  delete object;
 	}
+	else if (customObject.size())
+	{
+		customObject = "";
+		customLines.clear();
+	}
       }
       object = newObject;
       newObject = NULL;
@@ -224,6 +242,13 @@ bool BZWReader::readWorldStream(std::vector<WorldFileObject*>& wlist,
 	  } else {
 	    wlist.push_back(object);
 	  }
+	} else if (customObject.size())	{
+	  bz_CustomMapObjectInfo data;
+	  data.name = bzApiString(customObject);
+	  for (unsigned int i = 0; i < customLines.size(); i++)
+	    data.data.push_back(customLines[i]);
+	  customObjectMap[customObject]->handle(bzApiString(customObject),&data);
+	  object = NULL;
 	}
 	object = NULL;
       } else {
@@ -329,16 +354,32 @@ bool BZWReader::readWorldStream(std::vector<WorldFileObject*>& wlist,
 	  // delete object;
 	  // return false;
 	}
+      } else if (customObject.size()) {
+	std::string thisline = buffer;
+	thisline += " ";
+
+	while (input->good() && input->peek() != '\n') {
+	  input->get(buffer, sizeof(buffer));
+	  thisline += buffer;
+	}
+
+	customLines.push_back(thisline);
       }
 
     } else { // filling the current object
       // unknown token
-      errorHandler->warning(
-	std::string("invalid object type \"") +
-	std::string(buffer) + std::string("\" - skipping"), line);
-      if (object != fakeObject)
-	delete object;
-     // return false;
+      if (customObjectMap.find(TextUtils::toupper(std::string(buffer))) != customObjectMap.end()) {
+	customObject = TextUtils::toupper(std::string(buffer));
+	object = fakeObject;
+	customLines.clear();
+      }	else {
+	errorHandler->warning(
+	  std::string("invalid object type \"") +
+	  std::string(buffer) + std::string("\" - skipping"), line);
+	if (object != fakeObject)
+	  delete object;
+      }
+      // return false;
     }
 
     // discard remainder of line
@@ -370,8 +411,8 @@ bool BZWReader::readWorldStream(std::vector<WorldFileObject*>& wlist,
 WorldInfo* BZWReader::defineWorldFromFile()
 {
   // create world object
-  WorldInfo *world = new WorldInfo;
-  if (!world) {
+  WorldInfo *myWorld = new WorldInfo;
+  if (!myWorld) {
     errorHandler->fatalError(std::string("WorldInfo failed to initialize"), 0);
     return NULL;
   }
@@ -382,17 +423,23 @@ WorldInfo* BZWReader::defineWorldFromFile()
   if (!readWorldStream(list, worldDef)) {
     emptyWorldFileObjectList(list);
     errorHandler->fatalError(std::string("world file failed to load."), 0);
-    delete world;
+    delete myWorld;
     return NULL;
   }
 
-  // make walls
-  float wallHeight = BZDB.eval(StateDatabase::BZDB_WALLHEIGHT);
-  float worldSize = BZDBCache::worldSize;
-  world->addWall(0.0f, 0.5f * worldSize, 0.0f, (float)(1.5 * M_PI), 0.5f * worldSize, wallHeight);
-  world->addWall(0.5f * worldSize, 0.0f, 0.0f, (float)M_PI, 0.5f * worldSize, wallHeight);
-  world->addWall(0.0f, -0.5f * worldSize, 0.0f, (float)(0.5 * M_PI), 0.5f * worldSize, wallHeight);
-  world->addWall(-0.5f * worldSize, 0.0f, 0.0f, 0.0f, 0.5f * worldSize, wallHeight);
+  if (!BZDB.isTrue("noWalls")) {
+    // make walls
+    float wallHeight = BZDB.eval(StateDatabase::BZDB_WALLHEIGHT);
+    float worldSize = BZDBCache::worldSize;
+    myWorld->addWall(0.0f, 0.5f * worldSize, 0.0f, (float)(1.5 * M_PI),
+		     0.5f * worldSize, wallHeight);
+    myWorld->addWall(0.5f * worldSize, 0.0f, 0.0f, (float)M_PI, 0.5f * worldSize,
+		     wallHeight);
+    myWorld->addWall(0.0f, -0.5f * worldSize, 0.0f, (float)(0.5 * M_PI),
+		     0.5f * worldSize, wallHeight);
+    myWorld->addWall(-0.5f * worldSize, 0.0f, 0.0f, 0.0f, 0.5f * worldSize,
+		     wallHeight);
+  }
 
   // generate group instances
   OBSTACLEMGR.makeWorld();
@@ -413,14 +460,14 @@ WorldInfo* BZWReader::defineWorldFromFile()
   // add objects
   const unsigned int n = list.size();
   for (i = 0; i < n; ++i) {
-    list[i]->writeToWorld(world);
+    list[i]->writeToWorld(myWorld);
   }
 
   // clean up
   emptyWorldFileObjectList(list);
-  world->finishWorld();
+  myWorld->finishWorld();
 
-  return world;
+  return myWorld;
 }
 
 
