@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifdef _MSC_VER
@@ -19,15 +19,17 @@
  */
 #include "CmdLineOptions.h"
 
+
 // implementation-specific bzflag headers
 #include "version.h"
 #include "Team.h"
 #include "TextUtils.h"
 #include "BZDBCache.h"
+#include "BzMaterial.h"
 
-/* FIXME implementation specific header for global that should eventually go
- * away */
+/* FIXME implementation specific header for global that should eventually go away */
 #include <vector>
+#include <set>
 
 // for -pidfile option
 #ifdef _WIN32
@@ -38,27 +40,20 @@
 #endif
 
 // implementation-specific bzfs-specific headers
-#include "RecordReplay.h"
 #include "bzfs.h"
-#include "FlagInfo.h"
+#include "RecordReplay.h"
 #include "BZWError.h"
+#include "Permissions.h"
+#include "EntryZones.h"
+
 
 // import from TextUtils for convenience
 using TextUtils::compare_nocase;
 
-/* data nasties */
-extern float speedTolerance;
-extern bool handlePings;
-extern int numFlags;
-extern std::string passFile;
-extern std::string groupsFile;
-extern std::string userDatabaseFile;
-extern uint16_t maxPlayers;
-extern uint16_t maxRealPlayers;
-
 const char *usageString =
 "[-a <vel> <rot>] "
 "[-admsg <text>] "
+"[-advertise <group,group...>]"
 "[-autoTeam] "
 "[-b] "
 "[-badwords <filename>] "
@@ -79,6 +74,7 @@ const char *usageString =
 "[-filterChat] "
 "[-filterSimple] "
 "[-g] "
+"[-gndtex <texture name>] "
 "[-groupdb <group file>] "
 "[-h] "
 "[-handicap] "
@@ -87,6 +83,7 @@ const char *usageString =
 "[-j] "
 "[-lagdrop <num>] "
 "[-lagwarn <time/ms>] "
+"[-loadplugin <pluginname,commandline>] "
 "[-masterBanURL <URL>]"
 "[-maxidle <time/s>] "
 "[-mp {<count>|[<count>][,<count>][,<count>][,<count>][,<count>][,<count>]}] "
@@ -110,6 +107,7 @@ const char *usageString =
 "[+r] "
 "[-rabbit [score|killer|random]] "
 "[-recbuf <Mbytes>] "
+"[-recbufonly] "
 "[-recdir <dirname>] "
 "[-replay] "
 "[-reportfile <filename>] "
@@ -120,6 +118,7 @@ const char *usageString =
 "[-sa] "
 "[-sb] "
 "[-set <name> <value>] "
+"[-setforced <name> <value>] "
 "[-sl <id> <num>] "
 "[-spamtime <time>] "
 "[-spamwarn <warnAmt>] "
@@ -135,6 +134,7 @@ const char *usageString =
 "[-timemanual] "
 "[-tk] "
 "[-tkkr <percent>] "
+"[-ts [micros]] "
 "[-userdb <user permissions file>] "
 "[-vars <filename>] "
 "[-version] "
@@ -147,18 +147,19 @@ const char *extraUsageString =
 "\n"
 "\t-a: maximum acceleration settings\n"
 "\t-admsg: specify a <msg> which will be broadcast every 15 minutes\n"
+"\t-advertise: specify which groups to advertise to (list server)\n"
 "\t-autoTeam: automatically assign players to teams when they join\n"
 "\t-b: randomly oriented buildings\n"
 "\t-badwords: bad-world file\n"
 "\t-ban ip{,ip}*: ban players based on ip address\n"
 "\t-banfile: specify a file to load and store the banlist in\n"
 "\t-c: capture-the-flag style game,\n"
-"\t-cache: url to get binary formatted world,\n"
+"\t-cache: url to get binary formatted world\n"
 "\t-cacheout: generate a binary cache file\n"
 "\t-conf: configuration file\n"
 "\t-cr: capture-the-flag style game with random world\n"
 "\t-d: increase debugging level\n"
-"\t-density: specify building density for random worlds (default is 5)\n"
+"\t-density: specify building density for random worlds (default=5)\n"
 "\t-disableBots: disallow clients from using autopilot or robots\n"
 "\t+f: always have flag <id> available\n"
 "\t-f: never randomly generate flag <id>\n"
@@ -167,6 +168,7 @@ const char *extraUsageString =
 "\t-filterChat: filter chat messages\n"
 "\t-filterSimple: perform simple exact matches with the bad word list\n"
 "\t-g: serve one game and then exit\n"
+"\t-gndtex: specify ground texture\n"
 "\t-groupdb: file to read for group permissions\n"
 "\t-h: use random building heights\n"
 "\t-handicap: give advantage based on relative playing ability\n"
@@ -175,6 +177,8 @@ const char *extraUsageString =
 "\t-j: allow jumping\n"
 "\t-lagdrop: drop player after this many lag warnings\n"
 "\t-lagwarn: lag warning threshhold time [ms]\n"
+"\t-loadplugin: load the specified plugin with the specified commandline\n"
+"\t\tstring\n"
 "\t-masterBanURL: URL to atempt to get the master ban list from <URL>\n"
 "\t-maxidle: idle kick threshhold [s]\n"
 "\t-mp: maximum players total or per team\n"
@@ -183,7 +187,7 @@ const char *extraUsageString =
 "\t-mts: set team score limit on each game\n"
 "\t-noMasterBanlist: has public servers ignore the master ban list\n"
 "\t-noradar: disallow the use of radar\n"
-"\t-p: use alternative port (default is 5154)\n"
+"\t-p: use alternative port (default=5154)\n"
 "\t-passdb: file to read for user passwords\n"
 "\t-passwd: specify a <password> for operator commands\n"
 "\t-pidfile: write the process id into <filename> on startup\n"
@@ -198,6 +202,7 @@ const char *extraUsageString =
 "\t+r: all shots ricochet\n"
 "\t-rabbit [score|killer|random]: rabbit chase style\n"
 "\t-recbuf <Mbytes>: start with a recording buffer of specified megabytes\n"
+"\t-recbufonly: disable recording directly to files\n"
 "\t-recdir <dirname>: specify the directory for recorded file\n"
 "\t-replay: setup the server to replay a previously saved game\n"
 "\t-reportfile <filename>: the file to store reports in\n"
@@ -208,11 +213,15 @@ const char *extraUsageString =
 "\t-sa: insert antidote superflags\n"
 "\t-sb: allow tanks to respawn on buildings\n"
 "\t-set <name> <value>: set a BZDB variable's value\n"
+"\t-setforced <name> <value>: set a BZDB variable's value (whether it\n"
+"\t\texists or not)\n"
 "\t-sl: limit flag <id> to <num> shots\n"
-"\t-spamtime <time>: make <time> be the required time in seconds between messages sent that are alike\n"
-"\t-spamwarn <warnAmt>: warn a spammer that sends messages before -spamtime times out <warnAmt> many times\n"
-"\t-speedtol: multiplyers over normal speed to auto kick at\n"
-"\t\tdefaults to 1.25, should not be less then 1.0\n"
+"\t-spamtime <time>: make <time> be the required time in seconds between\n"
+"\t\tmessages sent that are alike\n"
+"\t-spamwarn <warnAmt>: warn a spammer that sends messages before\n"
+"\t\tspamtime times out <warnAmt> many times\n"
+"\t-speedtol: multiplier of normal speed for auto kick (default=1.25)\n"
+"\t\tshould not be less than 1.0\n"
 "\t-srvmsg: specify a <msg> to print upon client login\n"
 "\t-st: shake bad flags in <time> seconds\n"
 "\t-sw: shake bad flags after <num> wins\n"
@@ -224,23 +233,26 @@ const char *extraUsageString =
 "\t-timemanual: countdown for timed games is started with /countdown\n"
 "\t-tk: player does not die when killing a teammate\n"
 "\t-tkkr: team-kills-to-wins percentage (1-100) for kicking tk-ing players\n"
+"\t-ts [micros]: timestamp all console output, [micros] to include\n"
+"\t\tmicroseconds\n"
 "\t-userdb: file to read for user access permissions\n"
 "\t-vars: file to read for worlds configuration variables\n"
 "\t-version: print version and exit\n"
 "\t-world: world file to load\n"
-"\t-worldsize: numeric value for the size of the world (default 400)\n"
+"\t-worldsize: numeric value for the size of the world (default=400)\n"
 "\n"
 "Poll Variables:  (see -poll)\n"
 "\n"
-"\tbanTime: number of minutes player should be banned (default is 300)\n"
-"\tvetoTime: max seconds authorized user has to abort poll (default is 20)\n"
+"\tbanTime: number of minutes player should be banned (default=300)\n"
+"\tvetoTime: max seconds authorized user has to abort poll (default=20)\n"
 "\tvotePercentage: percentage of players required to affirm a poll\n"
-"\t\t(default is 50.1%)\n"
+"\t\t(default=50.1%)\n"
 "\tvoteRepeatTime: minimum seconds required before a player may request\n"
-"\t\tanother vote (default is 300)\n"
+"\t\tanother vote (default=300)\n"
 "\tvotesRequired: minimum number of additional votes required to make a\n"
-"\t\tvote valid (default is 2)\n"
-"\tvoteTime: maximum amount of time player has to vote, in seconds (default is 60)\n"
+"\t\tvote valid (default=2)\n"
+"\tvoteTime: maximum amount of time player has to vote, in seconds\n"
+"\t\t(default=60)\n"
 "\n";
 
 
@@ -276,7 +288,7 @@ static void extraUsage(const char *pname)
 
 static void checkArgc(int count, int& i, int argc, const char* option, const char *type = NULL)
 {
-  if ((i+count) == argc) {
+  if ((i+count) >= argc) {
     if (count > 1) {
       std::cerr << count << " argument(s) expected for " << option << '\n';
     } else if (type != NULL) {
@@ -298,6 +310,8 @@ static void checkFromWorldFile (const char *option, bool fromWorldFile)
   }
 }
 
+
+
 static bool parsePlayerCount(const char *argv, CmdLineOptions &options)
 {
   /* either a single number or 5 or 6 optional numbers separated by
@@ -444,7 +458,7 @@ static char **parseConfFile( const char *file, int &ac)
       }
    }
   } else {
-	errorHandler.fatalError(std::string("could not find bzflag configuration file"), 0);
+    errorHandler.fatalError(std::string("could not find bzflag configuration file"), 0);
   }
 
   const char **av = new const char*[tokens.size()+1];
@@ -473,7 +487,7 @@ static char **parseWorldOptions (const char *file, int &ac)
     while (confStrm.good()) {
       std::string line = buffer;
       int startPos = line.find_first_not_of("\t \r\n");
-      if (strncmp ("options", line.c_str() + startPos, 7) == 0) {
+      if (strncasecmp ("options", line.c_str() + startPos, 7) == 0) {
 	confStrm.getline(buffer,1024);
 	break;
       }
@@ -483,7 +497,7 @@ static char **parseWorldOptions (const char *file, int &ac)
     while (confStrm.good()) {
       std::string line = buffer;
       int startPos = line.find_first_not_of("\t \r\n");
-      if (strncmp ("end", line.c_str() + startPos, 3) == 0) {
+      if (strncasecmp ("end", line.c_str() + startPos, 3) == 0) {
 	break;
       }
 
@@ -514,22 +528,24 @@ static char **parseWorldOptions (const char *file, int &ac)
 }
 
 
-/* protected */
-
-/* public: */
+static bool accLimitSet = false;
+static bool allFlagsOut = false;
 
 void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
 {
   // prepare flag counts
   int i;
-  static bool allFlagsOut = false;
-  static bool teamFlagsAdded = false;
+
+  // InertiaGameStyle maintained just for compatibility
+  // Same effect is achieved setting linear/angular Acceleration
+  options.gameStyle |= int(InertiaGameStyle);
 
   // parse command line
   int playerCountArg = 0,playerCountArg2 = 0;
   for (i = 1; i < argc; i++) {
     if (strcmp(argv[i], "-a") == 0) {
       // momentum settings
+      accLimitSet = true;
       checkArgc(2, i, argc, argv[i]);
       options.linearAcceleration = (float)atof(argv[i]);
       options.angularAcceleration = (float)atof(argv[++i]);
@@ -537,13 +553,18 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
 	options.linearAcceleration = 0.0f;
       if (options.angularAcceleration < 0.0f)
 	options.angularAcceleration = 0.0f;
-      options.gameStyle |= int(InertiaGameStyle);
     } else if (strcmp(argv[i], "-admsg") == 0) {
       checkArgc(1, i, argc, argv[i]);
       if ((options.advertisemsg != "") || (strlen (argv[i]) == 0)) {
 	options.advertisemsg += "\\n";
       }
       options.advertisemsg += argv[i];
+    } else if (strcmp(argv[i], "-advertise") == 0) {
+      checkArgc(1, i, argc, argv[i]);
+      if (checkCommaList (argv[i], 2048))
+	      std::cerr << "Invalid group list for -advertise" << std::endl;
+      else
+	options.advertiseGroups = argv[i];
     } else if (strcmp(argv[i], "-autoTeam") == 0) {
       options.autoTeam = true;
     } else if (strcmp(argv[i], "-b") == 0) {
@@ -570,11 +591,6 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
 	std::cerr << "Capture the flag incompatible with Rabbit Chase" << std::endl;
 	std::cerr << "Capture the flag assumed" << std::endl;
       }
-      if (!teamFlagsAdded) {
-	for (int t = RedTeam; t <= PurpleTeam; t++)
-	  options.numTeamFlags[t] += 1;
-	teamFlagsAdded = true;
-      }
     } else if (strcmp(argv[i], "-cache") == 0) {
       checkArgc(1, i, argc, argv[i]);
       options.cacheURL = argv[i];
@@ -591,8 +607,8 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
       // the cmd line options. But for now just override them on the spot
       parse(ac, av, options);
 
-      for (int i = 0; i < ac; i++)
-	free(av[i]);
+      for (int j = 0; j < ac; j++)
+	free(av[j]);
       delete[] av;
 
       options.numAllowedFlags = 0;
@@ -607,11 +623,6 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
 	std::cerr << "Capture the flag incompatible with Rabbit Chase" << std::endl;
 	std::cerr << "Capture the flag assumed" << std::endl;
       }
-      if (!teamFlagsAdded) {
-	for (int t = RedTeam; t <= PurpleTeam; t++)
-	  options.numTeamFlags[t] += 1;
-	teamFlagsAdded = true;
-      }
     } else if (strcmp(argv[i], "-density") ==0) {
       if (i+1 != argc && isdigit(*argv[i+1])) {
 	options.citySize = atoi(argv[i+1]);
@@ -698,6 +709,12 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
       options.filterSimple = true;
     } else if (strcmp(argv[i], "-g") == 0) {
       options.oneGameOnly = true;
+    } else if (strcmp(argv[i], "-gndtex") == 0) {
+      checkArgc(1, i, argc, argv[i]);
+      BzMaterial material;
+      material.setName("GroundMaterial");
+      material.setTexture(argv[i]);
+      MATERIALMGR.addMaterial(&material);
     } else if (strcmp(argv[i], "-groupdb") == 0) {
       checkArgc(1, i, argc, argv[i]);
       groupsFile = argv[i];
@@ -708,7 +725,7 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
       extraUsage(argv[0]);
     } else if (strcmp(argv[i], "-helpmsg") == 0) {
       checkArgc(2, i, argc, argv[i]);
-      if (!options.textChunker.parseFile(argv[i], argv[i+1])){
+      if (!options.textChunker.parseFile(argv[i], argv[i+1], 50, MessageLen)){
 	std::cerr << "couldn't read helpmsg file \"" << argv[i] << "\"" << std::endl;
 	usage(argv[0]);
       }
@@ -729,10 +746,20 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
     } else if (strcmp(argv[i], "-lagwarn") == 0) {
       checkArgc(1, i, argc, argv[i]);
       options.lagwarnthresh = atoi(argv[i])/1000.0f;
+    } else if (strcmp(argv[i], "-loadplugin") == 0) {
+      checkArgc(1, i, argc, argv[i]);
+      std::vector<std::string> a = TextUtils::tokenize(argv[i],std::string(","), 2);
+      CmdLineOptions::pluginDef	pDef;
+      if (a.size() >= 1)
+	pDef.plugin = a[0];
+      if (a.size() >= 2)
+	pDef.command = a[1];
+      if (pDef.plugin.size())
+	options.pluginList.push_back(pDef);
     } else if (strcmp(argv[i], "-maxidle") == 0) {
       checkArgc(1, i, argc, argv[i]);
       options.idlekickthresh = (float) atoi(argv[i]);
-    } else if (strcmp(argv[i], "-mp") == 0) {
+	} else if (strcmp(argv[i], "-mp") == 0) {
       // set maximum number of players
       checkArgc(1, i, argc, argv[i]);
       if (playerCountArg == 0)
@@ -852,7 +879,6 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
       options.printScore = true;
 #endif
     } else if (strcmp(argv[i], "-public") == 0) {
-      checkFromWorldFile(argv[i], fromWorldFile);
       checkArgc(1, i, argc, argv[i]);
       options.publicizeServer = true;
       options.publicizedTitle = argv[i];
@@ -912,6 +938,8 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
       checkArgc(1, i, argc, argv[i]);
       Record::setSize (ServerPlayer, atoi(argv[i]));
       options.startRecording = true;
+    } else if (strcmp(argv[i], "-recbufonly") == 0) {
+      Record::setAllowFileRecs (false);
     } else if (strcmp(argv[i], "-recdir") == 0) {
       checkArgc(1, i, argc, argv[i]);
       Record::setDirectory (argv[i]);
@@ -955,6 +983,14 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
       value = argv[i];
       BZDB.set(name, value);
       DEBUG1 ("set variable: %s = %s\n", name, BZDB.get(name).c_str());
+    } else if (strcmp(argv[i], "-setforced") == 0) {
+      const char *name, *value;
+      checkArgc(2, i, argc, argv[i]);
+      name = argv[i];
+      i++;
+      value = argv[i];
+      BZDB.set(name, value);
+      DEBUG1 ("set variable: %s = %s\n", name, BZDB.get(name).c_str());
     } else if (strcmp(argv[i], "-sl") == 0) {
       // add required flag
       checkArgc(2, i, argc, argv[i]);
@@ -1062,6 +1098,16 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
 	options.teamKillerKickRatio = 0;
 	std::cerr << "disabling team killer kick ratio";
       }
+	} else if (strcmp(argv[i], "-ts") == 0) {
+      // timestamp output
+      options.timestampLog = true;
+      // if there is an argument following, see if it is 'micros'
+      if (i+1 != argc) {
+	if (strcasecmp(argv[i+1], "micros") == 0) {
+	  options.timestampMicros = true;
+	  i++;
+	}
+      }
     } else if (strcmp(argv[i], "-userdb") == 0) {
       checkArgc(1, i, argc, argv[i]);
       userDatabaseFile = argv[i];
@@ -1080,8 +1126,8 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
       char **av = parseWorldOptions(argv[i], ac);
       parse(ac, av, options, true); // true - from a world file
 
-      for (int i = 0; i < ac; i++)
-	free(av[i]);
+      for (int j = 0; j < ac; j++)
+	free(av[j]);
       delete[] av;
 
       options.numAllowedFlags = 0; // FIXME - Huh, does a reset?
@@ -1098,39 +1144,99 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
     }
   }
 
+  // get player counts.  done after other arguments because we need
+  // to ignore counts for rogues if rogues aren't allowed.
+  if ((playerCountArg > 0) && !parsePlayerCount(argv[playerCountArg], options)) {
+    usage(argv[0]);
+  }
+  if ((playerCountArg2 > 0)
+      && !parsePlayerCount(argv[playerCountArg2], options)) {
+    usage(argv[0]);
+  }
+
+  return;
+}
+
+
+static int getZoneTeamFlagCount(FlagType* flagType, EntryZones& entryZones,
+				 const std::set<FlagType*>& forbidden)
+{
+  if (forbidden.find(flagType) != forbidden.end()) {
+    return 0;
+  }
+  // tally zone team flags
+  int count = 0;
+  const ZoneList& zl = entryZones.getZoneList();
+  for (int z = 0; z < (int)zl.size(); z++) {
+    const ZoneFlagMap& zfm = zl[z].getZoneFlagMap();
+    ZoneFlagMap::const_iterator zfmIt;
+    for (zfmIt = zfm.begin(); zfmIt != zfm.end(); zfmIt++) {
+      if (zfmIt->first == flagType) {
+	count += zfmIt->second;
+      }
+    }
+  }
+  return count;
+}
+
+
+static int addZoneTeamFlags(int startIndex,
+			    FlagType* flagType, EntryZones& entryZones,
+			    const std::set<FlagType*>& forbidden)
+{
+  if (forbidden.find(flagType) != forbidden.end()) {
+    return startIndex;
+  }
+  // add zone team flags
+  const ZoneList& zl = entryZones.getZoneList();
+  for (int z = 0; z < (int)zl.size(); z++) {
+    const ZoneFlagMap& zfm = zl[z].getZoneFlagMap();
+    ZoneFlagMap::const_iterator zfmIt;
+    for (zfmIt = zfm.begin(); zfmIt != zfm.end(); zfmIt++) {
+      if (zfmIt->first == flagType) {
+	const int count = zfmIt->second;
+	for (int c = 0; c < count; c++) {
+	  entryZones.addZoneFlag(z, startIndex);
+	  FlagInfo::get(startIndex++)->setRequiredFlag(flagType);
+	}
+      }
+    }
+  }
+  return startIndex;
+}
+
+
+void finalizeParsing(int /*argc*/, char **argv,
+		     CmdLineOptions &options, EntryZones& entryZones)
+{
   if (options.flagsOnBuildings && !(options.gameStyle & JumpingGameStyle)) {
     std::cerr << "flags on boxes requires jumping" << std::endl;
     usage(argv[0]);
   }
 
-  // get player counts.  done after other arguments because we need
-  // to ignore counts for rogues if rogues aren't allowed.
-  if ((playerCountArg > 0) && !parsePlayerCount(argv[playerCountArg], options))
-    usage(argv[0]);
-  if ((playerCountArg2 > 0)
-      && !parsePlayerCount(argv[playerCountArg2], options))
-    usage(argv[0]);
+  if (options.gameStyle & int(RabbitChaseGameStyle)) {
+    for (int j = RedTeam; j <= PurpleTeam; j++) {
+      if (options.maxTeam[j] > 0
+	  && options.maxTeam[RogueTeam] != maxRealPlayers)
+	std::cout << "only rogues are allowed in Rabbit Style; zeroing out "
+		  << Team::getName((TeamColor) j) << std::endl;
+      options.maxTeam[j] = 0;
+    }
+  }
 
-  // first disallow flags inconsistent with game style
-  if (options.gameStyle & InertiaGameStyle) {
-    options.flagCount[Flags::Momentum] = 0;
-    options.flagDisallowed[Flags::Momentum] = true;
-  }
-  if (options.gameStyle & JumpingGameStyle) {
-    options.flagCount[Flags::Jumping] = 0;
-    options.flagDisallowed[Flags::Jumping] = true;
-  } else {
-    options.flagCount[Flags::NoJumping] = 0;
-    options.flagDisallowed[Flags::NoJumping] = true;
-  }
-  if (options.gameStyle & RicochetGameStyle) {
-    options.flagCount[Flags::Ricochet] = 0;
-    options.flagDisallowed[Flags::Ricochet] = true;
-  }
-  if (!options.useTeleporters && (options.worldFile == "")) {
-    options.flagCount[Flags::PhantomZone] = 0;
-    options.flagDisallowed[Flags::PhantomZone] = true;
+  if (!accLimitSet) {
+    printf("Note: no acceleration limit has been set.  Players using \"mouse\n"
+	   "enhancements\" may cause problems on this server due to very high\n"
+	   "acceleration rates which are not handled well by dead reckoning\n"
+	   "algorithms.  To eliminate this warning, set the -a switch in your\n"
+	   "configuration.  '-a 50 38' is recommended for standard-speed servers.\n"
+	   "Higher speed servers will need higher values for -a in order to not\n"
+	   "affect gameplay.  '-a 0 0' may be used to shut this message up without\n"
+	   "affecting any players, including those using \"mouse enhancements\".\n");
   }
+
+
+  // do we have any team flags?
   bool hasTeam = false;
   for (int p = RedTeam; p <= PurpleTeam; p++) {
     if (options.maxTeam[p] > 1) {
@@ -1138,20 +1244,71 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
       break;
     }
   }
+
+  std::set<FlagType*> forbidden;
+  forbidden.insert(Flags::Null);
+
+  // first disallow flags inconsistent with game style
+  if (options.gameStyle & JumpingGameStyle) {
+    forbidden.insert(Flags::Jumping);
+  } else {
+    forbidden.insert(Flags::NoJumping);
+  }
+  if (options.gameStyle & RicochetGameStyle) {
+    forbidden.insert(Flags::Ricochet);
+  }
+  if (!options.useTeleporters && (options.worldFile == "")) {
+    forbidden.insert(Flags::PhantomZone);
+  }
   if (!hasTeam) {
-    options.flagCount[Flags::Genocide] = 0;
-    options.flagDisallowed[Flags::Genocide] = true;
-    options.flagCount[Flags::Colorblindness] = 0;
-    options.flagDisallowed[Flags::Colorblindness] = true;
-    options.flagCount[Flags::Masquerade] = 0;
-    options.flagDisallowed[Flags::Masquerade] = true;
+    forbidden.insert(Flags::Genocide);
+    forbidden.insert(Flags::Colorblindness);
+    forbidden.insert(Flags::Masquerade);
+  }
+  if ((options.gameStyle & TeamFlagGameStyle) == 0) {
+    forbidden.insert(Flags::RedTeam);
+    forbidden.insert(Flags::GreenTeam);
+    forbidden.insert(Flags::BlueTeam);
+    forbidden.insert(Flags::PurpleTeam);
+  }
+  if (options.maxTeam[RedTeam] <= 0) {
+    forbidden.insert(Flags::RedTeam);
+  }
+  if (options.maxTeam[GreenTeam] <= 0) {
+    forbidden.insert(Flags::GreenTeam);
+  }
+  if (options.maxTeam[BlueTeam] <= 0) {
+    forbidden.insert(Flags::BlueTeam);
+  }
+  if (options.maxTeam[PurpleTeam] <= 0) {
+    forbidden.insert(Flags::PurpleTeam);
   }
 
-  if (options.gameStyle & int(RabbitChaseGameStyle)) {
-    for (int i = RedTeam; i <= PurpleTeam; i++) {
-      if (options.maxTeam[i] > 0 && options.maxTeam[RogueTeam] != maxRealPlayers)
-	std::cout << "only rogues are allowed in Rabbit Style; zeroing out " << Team::getName((TeamColor) i) << std::endl;
-      options.maxTeam[i] = 0;
+  // void the forbidden flags
+  std::set<FlagType*>::const_iterator sit;
+  for (sit = forbidden.begin(); sit != forbidden.end(); sit++) {
+    FlagType* ft = *sit;
+    options.flagCount[ft] = 0;
+    options.flagDisallowed[ft] = true;
+  }
+
+  // zone team flag counts
+  const int zoneTeamFlagCounts[5] = {
+    0, // rogue
+    getZoneTeamFlagCount(Flags::RedTeam, entryZones, forbidden),
+    getZoneTeamFlagCount(Flags::GreenTeam, entryZones, forbidden),
+    getZoneTeamFlagCount(Flags::BlueTeam, entryZones, forbidden),
+    getZoneTeamFlagCount(Flags::PurpleTeam, entryZones, forbidden)
+  };
+
+  // make sure there is at least one team flag for each active team
+  if (options.gameStyle & TeamFlagGameStyle) {
+    for (int col = RedTeam; col <= PurpleTeam; col++) {
+      if ((options.maxTeam[col] > 0) &&
+	  (options.numTeamFlags[col] <= 0) &&
+	  (zoneTeamFlagCounts[col] <= 0)) {
+	options.numTeamFlags[col] = 1;
+      }
     }
   }
 
@@ -1159,17 +1316,15 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
   if (options.numExtraFlags > 0) {
     // now count how many aren't disallowed
     for (FlagTypeMap::iterator it = FlagType::getFlagMap().begin();
-	 it != FlagType::getFlagMap().end(); ++it)
-      if (!options.flagDisallowed[it->second])
+	 it != FlagType::getFlagMap().end(); ++it) {
+      if (!options.flagDisallowed[it->second]) {
 	options.numAllowedFlags++;
-
+      }
+    }
     // if none allowed then no extra flags either
     if (options.numAllowedFlags == 0) {
       options.numExtraFlags = 0;
-    }
-
-    // otherwise make table of allowed flags
-    else {
+    } else {
       // types of extra flags allowed
       std::vector<FlagType*> allowedFlags;
       allowedFlags.clear();
@@ -1186,47 +1341,72 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
     }
   }
 
-  // allocate space for flags
+  const ZoneList& zl = entryZones.getZoneList();
+
+  // allocate space for extra flags
   numFlags = options.numExtraFlags;
+  // allocate space for team flags
   if (options.gameStyle & TeamFlagGameStyle) {
-    for (int col = RedTeam; col <= PurpleTeam; col++)
-      if (options.maxTeam[col] > 0)
+    for (int col = RedTeam; col <= PurpleTeam; col++) {
+      if (options.maxTeam[col] > 0) {
 	numFlags += options.numTeamFlags[col];
+      }
+    }
   }
+  // allocate space for normal flags
   for (FlagTypeMap::iterator it = FlagType::getFlagMap().begin();
        it != FlagType::getFlagMap().end(); ++it) {
     numFlags += options.flagCount[it->second];
   }
+  // allocate space for zone flags (including teams flags)
+  for (int z = 0; z < (int)zl.size(); z++) {
+    const CustomZone& zone = zl[z];
+    const ZoneFlagMap& zfm = zone.getZoneFlagMap();
+    ZoneFlagMap::const_iterator zfmIt;
+    for (zfmIt = zfm.begin(); zfmIt != zfm.end(); zfmIt++) {
+      if (forbidden.find(zfmIt->first) == forbidden.end()) {
+	numFlags += zfmIt->second;
+      }
+    }
+  }
 
   FlagInfo::setSize(numFlags);
 
+  // add team flags (ordered)
   int f = 0;
   if (options.gameStyle & TeamFlagGameStyle) {
     if (options.maxTeam[RedTeam] > 0) {
+      f = addZoneTeamFlags(f, Flags::RedTeam, entryZones, forbidden);
       for (int n = 0; n < options.numTeamFlags[RedTeam]; n++) {
 	FlagInfo::get(f++)->setRequiredFlag(Flags::RedTeam);
       }
     }
     if (options.maxTeam[GreenTeam] > 0) {
+      f = addZoneTeamFlags(f, Flags::GreenTeam, entryZones, forbidden);
       for (int n = 0; n < options.numTeamFlags[GreenTeam]; n++) {
 	FlagInfo::get(f++)->setRequiredFlag(Flags::GreenTeam);
       }
     }
     if (options.maxTeam[BlueTeam] > 0) {
+      f = addZoneTeamFlags(f, Flags::BlueTeam, entryZones, forbidden);
       for (int n = 0; n < options.numTeamFlags[BlueTeam]; n++) {
 	FlagInfo::get(f++)->setRequiredFlag(Flags::BlueTeam);
       }
     }
     if (options.maxTeam[PurpleTeam] > 0) {
+      f = addZoneTeamFlags(f, Flags::PurpleTeam, entryZones, forbidden);
       for (int n = 0; n < options.numTeamFlags[PurpleTeam]; n++) {
 	FlagInfo::get(f++)->setRequiredFlag(Flags::PurpleTeam);
       }
     }
   }
 
-
-  if (f < numFlags)
+  // super flags?
+  if (f < numFlags) {
     options.gameStyle |= int(SuperFlagGameStyle);
+  }
+
+  // add normal flags
   for (FlagTypeMap::iterator it2 = FlagType::getFlagMap().begin();
        it2 != FlagType::getFlagMap().end(); ++it2) {
     FlagType *fDesc = it2->second;
@@ -1237,12 +1417,37 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
       }
     }
   }
+  // add zone flags
+  for (int z = 0; z < (int)zl.size(); z++) {
+    const CustomZone& zone = zl[z];
+    const ZoneFlagMap& zfm = zone.getZoneFlagMap();
+    ZoneFlagMap::const_iterator zfmIt;
+    for (zfmIt = zfm.begin(); zfmIt != zfm.end(); zfmIt++) {
+      FlagType* ft = zfmIt->first;
+      if ((ft->flagTeam == ::NoTeam) && // no team flags here
+	  (forbidden.find(ft) == forbidden.end())) {
+	const int count = zfmIt->second;
+	for (int c = 0; c < count; c++) {
+	  entryZones.addZoneFlag(z, f);
+	  FlagInfo::get(f++)->setRequiredFlag(ft);
+	}
+      }
+    }
+  }
+  // add extra flags
   for (; f < numFlags; f++) {
     FlagInfo::get(f)->required = allFlagsOut;
   }
 
+  // sum the sources of team flags
+  if (options.gameStyle & TeamFlagGameStyle) {
+    for (int col = RedTeam; col <= PurpleTeam; col++) {
+      options.numTeamFlags[col] += zoneTeamFlagCounts[col];
+    }
+  }
+
+
   // debugging
-  // print style
   DEBUG1("style: %x\n", options.gameStyle);
   if (options.gameStyle & int(TeamFlagGameStyle))
     DEBUG1("  capture the flag\n");
@@ -1252,8 +1457,6 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
     DEBUG1("  super flags allowed\n");
   if (options.gameStyle & int(JumpingGameStyle))
     DEBUG1("  jumping allowed\n");
-  if (options.gameStyle & int(InertiaGameStyle))
-    DEBUG1("  inertia: %f, %f\n", options.linearAcceleration, options.angularAcceleration);
   if (options.gameStyle & int(RicochetGameStyle))
     DEBUG1("  all shots ricochet\n");
   if (options.gameStyle & int(ShakableGameStyle))
@@ -1261,6 +1464,23 @@ void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile)
 	   0.1f * float(options.shakeTimeout), options.shakeWins);
   if (options.gameStyle & int(AntidoteGameStyle))
     DEBUG1("  antidote flags\n");
+
+  return;
+}
+
+
+// simple syntax check of comma-seperated list of group names (returns true if error)
+bool checkCommaList (const char *list, int maxlen){
+  int x = strlen (list);
+  unsigned char c;
+  if (x > maxlen)
+    return true;
+  if (*list==',' || list[x-1]==',')
+    return true;
+  while ((c=*list++) != '\0')
+    if (c<' ' || c>'z' ||  c=='\'' || c=='"')
+      return true;
+  return false;
 }
 
 
