@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
@@ -25,14 +25,20 @@ CustomBase::CustomBase()
   rotation = 0.0f;
   size[0] = size[1] = BZDB.eval(StateDatabase::BZDB_BASESIZE);
   color = 0;
+
+  triggerWorldWep = false;
+  worldWepType = "SW";
 }
 
 
 bool CustomBase::read(const char *cmd, std::istream& input) {
   if (strcmp(cmd, "color") == 0) {
     input >> color;
-    if ((color < 0) || (color >= CtfTeams))
+    if ((color <= 0) || (color >= CtfTeams))
       return false;
+  } else if (strcmp(cmd, "oncap") == 0) {
+    triggerWorldWep = true;
+    input >> worldWepType;
   } else {
     if (!WorldFileObstacle::read(cmd, input))
       return false;
@@ -46,6 +52,9 @@ void CustomBase::writeToGroupDef(GroupDefinition *groupdef) const
   float absSize[3] = { fabsf(size[0]), fabsf(size[1]), fabsf(size[2]) };
   BaseBuilding* base = new BaseBuilding(pos, rotation, absSize, color);
   groupdef->addObstacle(base);
+
+  if (triggerWorldWep)
+    worldEventManager.addEvent(bz_eCaptureEvent,new WorldWeaponGlobalEventHandler(Flag::getDescFromAbbreviation(worldWepType.c_str()), pos, rotation, 0,(TeamColor)color));
 }
 
 // Local variables: ***
