@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 // TODO
@@ -18,6 +18,11 @@
 // - improve skipping
 
 
+// OOPS, get rid of these during the next protocol change
+static const int PACKET_SIZE_STUFFING = 8;
+static const int HEADER_SIZE_STUFFING = 0;
+
+
 // interface header
 #include "RecordReplay.h"
 
@@ -28,6 +33,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <time.h>
+#include <vector>
 #ifndef _WIN32
 #  include <sys/time.h>
 #  include <unistd.h>
@@ -45,7 +51,9 @@ typedef __int64 s64;
 
 // common headers
 #include "global.h"
+#include "bzglob.h"
 #include "Pack.h"
+#include "GameTime.h"
 #include "StateDatabase.h"
 #include "DirectoryNames.h"
 #include "NetHandler.h"
@@ -54,9 +62,7 @@ typedef __int64 s64;
 #include "version.h"
 
 // bzfs specific headers
-#include "CmdLineOptions.h"
-#include "GameKeeper.h"
-#include "FlagInfo.h"
+#include "bzfs.h"
 
 
 // Type Definitions
@@ -82,8 +88,11 @@ typedef struct RRpacket {
   RRtime timestamp;
   char *data;
 } RRpacket;
+//static const unsigned int RRpacketHdrSize =
+//  sizeof(RRpacket) - (2 * sizeof(RRpacket*) - sizeof(char*));
 static const unsigned int RRpacketHdrSize =
-  sizeof(RRpacket) - (2 * sizeof(RRpacket*) - sizeof(char*));
+  PACKET_SIZE_STUFFING +
+  (2 * sizeof(u16)) + (3 * sizeof(u32)) + sizeof(RRtime);
 
 typedef struct {
   u32 byteCount;
@@ -110,17 +119,30 @@ typedef struct {
   char *flags;		  // a list of the flags types
   char *world;		  // the world
 } ReplayHeader;
+//static const unsigned int ReplayHeaderSize =
+//  sizeof(ReplayHeader) - (2 * sizeof(char*));
 static const unsigned int ReplayHeaderSize =
-  sizeof(ReplayHeader) - (2 * sizeof(char*));
+  HEADER_SIZE_STUFFING +
+  (sizeof(u32) * 6) + sizeof(RRtime) +
+  CallSignLen + EmailLen + 8 + MessageLen + 64 + 4 + WorldSettingsSize;
+
+typedef struct {
+  std::string file;
+  float time;
+  int entryNum;
+} FileEntry;
 
 
 // Local Variables
 // ---------------
 
+#define REPLAY_LABEL "[REPLAY] "
+
 static const u32 ReplayMagic       = 0x7272425A; // "rrBZ"
 static const u32 ReplayVersion     = 0x0001;
 static const u32 DefaultMaxBytes   = (16 * 1024 * 1024); // 16 Mbytes
 static const u32 DefaultUpdateRate = (10 * 1000000); // seconds
+static const int MaxListOutput     = 100;
 
 static std::string RecordDir = getRecordDirName();
 
@@ -133,9 +155,14 @@ static u32 RecordMaxBytes = DefaultMaxBytes;
 static u32 RecordFileBytes = 0;
 static u32 RecordFilePackets = 0;
 static u32 RecordFilePrevPos = 0;
+static bool allowFileRecords = true;
+
 
 static bool Replaying = false;
 static bool ReplayMode = false;
+static bool ReplayLoop = false;
+static RRtime ReplayFileTime = 0;
+static RRtime ReplayStartTime = 0;
 static RRtime ReplayOffset = 0;
 static long ReplayFileStart = 0;
 static RRpacket *ReplayPos = NULL;
@@ -146,7 +173,10 @@ static RRbuffer ReplayBuf = {0, 0, NULL, NULL}; // for replaying
 static RRbuffer RecordBuf = {0, 0, NULL, NULL};  // for recording
 
 static FILE *ReplayFile = NULL;
+static std::string ReplayFilename = "";
+
 static FILE *RecordFile = NULL;
+static std::string RecordFilename = "";
 
 
 // Local Function Prototypes
@@ -158,11 +188,14 @@ static bool saveFlagsState();
 static bool saveRabbitState();
 static bool savePlayersState();
 static bool saveVariablesState();
+static bool saveGameTimeState();
 static bool resetStates();
 
 static bool setVariables(void *data);
 static bool preloadVariables();
 
+static void rewind();
+
 // saves straight to a file, or into the buffer
 static bool routePacket(u16 code, int len, const void *data, u16 mode);
 
@@ -186,6 +219,7 @@ static bool replaceWorldDatabase(ReplayHeader *h);
 static bool flagIsActive(FlagType *type);
 static bool packFlagTypes(char *flags, u32 *flagsSize);
 
+static bool getFileList(int playerIndex, std::vector<FileEntry>& entries);
 static FILE *openFile(const char *filename, const char *mode);
 static FILE *openWriteFile(int playerIndex, const char *filename);
 static bool badFilename(const char *name);
@@ -205,30 +239,12 @@ static RRtime getRRtime();
 static void *nboPackRRtime(void *buf, RRtime value);
 static void *nboUnpackRRtime(void *buf, RRtime& value);
 
+static bool checkReplayMode(int playerIndex);
+
 static const char *msgString(u16 code);
 
 
-// External Dependencies   (from bzfs.cxx)
-// ---------------------
-
-extern char hexDigest[50];
-extern int numFlags;
-extern u16 curMaxPlayers;
-extern TeamInfo team[NumTeams];
-extern char *worldDatabase;
-extern u32 worldDatabaseSize;
-extern char worldSettings[4 + WorldSettingsSize];
-extern uint8_t rabbitIndex;
-extern CmdLineOptions *clOptions;
-
-extern char *getDirectMessageBuffer(void);
-extern void directMessage(int playerIndex, u16 code,
-			  int len, const void *msg);
-extern void sendMessage(int playerIndex, PlayerId targetPlayer,
-			const char *message);
-
-
-/****************************************************************************/
+/******************************************************************************/
 
 // Record Functions
 
@@ -243,6 +259,7 @@ static bool recordReset()
     fclose(RecordFile);
     RecordFile = NULL;
   }
+  RecordFilename = "";
   freeBuffer(&RecordBuf);
 
   Recording = false;
@@ -285,7 +302,7 @@ bool Record::start(int playerIndex)
   }
   Recording = true;
   saveStates();
-  sendMessage(ServerPlayer, playerIndex, "Record started");
+  sendMessage(ServerPlayer, playerIndex, "Recording started");
 
   return true;
 }
@@ -298,17 +315,20 @@ bool Record::stop(int playerIndex)
     return false;
   }
 
-  sendMessage(ServerPlayer, playerIndex, "Record stopped");
+  sendMessage(ServerPlayer, playerIndex, "Recording stopped");
 
-  Recording = false;
-  if (RecordMode == StraightToFile) {
-    recordReset();
-  }
+  recordReset();
 
   return true;
 }
 
 
+const char* Record::getDirectory()
+{
+  return RecordDir.c_str();
+}
+
+
 bool Record::setDirectory(const char *dirname)
 {
   int len = strlen(dirname);
@@ -358,33 +378,35 @@ bool Record::setRate(int playerIndex, int seconds)
 
 bool Record::sendStats(int playerIndex)
 {
+  if (!Recording) {
+    sendMessage(ServerPlayer, playerIndex, "Not Recording");
+    return false;
+  }
+
   char buffer[MessageLen];
   float saveTime = 0.0f;
 
-  if (Recording) {
-    sendMessage(ServerPlayer, playerIndex, "Recording enabled");
-  }
-  else {
-    sendMessage(ServerPlayer, playerIndex, "Recording disabled");
-  }
-
   if (RecordMode == BufferedRecord) {
     if ((RecordBuf.tail != NULL) && (RecordBuf.head != NULL)) {
       RRtime diff = RecordBuf.head->timestamp - RecordBuf.tail->timestamp;
       saveTime = (float)diff / 1000000.0f;
     }
     snprintf(buffer, MessageLen,
-	      "  buffered: %i bytes / %i packets / %.1f seconds",
+	      "Buffered:  %i bytes / %i packets / %.1f seconds",
 	      RecordBuf.byteCount, RecordBuf.packetCount, saveTime);
+    sendMessage(ServerPlayer, playerIndex, buffer);
   }
   else {
     RRtime diff = getRRtime() - RecordStartTime;
     saveTime = (float)diff / 1000000.0f;
     snprintf(buffer, MessageLen,
-	      "  saved: %i bytes / %i packets / %.1f seconds",
+	      "Filename:  %s", RecordFilename.c_str());
+    sendMessage(ServerPlayer, playerIndex, buffer);
+    snprintf(buffer, MessageLen,
+	      "   saved:  %i bytes / %i packets / %.1f seconds",
 	      RecordFileBytes, RecordFilePackets, saveTime);
+    sendMessage(ServerPlayer, playerIndex, buffer);
   }
-  sendMessage(ServerPlayer, playerIndex, buffer);
 
   return true;
 }
@@ -401,6 +423,12 @@ bool Record::saveFile(int playerIndex, const char *filename)
     return false;
   }
 
+  if (!allowFileRecords) {
+    sendMessage(ServerPlayer, playerIndex,
+		"This server does not allow recording straight to a file");
+    return false;
+  }
+
   if (badFilename(filename)) {
     sendMessage(ServerPlayer, playerIndex,
 		 "Files must be with the local directory");
@@ -438,6 +466,8 @@ bool Record::saveFile(int playerIndex, const char *filename)
   snprintf(buffer, MessageLen, "Recording to file: %s", name.c_str());
   sendMessage(ServerPlayer, playerIndex, buffer);
 
+  RecordFilename = name.c_str();
+
   return true;
 }
 
@@ -577,7 +607,6 @@ static bool routePacket(u16 code, int len, const void * data, u16 mode)
 	    (int)p->mode, p->len, msgString(p->code), p->data);
 
     if (RecordBuf.byteCount > RecordMaxBytes) {
-      RRpacket *p;
       DEBUG4 ("routePacket: deleting until State Update\n");
       while (((p = delTailPacket(&RecordBuf)) != NULL) &&
 	     (p->mode != UpdatePacket)) {
@@ -599,6 +628,19 @@ static bool routePacket(u16 code, int len, const void * data, u16 mode)
 }
 
 
+void Record::setAllowFileRecs(bool value)
+{
+  allowFileRecords = value;
+  return;
+}
+
+
+bool Record::getAllowFileRecs()
+{
+  return allowFileRecords;
+}
+
+
 bool Record::enabled()
 {
   return Recording;
@@ -613,26 +655,40 @@ void Record::sendHelp(int playerIndex)
   sendMessage(ServerPlayer, playerIndex, "  /record size <Mbytes>");
   sendMessage(ServerPlayer, playerIndex, "  /record rate <seconds>");
   sendMessage(ServerPlayer, playerIndex, "  /record stats");
-  sendMessage(ServerPlayer, playerIndex, "  /record list");
   sendMessage(ServerPlayer, playerIndex, "  /record save <filename> [seconds]");
   sendMessage(ServerPlayer, playerIndex, "  /record file <filename>");
+  sendMessage(ServerPlayer, playerIndex, "  /record list [-t | -n | --] [pattern]");
   return;
 }
 
-/****************************************************************************/
+/******************************************************************************/
 
 // Replay Functions
 
+static bool checkReplayMode(int playerIndex)
+{
+  if (!ReplayMode) {
+    sendMessage(ServerPlayer, playerIndex,
+		"Server is not in replay mode. "
+		"Restart server with '-replay' option to enable playback.");
+    return false;
+  }
+  return true;
+}
+
+
 static bool replayReset()
 {
   if (ReplayFile != NULL) {
     fclose(ReplayFile);
     ReplayFile = NULL;
   }
+  ReplayFilename = "";
   freeBuffer(&ReplayBuf);
 
   ReplayMode = true;
   Replaying = false;
+  ReplayLoop = false;
   ReplayOffset = 0;
   ReplayFileStart = 0;
   ReplayPos = NULL;
@@ -691,23 +747,39 @@ static bool preloadVariables()
 
 bool Replay::loadFile(int playerIndex, const char *filename)
 {
+  if (!checkReplayMode(playerIndex)) {
+    return false;
+  }
+
+  std::string indexname;
+  if (filename[0] == '#') {
+    std::vector<FileEntry> entries;
+    if (!getFileList(playerIndex, entries)) {
+      sendMessage(ServerPlayer, playerIndex, "Could not get file index list");
+      return false;
+    }
+    int index = atoi(filename + 1);
+    if ((index < 1) || (index > (int)entries.size())) {
+      sendMessage(ServerPlayer, playerIndex, "Invalid file index");
+      return false;
+    }
+    indexname = entries[index - 1].file;
+    filename = indexname.c_str();    // new filename
+  }
+  else {
+    if (badFilename(filename)) {
+      sendMessage(ServerPlayer, playerIndex,
+		  "Files must be in the recordings directory");
+      return false;
+    }
+  }
+
   ReplayHeader header;
   RRpacket *p;
   char buffer[MessageLen];
   std::string name = RecordDir;
   name += filename;
 
-  if (!ReplayMode) {
-    sendMessage(ServerPlayer, playerIndex, "Server is not in replay mode");
-    return false;
-  }
-
-  if (badFilename(filename)) {
-    sendMessage(ServerPlayer, playerIndex,
-		 "Files must be in the recordings directory");
-    return false;
-  }
-
   replayReset();
   resetStates();
 
@@ -753,28 +825,40 @@ bool Replay::loadFile(int playerIndex, const char *filename)
   }
 
   ReplayPos = ReplayBuf.tail; // setup the initial position
+  ReplayFileTime = header.filetime;
+  ReplayStartTime = ReplayPos->timestamp;
 
   if (!preloadVariables()) {
     snprintf(buffer, MessageLen, "Could not preload variables: %s",
-	      name.c_str());
+	     name.c_str());
     sendMessage(ServerPlayer, playerIndex, buffer);
     replayReset();
     return false;
   }
 
-  snprintf(buffer, MessageLen, "Loaded file: %s", name.c_str());
+  ReplayFilename = filename;
+
+  snprintf(buffer, MessageLen, "Loaded file:  %s", name.c_str());
   sendMessage(ServerPlayer, playerIndex, buffer);
-  snprintf(buffer, MessageLen, "  author:    %s (%s)",
+  snprintf(buffer, MessageLen, "  author:     %s (%s)",
 	    header.callSign, header.email);
   sendMessage(ServerPlayer, playerIndex, buffer);
-  snprintf(buffer, MessageLen, "  protocol:  %.8s", header.serverVersion);
+  snprintf(buffer, MessageLen, "  protocol:   %.8s", header.serverVersion);
   sendMessage(ServerPlayer, playerIndex, buffer);
-  snprintf(buffer, MessageLen, "  server:    %s", header.appVersion);
+  snprintf(buffer, MessageLen, "  server:     %s", header.appVersion);
   sendMessage(ServerPlayer, playerIndex, buffer);
-  snprintf(buffer, MessageLen, "  seconds:   %.1f",
+  snprintf(buffer, MessageLen, "  seconds:    %.1f",
 	    (float)header.filetime/1000000.0f);
   sendMessage(ServerPlayer, playerIndex, buffer);
 
+  time_t startTime = (time_t)(ReplayPos->timestamp / 1000000);
+  snprintf(buffer, MessageLen, "  start:      %s", ctime(&startTime));
+  sendMessage(ServerPlayer, playerIndex, buffer);
+
+  time_t endTime =
+    (time_t)((header.filetime + ReplayPos->timestamp) / 1000000);
+  snprintf(buffer, MessageLen, "  end:	%s", ctime(&endTime));
+  sendMessage(ServerPlayer, playerIndex, buffer);
 
   return true;
 }
@@ -805,13 +889,10 @@ static FILE *getRecordFile(const char *filename)
 }
 
 
-bool Replay::sendFileList(int playerIndex)
+static bool getFileList(int playerIndex, std::vector<FileEntry>& entries)
 {
-  int count = 0;
-  char buffer[MessageLen];
-
-  snprintf(buffer, MessageLen, "dir:   %s",RecordDir.c_str());
-  sendMessage(ServerPlayer, playerIndex, buffer);
+  entries.clear();
+  int entNum = 0;
 
 #ifndef _MSC_VER
 
@@ -834,10 +915,11 @@ bool Replay::sendFileList(int playerIndex)
     if (file != NULL) {
       RRtime filetime;
       if (loadFileTime(&filetime, file)) {
-	snprintf(buffer, MessageLen, "file:  %-20s  [%9.1f seconds]",
-		  de->d_name, (float)filetime/1000000.0f);
-	sendMessage(ServerPlayer, playerIndex, buffer);
-	count++;
+	FileEntry entry;
+	entry.file = de->d_name;
+	entry.time = (float)filetime / 1000000.0f;
+	entry.entryNum = entNum++;
+	entries.push_back(entry);
       }
       fclose(file);
     }
@@ -863,10 +945,11 @@ bool Replay::sendFileList(int playerIndex)
       if (file != NULL) {
 	RRtime filetime;
 	if (loadFileTime(&filetime, file)) {
-	  snprintf(buffer, MessageLen, "file:  %-20s  [%9.1f seconds]",
-		    findData.cFileName, (float)filetime/1000000.0f);
-	  sendMessage(ServerPlayer, playerIndex, buffer);
-	  count++;
+	  FileEntry entry;
+	  entry.file = findData.cFileName;
+	  entry.time = (float)filetime / 1000000.0f;
+	  entry.entryNum = entNum++;
+		entries.push_back(entry);
 	}
 	fclose(file);
       }
@@ -877,8 +960,106 @@ bool Replay::sendFileList(int playerIndex)
 
 #endif // _MSC_VER
 
-  if (count == 0) {
-    sendMessage(ServerPlayer, playerIndex, "*** no record files found ***");
+  return true;
+}
+
+
+static bool sortFileTime (const FileEntry& a, const FileEntry& b) {
+  return (a.time < b.time);
+}
+
+static bool sortFileName (const FileEntry& a, const FileEntry& b) {
+  return (a.file < b.file);
+}
+
+
+static bool parseListOptions(const char* opts,
+			     int& sortMode, std::string& pattern)
+{
+  // defaults
+  pattern = "*";
+  sortMode = Replay::SortNone;
+
+  // parse the opts
+  while (opts[0] != '\0') {
+    while ((opts[0] != '\0') && isspace(opts[0])) {
+      opts++; // eat whitespace
+    }
+    if (opts[0] == '-') {
+      if (opts[1] == '-') {
+	opts += 2;
+	break; // end of options
+      }
+      else if (opts[1] == 't') {
+	sortMode = Replay::SortByTime;
+	opts += 2;
+      }
+      else if (opts[1] == 'n') {
+	sortMode = Replay::SortByName;
+	opts += 2;
+      }
+      else {
+	return false; // unknown option
+      }
+    } else {
+      break;
+    }
+  }
+
+  while ((opts[0] != '\0') && isspace(opts[0])) {
+    opts++; // eat whitespace
+  }
+
+  // setup the globbing pattern
+  if (opts[0] != '\0') {
+    pattern = opts;
+    if (strstr(pattern.c_str(), "*") == NULL) {
+      pattern = "*" + pattern + "*";
+    }
+  }
+
+  return true;
+}
+
+
+bool Replay::sendFileList(int playerIndex, const char* options)
+{
+  int sortMode;
+  std::string pattern;
+  if (!parseListOptions(options, sortMode, pattern)) {
+    return false;
+  }
+
+  std::vector<FileEntry> entries;
+  if (!getFileList(playerIndex, entries)) {
+    return false;
+  }
+
+  char buffer[MessageLen];
+  snprintf(buffer, MessageLen, "dir:  %s",RecordDir.c_str());
+  sendMessage(ServerPlayer, playerIndex, buffer);
+
+  if (sortMode == SortByTime) {
+    std::sort (entries.begin(), entries.end(), sortFileTime);
+  } else if (sortMode == SortByName) {
+    std::sort (entries.begin(), entries.end(), sortFileName);
+  }
+
+  int entriesSent = 0;
+  for (unsigned int i = 0; i < entries.size(); i++) {
+    const FileEntry& entry = entries[i];
+    if (glob_match(pattern, entry.file)) {
+      entriesSent++;
+      snprintf(buffer, MessageLen, "#%02i:  %-30s  [%9.1f seconds]",
+	       entry.entryNum + 1, entry.file.c_str(), entry.time);
+      sendMessage(ServerPlayer, playerIndex, buffer);
+      if (entriesSent >= MaxListOutput) {
+	snprintf(buffer, MessageLen, "Not listing more then %i entries, "
+				     "try using pattern matching.", MaxListOutput);
+	sendMessage(ServerPlayer, playerIndex, buffer);
+	break;
+      }
+    }
   }
 
   return true;
@@ -887,8 +1068,7 @@ bool Replay::sendFileList(int playerIndex)
 
 bool Replay::play(int playerIndex)
 {
-  if (!ReplayMode) {
-    sendMessage(ServerPlayer, playerIndex, "Server is not in replay mode");
+  if (!checkReplayMode(playerIndex)) {
     return false;
   }
 
@@ -898,6 +1078,7 @@ bool Replay::play(int playerIndex)
   }
 
   Replaying = true;
+  ReplayLoop = false;
 
   if (ReplayPos != NULL) {
     ReplayOffset = getRRtime() - ReplayPos->timestamp;
@@ -911,7 +1092,73 @@ bool Replay::play(int playerIndex)
   // reset the replay observers' view of state
   resetStates();
 
-  sendMessage(ServerPlayer, playerIndex, "Starting replay");
+  sendMessage(playerIndex, AllPlayers, REPLAY_LABEL "starting replay");
+
+  return true;
+}
+
+
+bool Replay::loop(int playerIndex)
+{
+  if (!checkReplayMode(playerIndex)) {
+    return false;
+  }
+
+  if (ReplayFile == NULL) {
+    sendMessage(ServerPlayer, playerIndex, "No replay file loaded");
+    return false;
+  }
+
+  Replaying = true;
+  ReplayLoop = true;
+
+  if (ReplayPos != NULL) {
+    ReplayOffset = getRRtime() - ReplayPos->timestamp;
+  }
+  else {
+    sendMessage(ServerPlayer, playerIndex, "Internal replay error");
+    replayReset();
+    return false;
+  }
+
+  // reset the replay observers' view of state
+  resetStates();
+
+  sendMessage(playerIndex, AllPlayers, REPLAY_LABEL "starting replay loop");
+
+  return true;
+}
+
+
+bool Replay::sendStats(int playerIndex)
+{
+  if (!checkReplayMode(playerIndex)) {
+    return false;
+  }
+
+  if (ReplayFile == NULL) {
+    sendMessage(ServerPlayer, playerIndex, "No replay file loaded");
+    return false;
+  }
+
+  if (ReplayPos == NULL) {
+    sendMessage(ServerPlayer, playerIndex, "Internal replay error");
+    return false;
+  }
+
+  char buffer[MessageLen];
+  snprintf(buffer, MessageLen,
+	   "Replay File:  %s", ReplayFilename.c_str());
+  sendMessage(ServerPlayer, playerIndex, buffer);
+
+  time_t replayTime = (time_t)(ReplayPos->timestamp / 1000000);
+  float fullTime = (float)ReplayFileTime / 1000000.0f;
+  float usedTime =(float)(ReplayPos->timestamp - ReplayStartTime) / 1000000.0f;
+  float percent = 100.0f * (usedTime / fullTime);
+  snprintf(buffer, MessageLen,
+	   "Replay Date:  %s [%.2f %%]  (%.1f secs / %.1f secs)",
+	   ctime(&replayTime), percent, usedTime, fullTime);
+  sendMessage(ServerPlayer, playerIndex, buffer);
 
   return true;
 }
@@ -919,8 +1166,7 @@ bool Replay::play(int playerIndex)
 
 bool Replay::skip(int playerIndex, int seconds)
 {
-  if (!ReplayMode) {
-    sendMessage(ServerPlayer, playerIndex, "Server is not in replay mode");
+  if (!checkReplayMode(playerIndex)) {
     return false;
   }
 
@@ -946,7 +1192,7 @@ bool Replay::skip(int playerIndex, int seconds)
 	p = nextStatePacket();
       } while ((p != NULL) && (p->timestamp < target));
       if (p == NULL) {
-	sendMessage(ServerPlayer, playerIndex, "skipped to the end");
+	sendMessage(playerIndex, AllPlayers, REPLAY_LABEL "skipped to the end");
       }
     }
     else {
@@ -954,7 +1200,7 @@ bool Replay::skip(int playerIndex, int seconds)
 	p = prevStatePacket();
       } while ((p != NULL) && (p->timestamp > target));
       if (p == NULL) {
-	sendMessage(ServerPlayer, playerIndex, "skipped to the beginning");
+	sendMessage(playerIndex, AllPlayers, REPLAY_LABEL "skipped to the beginning");
       }
     }
 
@@ -968,9 +1214,9 @@ bool Replay::skip(int playerIndex, int seconds)
   // print the amount of time skipped
   RRtime diff = ReplayPos->timestamp - nowtime;
   char buffer[MessageLen];
-  snprintf(buffer, MessageLen, "Skipping %.1f seconds (asked %i)",
+  snprintf(buffer, MessageLen, REPLAY_LABEL "skipped %.1f seconds (asked %i)",
 	   (float)diff/1000000.0f, seconds);
-  sendMessage(ServerPlayer, playerIndex, buffer);
+  sendMessage(playerIndex, AllPlayers, buffer);
 
   return true;
 }
@@ -980,7 +1226,7 @@ bool Replay::pause(int playerIndex)
 {
   if ((ReplayFile != NULL) && (ReplayPos != NULL) && Replaying) {
     Replaying = false;
-    sendMessage(ServerPlayer, playerIndex, "Replay paused");
+    sendMessage(playerIndex, AllPlayers, REPLAY_LABEL "paused");
   }
   else {
     sendMessage(ServerPlayer, playerIndex, "Can't pause, not playing");
@@ -1069,11 +1315,19 @@ bool Replay::sendPackets()
   } // while loop
 
   if (p == NULL) {
-    resetStates();
-    Replaying = false;
-    ReplayPos = ReplayBuf.tail;
-    sendMessage(ServerPlayer, AllPlayers, "Replay Finished");
-    return false;
+    if (ReplayLoop) {
+      rewind();
+      resetStates();
+      sendMessage(ServerPlayer, AllPlayers, "Replay Looping");
+      return true;
+    }
+    else {
+      resetStates();
+      Replaying = false;
+      ReplayPos = ReplayBuf.tail;
+      sendMessage(ServerPlayer, AllPlayers, "Replay Finished");
+      return false;
+    }
   }
 
   if (sent && (ReplayPos->prev != NULL)) {
@@ -1116,14 +1370,28 @@ bool Replay::playing()
 void Replay::sendHelp(int playerIndex)
 {
   sendMessage(ServerPlayer, playerIndex, "usage:");
-  sendMessage(ServerPlayer, playerIndex, "  /replay list");
-  sendMessage(ServerPlayer, playerIndex, "  /replay load <filename>");
+  sendMessage(ServerPlayer, playerIndex, "  /replay list [-t | -n | --] [pattern]");
+  sendMessage(ServerPlayer, playerIndex, "  /replay load <filename|#index>");
   sendMessage(ServerPlayer, playerIndex, "  /replay play");
+  sendMessage(ServerPlayer, playerIndex, "  /replay loop");
   sendMessage(ServerPlayer, playerIndex, "  /replay skip [+/-seconds]");
+  sendMessage(ServerPlayer, playerIndex, "  /replay stats");
   return;
 }
 
 
+static void rewind()
+{
+  RRpacket* p;
+  do {
+    p = prevStatePacket();
+  } while (p != NULL);
+
+  // setup the new time offset
+  ReplayOffset = getRRtime() - ReplayPos->timestamp;
+}
+
+
 static bool setVariables(void *data)
 {
   // copied this function from [playing.cxx]
@@ -1265,7 +1533,7 @@ static RRpacket *prevStatePacket()
 }
 
 
-/****************************************************************************/
+/******************************************************************************/
 
 // State Management Functions
 
@@ -1284,6 +1552,7 @@ static bool saveStates()
   saveFlagsState();
   savePlayersState();
   saveRabbitState();
+  saveGameTimeState();
 
   RecordUpdateTime = getRRtime();
 
@@ -1323,7 +1592,7 @@ static bool saveFlagsState()
 
   for (flagIndex = 0; flagIndex < numFlags; flagIndex++) {
     FlagInfo &flag = *FlagInfo::get(flagIndex);
-    if (flag.flag.status != FlagNoExist) {
+    if (flag.exist()) {
       if ((length + sizeof(u16) + FlagPLen) > MaxPacketLen - 2*sizeof(u16)) {
 	// packet length overflow
 	nboPackUShort(bufStart, cnt);
@@ -1487,6 +1756,23 @@ static bool saveVariablesState()
 }
 
 
+static bool saveGameTimeState()
+{
+  // FIXME: the packets that are sent out during replay will not
+  //	be properly lag compensated for the connections over
+  //	which they are sent.
+  //	- provide GameTime with an offset capability
+  //	- allow resetting of players' next GameTime update
+  //	  (and do so during a replay state update)
+  //	- send GameTime packets out regardless of replay state
+  char buffer[MaxPacketLen];
+  void* buf = GameTime::pack(buffer, 0.150f);
+  int length = (char*)buf - buffer;
+  routePacket(MsgGameTime, length, buffer, RealPacket);
+  return true;
+}
+
+
 static bool resetStates()
 {
   int i;
@@ -1533,7 +1819,7 @@ static bool resetStates()
 }
 
 
-/****************************************************************************/
+/******************************************************************************/
 
 // File Functions
 
@@ -1735,16 +2021,24 @@ static bool saveHeader(int p, RRtime filetime, FILE *f)
     return false;
   }
 
-  GameKeeper::Player *gkPlayer = GameKeeper::Player::getPlayerByIndex(p);
-  if (gkPlayer == NULL) {
-    return false;
+  // player callsign and email
+  const char* callsign = "SERVER";
+  const char* email = "";
+  if (p != ServerPlayer) {
+    GameKeeper::Player *gkPlayer = GameKeeper::Player::getPlayerByIndex(p);
+    if (gkPlayer == NULL) {
+      return false;
+    } else {
+      PlayerInfo *pi = &gkPlayer->player;
+      callsign = pi->getCallSign();
+      email = pi->getEMail();
+    }
   }
-  PlayerInfo *pi = &gkPlayer->player;
 
   // setup the data
   memset(&hdr, 0, sizeof(hdr));
-  strncpy(hdr.callSign, pi->getCallSign(), sizeof(hdr.callSign));
-  strncpy(hdr.email, pi->getEMail(), sizeof(hdr.email));
+  strncpy(hdr.callSign, callsign, sizeof(hdr.callSign));
+  strncpy(hdr.email, email, sizeof(hdr.email));
   strncpy(hdr.serverVersion, getServerVersion(), sizeof(hdr.serverVersion));
   strncpy(hdr.appVersion, getAppVersion(), sizeof(hdr.appVersion));
   strncpy(hdr.realHash, hexDigest, sizeof(hdr.realHash));
@@ -2073,7 +2367,7 @@ static bool packFlagTypes(char *flags, u32 *flagsSize)
 }
 
 
-/****************************************************************************/
+/******************************************************************************/
 
 // Buffer Functions
 
@@ -2214,7 +2508,7 @@ static void freeBuffer(RRbuffer *b)
 }
 
 
-/****************************************************************************/
+/******************************************************************************/
 
 // Timing Functions
 
@@ -2267,7 +2561,7 @@ static void *nboUnpackRRtime(void *buf, RRtime& value)
 }
 
 
-/****************************************************************************/
+/******************************************************************************/
 
 static const char *msgString(u16 code)
 {
@@ -2329,7 +2623,7 @@ static const char *msgString(u16 code)
 }
 
 
-/****************************************************************************/
+/******************************************************************************/
 
 // Local Variables: ***
 // mode: C++ ***
