@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifdef _MSC_VER
@@ -17,6 +17,7 @@
 // class-interface header
 #include "WorldWeapons.h"
 
+#include "WorldInfo.h"
 // system headers
 #include <vector>
 
@@ -26,11 +27,47 @@
 #include "Protocol.h"
 #include "Address.h"
 #include "StateDatabase.h"
+#include "bzfsAPI.h"
+
+// bzfs specific headers
+#include "bzfs.h"
 
 char *getDirectMessageBuffer();
 void broadcastMessage(uint16_t code, int len, const void *msg);
 
 
+static int fireWorldWepReal(FlagType* type, float lifetime, PlayerId player,
+			    TeamColor teamColor, float *pos, float tilt, float dir,
+			    int shotID, float dt)
+{
+  void *buf, *bufStart = getDirectMessageBuffer();
+
+  FiringInfo firingInfo;
+  firingInfo.timeSent = (float)TimeKeeper::getCurrent().getSeconds();
+  firingInfo.flagType = type;
+  firingInfo.lifetime = lifetime;
+  firingInfo.shot.player = player;
+  memmove(firingInfo.shot.pos, pos, 3 * sizeof(float));
+  const float shotSpeed = BZDB.eval(StateDatabase::BZDB_SHOTSPEED);
+  const float tiltFactor = cosf(tilt);
+  firingInfo.shot.vel[0] = shotSpeed * tiltFactor * cosf(dir);
+  firingInfo.shot.vel[1] = shotSpeed * tiltFactor * sinf(dir);
+  firingInfo.shot.vel[2] = shotSpeed * sinf(tilt);
+  firingInfo.shot.id = shotID;
+  firingInfo.shot.dt = dt;
+
+  firingInfo.shot.team = teamColor;
+
+  buf = firingInfo.pack(bufStart);
+
+  if (BZDB.isTrue(StateDatabase::BZDB_WEAPONS)) {
+    broadcastMessage(MsgShotBegin, (char *)buf - (char *)bufStart, bufStart);
+  }
+  return shotID;
+}
+
+
+
 WorldWeapons::WorldWeapons()
 : worldShotId(0)
 {
@@ -64,7 +101,7 @@ float WorldWeapons::nextTime ()
       nextShot = w->nextTime;
     }
   }
-  return (nextShot - TimeKeeper::getCurrent());
+  return (float)(nextShot - TimeKeeper::getCurrent());
 }
 
 
@@ -77,28 +114,9 @@ void WorldWeapons::fire()
     Weapon *w = *it;
     if (w->nextTime <= nowTime) {
 
-      void *buf, *bufStart = getDirectMessageBuffer();
-
-      FiringInfo firingInfo;
-      firingInfo.flagType = (FlagType*)w->type;
-      firingInfo.lifetime = BZDB.eval(StateDatabase::BZDB_RELOADTIME);
-      firingInfo.shot.player = ServerPlayer;
-      memmove(firingInfo.shot.pos, w->origin, 3 * sizeof(float));
-      float shotSpeed = BZDB.eval(StateDatabase::BZDB_SHOTSPEED);
-      const float tiltFactor = cosf(w->tilt);
-      firingInfo.shot.vel[0] = shotSpeed * tiltFactor * cosf(w->direction);
-      firingInfo.shot.vel[1] = shotSpeed * tiltFactor * sinf(w->direction);
-      firingInfo.shot.vel[2] = shotSpeed * sinf(w->tilt);
-      firingInfo.shot.id = worldShotId++;
-      if (worldShotId > 30) { // Maximum of 30 world shots
-	    worldShotId = 0;
-      }
-      firingInfo.shot.dt = 0;
-      buf = firingInfo.pack(bufStart);
-
-      if (BZDB.isTrue(StateDatabase::BZDB_WEAPONS)) {
-	broadcastMessage(MsgShotBegin, (char *)buf - (char *)bufStart, bufStart);
-      }
+      fireWorldWepReal((FlagType*)w->type, BZDB.eval(StateDatabase::BZDB_RELOADTIME),
+		       ServerPlayer, w->teamColor, w->origin, w->tilt, w->direction,
+		       getNewWorldShotID(), 0);
 
       //Set up timer for next shot, and eat any shots that have been missed
       while (w->nextTime <= nowTime) {
@@ -114,12 +132,13 @@ void WorldWeapons::fire()
 
 
 void WorldWeapons::add(const FlagType *type, const float *origin,
-                       float direction, float tilt,
+		       float direction, float tilt, TeamColor teamColor,
 		       float initdelay, const std::vector<float> &delay,
 		       TimeKeeper &sync)
 {
   Weapon *w = new Weapon();
   w->type = type;
+  w->teamColor = teamColor;
   memmove(&w->origin, origin, 3*sizeof(float));
   w->direction = direction;
   w->tilt = tilt;
@@ -180,6 +199,65 @@ int WorldWeapons::packSize(void) const
 }
 
 
+int WorldWeapons::getNewWorldShotID(void)
+{
+  if (worldShotId > _MAX_WORLD_SHOTS) {
+    worldShotId = 0;
+  }
+  return worldShotId++;
+}
+
+
+//----------WorldWeaponGlobalEventHandler---------------------
+// where we do the world weapon handling for event based shots since they are not really done by the "world"
+
+WorldWeaponGlobalEventHandler::WorldWeaponGlobalEventHandler(FlagType *_type,
+							     const float *_origin,
+							     float _direction,
+							     float _tilt,
+							     TeamColor teamColor )
+{
+  type = _type;
+  if (_origin)
+    memcpy(origin,_origin,sizeof(float)*3);
+  else
+    origin[0] = origin[1] = origin[2] = 0.0f;
+
+  direction = _direction;
+  tilt = _tilt;
+  team = convertTeam(teamColor);
+}
+
+WorldWeaponGlobalEventHandler::~WorldWeaponGlobalEventHandler()
+{
+}
+
+void WorldWeaponGlobalEventHandler::process (bz_EventData *eventData)
+{
+  if (!eventData || eventData->eventType != bz_eCaptureEvent)
+    return;
+
+  bz_CTFCaptureEventData *capEvent = (bz_CTFCaptureEventData*)eventData;
+
+  if ( capEvent->teamCapped != team )
+	  return;
+
+  fireWorldWepReal(type, BZDB.eval(StateDatabase::BZDB_RELOADTIME),
+		   ServerPlayer, RogueTeam, origin, tilt, direction,
+		   world->getWorldWeapons().getNewWorldShotID(),0);
+}
+
+
+// for bzfsAPI: it needs to be global
+int fireWorldWep(FlagType* type, float lifetime, PlayerId player,
+			float *pos, float tilt, float direction,
+			int shotID, float dt)
+{
+  return fireWorldWepReal(type, lifetime, player, RogueTeam,
+			  pos, tilt, direction, shotID, dt);
+}
+
+
 // Local Variables: ***
 // mode: C++ ***
 // tab-width: 8 ***
