@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* class interface header */
@@ -22,10 +22,10 @@
 
 /* protected */
 
-Language::Language(int number, std::string iso2, std::string iso3, std::string english, std::string french)
-  : _number(number),
-    _iso2(iso2),
-    _iso3(iso3),
+Language::Language(int numberCode, std::string iso2Code, std::string iso3Code, std::string english, std::string french)
+  : _number(numberCode),
+    _iso2(iso2Code),
+    _iso3(iso3Code),
     _english(english),
     _french(french)
 {
