@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 
@@ -166,19 +166,19 @@ void LinkManager::doLinking()
   if (debugLevel >= 4) {
     for (i = 0; i < teles.size(); i++) {
       Teleporter* tele = (Teleporter*) teles[i];
-      printf ("TELE(%i): %s\n", i, tele->getName().c_str());
+      DEBUG0("TELE(%i): %s\n", i, tele->getName().c_str());
     }
     for (i = 0; i < linkNames.size(); i++) {
       LinkNameSet& link = linkNames[i];
-      printf ("LINKSRC: %-32sLINKDST: %s\n", link.src.c_str(), link.dst.c_str());
+      DEBUG0("LINKSRC: %-32sLINKDST: %s\n", link.src.c_str(), link.dst.c_str());
     }
     for (i = 0; i < linkNumbers.size(); i++) {
-      printf ("SRC %3i%c:  DSTS", (i / 2), ((i % 2) == 0) ? 'f' : 'b');
+      DEBUG0("SRC %3i%c:  DSTS", (i / 2), ((i % 2) == 0) ? 'f' : 'b');
       for (unsigned int j = 0; j < linkNumbers[i].dsts.size(); j++) {
 	int dst = linkNumbers[i].dsts[j];
-	printf (" %i%c", (dst / 2), ((dst % 2) == 0) ? 'f' : 'b');
+	DEBUG0(" %i%c", (dst / 2), ((dst % 2) == 0) ? 'f' : 'b');
       }
-      printf ("\n");
+      DEBUG0("\n");
     }
   }
 
@@ -206,7 +206,7 @@ void LinkManager::findTelesByName(const std::string& name,
   }
 
   // make the trailing face specification case-independent
-  const unsigned int last = (const unsigned int)glob.size() - 1;
+  const unsigned int last = (unsigned int)glob.size() - 1;
   if ((glob[last] == 'F') || (glob[last] == 'B')) {
     glob[last] = tolower(glob[last]);
   }
