@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "common.h"
@@ -237,14 +237,6 @@ static void spin(float m[4][4], const float radians, const float normal[3])
 
 MeshTransform::Tool::Tool(const MeshTransform& xform)
 {
-  if (xform.transforms.size() > 0) {
-    empty = false;
-  } else {
-    empty = true;
-    inverted = false;
-    return;
-  }
-
   // load the identity matrices
   int i, j;
   for (i = 0; i < 4; i++) {
@@ -266,6 +258,15 @@ MeshTransform::Tool::Tool(const MeshTransform& xform)
     }
   }
 
+  skewed = false;
+  if (xform.transforms.size() > 0) {
+    empty = false;
+  } else {
+    empty = true;
+    inverted = false;
+    return;
+  }
+
   // setup the matrices
   processTransforms(xform.transforms);
 
@@ -320,10 +321,12 @@ void MeshTransform::Tool::processTransforms(
 	break;
       }
       case ScaleTransform: {
+	skewed = true;
 	scale(vertexMatrix, transform.data);
 	break;
       }
       case ShearTransform: {
+	skewed = true;
 	shear(vertexMatrix, transform.data);
 	break;
       }
@@ -675,15 +678,15 @@ int MeshTransform::packSize() const
 
 void MeshTransform::print(std::ostream& out, const std::string& indent) const
 {
-  out << "transform" << std::endl;
+  out << indent << "transform" << std::endl;
 
   if (name.size() > 0) {
-    out << "  name " << name << std::endl;
+    out << indent << "  name " << name << std::endl;
   }
 
   printTransforms(out, indent);
 
-  out << "end" << std::endl << std::endl;
+  out << indent << "end" << std::endl << std::endl;
 
   return;
 }
