@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
@@ -15,7 +15,9 @@
 
 /* system implementation headers */
 #include <errno.h>
+#include <stdio.h>
 #include <string>
+#include <ctype.h>
 
 /* implementation-specific common headers */
 #include "TextUtils.h"
@@ -23,6 +25,10 @@
 WordFilter PlayerInfo::serverSpoofingFilter;
 TimeKeeper PlayerInfo::now = TimeKeeper::getCurrent();
 
+bool	PlayerInfo::callSignFiltering = false;
+WordFilter *PlayerInfo::filterData	= NULL;
+bool	PlayerInfo::simpleFiltering   = true;
+
 PlayerInfo::PlayerInfo(int _playerIndex) :
   playerIndex(_playerIndex), state(PlayerInLimbo), flag(-1),
   spamWarns(0), lastMsgTime(now), paused(false),
@@ -33,6 +39,18 @@ PlayerInfo::PlayerInfo(int _playerIndex) :
   memset(callSign, 0, CallSignLen);
   memset(token, 0, TokenLen);
   memset(clientVersion, 0, VersionLen);
+  clientVersionMajor = -1;
+  clientVersionMinor = -1;
+  clientVersionRevision = -1;
+}
+
+void PlayerInfo::setFilterParameters(bool	_callSignFiltering,
+				     WordFilter &_filterData,
+				     bool	_simpleFiltering)
+{
+  callSignFiltering = _callSignFiltering;
+  filterData	= &_filterData;
+  simpleFiltering   = _simpleFiltering;
 }
 
 void PlayerInfo::resetPlayer(bool ctf) {
@@ -50,31 +68,31 @@ void PlayerInfo::resetPlayer(bool ctf) {
 
 void PlayerInfo::setRestartOnBase(bool on) {
   restartOnBase = on;
-};
+}
 
 bool PlayerInfo::shouldRestartAtBase() {
   return restartOnBase;
-};
+}
 
 void PlayerInfo::signingOn() {
   state = PlayerDead;
-};
+}
 
 void PlayerInfo::setAlive() {
   state = PlayerAlive;
   paused = false;
   flag = -1;
-};
+}
 
 void PlayerInfo::setDead() {
   state = PlayerDead;
-};
+}
 
 void *PlayerInfo::packUpdate(void *buf) {
   buf = nboPackUShort(buf, uint16_t(type));
   buf = nboPackUShort(buf, uint16_t(team));
   return buf;
-};
+}
 
 void *PlayerInfo::packId(void *buf) {
   buf = nboPackString(buf, callSign, CallSignLen);
@@ -95,10 +113,24 @@ bool PlayerInfo::unpackEnter(void *buf, uint16_t &rejectCode, char *rejectMsg)
   buf = nboUnpackString(buf, email, EmailLen);
   buf = nboUnpackString(buf, token, TokenLen);
   buf = nboUnpackString(buf, clientVersion, VersionLen);
+
+  // terminate the strings
+  callSign[CallSignLen - 1] = '\0';
+  email[EmailLen - 1] = '\0';
+  token[TokenLen - 1] = '\0';
+  clientVersion[VersionLen - 1] = '\0';
   cleanEMail();
 
   DEBUG2("Player %s [%d] sent version string: %s\n",
 	 callSign, playerIndex, clientVersion);
+  int major, minor, rev;
+  if (sscanf(clientVersion, "%d.%d.%d", &major, &minor, &rev) == 3) {
+    clientVersionMajor = major;
+    clientVersionMinor = minor;
+    clientVersionRevision = rev;
+  }
+  DEBUG4("Player %s version code parsed as:  %i.%i.%i\n", callSign,
+	 clientVersionMajor, clientVersionMinor, clientVersionRevision);
 
   // spoof filter holds "SERVER" for robust name comparisons
   if (serverSpoofingFilter.wordCount() == 0) {
@@ -123,21 +155,52 @@ bool PlayerInfo::unpackEnter(void *buf, uint16_t &rejectCode, char *rejectMsg)
     strcpy(rejectMsg, "The e-mail was rejected.  Try a different e-mail.");
     return false;
   }
+
+  // make sure the callsign is not obscene/filtered
+  if (callSignFiltering) {
+    DEBUG2("checking callsign: %s\n",callSign);
+
+    char cs[CallSignLen];
+    memcpy(cs, callSign, sizeof(char) * CallSignLen);
+    if (filterData->filter(cs, simpleFiltering)) {
+      rejectCode = RejectBadCallsign;
+      strcpy(rejectMsg,
+	     "The callsign was rejected. Try a different callsign.");
+      return false;
+    }
+  }
+
+  // make sure the email is not obscene/filtered
+  if (callSignFiltering) {
+    DEBUG2("checking email: %s\n", email);
+    char em[EmailLen];
+    memcpy(em, email, sizeof(char) * EmailLen);
+    if (filterData->filter(em, simpleFiltering)) {
+      rejectCode = RejectBadEmail;
+      strcpy(rejectMsg, "The e-mail was rejected. Try a different e-mail.");
+      return false;
+    }
+  }
+
+  if (token[0] == 0) {
+    strcpy(token, "NONE");
+  }
+
   return true;
-};
+}
 
 const char *PlayerInfo::getCallSign() const {
   return callSign;
-};
+}
 
 bool PlayerInfo::isCallSignReadable() {
   // callsign readability filter, make sure there are more alphanum than non
   // keep a count of alpha-numerics
 
   int callsignlen = (int)strlen(callSign);
-  // reject less than 3 characters
-  if (callsignlen < 3) {
-    errorString = "Callsigns must be at least 3 characters.";
+  // reject less than 2 characters
+  if (callsignlen < 2) {
+    errorString = "Callsigns must be at least 2 characters.";
     return false;
   }
 
@@ -147,6 +210,13 @@ bool PlayerInfo::isCallSignReadable() {
     return false;
   }
 
+  // prevent spoofing global login indicators + and @ in the scoreboard
+  // and reserve # for /kick or /ban #slot
+  if (*callSign=='+' || *callSign=='@' || *callSign=='#' ) {
+    errorString = "Callsigns are not allowed to start with +, @, or #.";
+    return false;
+  }
+
   // start with true to reject leading space
   bool lastWasSpace = true;
   int alnumCount = 0;
@@ -159,22 +229,21 @@ bool PlayerInfo::isCallSignReadable() {
     }
 
     // reject ' and " and any nonprintable
-    if ((*sp == '\'') || (*sp == '"') || ((unsigned)*sp > 0x7f)
-	|| !isprint(*sp)) {
+    if ((*sp == '\'') || (*sp == '"') || ((unsigned)*sp > 0x7f) || !isprint(*sp)) {
       errorString = "Non-printable characters and quotes are not allowed in callsigns.";
       return false;
     }
     if (isspace(*sp)) {
       // only space is valid, not tab etc.
       if (*sp != ' ') {
-	errorString = "Invalid whitespace in callsign.";
-	return false;
+	      errorString = "Invalid whitespace in callsign.";
+	      return false;
       }
       lastWasSpace = true;
     } else {
       lastWasSpace = false;
       if (isalnum(*sp))
-	alnumCount++;
+	      alnumCount++;
     }
   } while (*++sp);
 
@@ -182,11 +251,11 @@ bool PlayerInfo::isCallSignReadable() {
   if (!readable)
     errorString = "Callsign rejected. Please use mostly letters and numbers.";
   return readable;
-};
+}
 
 const char *PlayerInfo::getEMail() const {
   return email;
-};
+}
 
 void PlayerInfo::cleanEMail() {
   // strip leading whitespace from email
@@ -207,7 +276,7 @@ void PlayerInfo::cleanEMail() {
   while (isspace(*--tp)) {
     *tp=0;
   }
-};
+}
 
 bool PlayerInfo::isEMailReadable() {
   // email/"team" readability filter, make sure there are more
@@ -221,63 +290,71 @@ bool PlayerInfo::isEMailReadable() {
   } while (*++sp);
   int emaillen = (int)strlen(email);
   return (emaillen <= 4) || (((float)emailAlnumCount / (float)emaillen) > 0.5);
-};
+}
 
 const char *PlayerInfo::getToken() const {
   return token;
-};
+}
 
 void PlayerInfo::clearToken() {
   token[0] = '\0';
-};
+}
 
 void *PlayerInfo::packVirtualFlagCapture(void *buf) {
   buf = nboPackUShort(buf, uint16_t(int(team) - 1));
   buf = nboPackUShort(buf, uint16_t(1 + (int(team) % 4)));
   return buf;
-};
+}
 
 bool PlayerInfo::isTeam(TeamColor _team) const {
   return team == _team;
-};
+}
 
 bool PlayerInfo::isObserver() const {
   return team == ObserverTeam;
-};
+}
 
-TeamColor PlayerInfo::getTeam() {
+TeamColor PlayerInfo::getTeam() const {
   return team;
-};
+}
 
 void PlayerInfo::setTeam(TeamColor _team) {
   team = _team;
-};
+}
 
 void PlayerInfo::wasARabbit() {
   team = RogueTeam;
   wasRabbit = true;
-};
+}
 
 void PlayerInfo::wasNotARabbit() {
   wasRabbit = false;
-};
+}
 
 void PlayerInfo::resetFlag() {
   flag = -1;
   lastFlagDropTime = now;
-};
+}
 
 void PlayerInfo::setFlag(int _flag) {
   flag = _flag;
-};
+}
 
 bool PlayerInfo::isFlagTransitSafe() {
   return now - lastFlagDropTime >= 2.0f;
-};
+}
 
 const char *PlayerInfo::getClientVersion() {
   return clientVersion;
-};
+}
+
+void PlayerInfo::getClientVersionNumbers(int& major, int& minor, int& rev)
+{
+  major = clientVersionMajor;
+  minor = clientVersionMinor;
+  rev = clientVersionRevision;
+  return;
+}
 
 std::string PlayerInfo::getIdleStat() {
   std::string reply;
@@ -289,74 +366,70 @@ std::string PlayerInfo::getIdleStat() {
     }
   }
   return reply;
-};
+}
 
 bool PlayerInfo::canBeRabbit(bool relaxing) {
   if (paused || notResponding || (team == ObserverTeam))
     return false;
   return relaxing ? (state > PlayerInLimbo) : (state == PlayerAlive);
-};
+}
 
 void PlayerInfo::setPaused(bool _paused) {
   paused = _paused;
   pausedSince = now;
-};
+}
 
 void PlayerInfo::setAutoPilot(bool _autopilot) {
   autopilot = _autopilot;
-};
+}
 
 bool PlayerInfo::isTooMuchIdling(float kickThresh) {
   bool idling = false;
   if ((state > PlayerInLimbo) && (team != ObserverTeam)) {
-    int idletime = (int)(now - lastupdate);
-    int pausetime = 0;
-    if (paused && now - pausedSince > idletime)
-      pausetime = (int)(now - pausedSince);
-    idletime = idletime > pausetime ? idletime : pausetime;
-    if (idletime
-	> (now - lastmsg < kickThresh ? 3 * kickThresh : kickThresh)) {
-      DEBUG1("Kicking player %s [%d] idle %d\n", callSign, playerIndex,
-	     idletime);
+    const float idletime = (float)(now - lastupdate);
+    if (idletime > kickThresh) {
       idling = true;
     }
   }
   return idling;
-};
+}
 
 bool PlayerInfo::hasStartedToNotRespond() {
-  float notRespondingTime = BZDB.eval(StateDatabase::BZDB_NOTRESPONDINGTIME);
+  const float notRespondingTime =
+    BZDB.eval(StateDatabase::BZDB_NOTRESPONDINGTIME);
   bool startingToNotRespond = false;
   if (state > PlayerInLimbo) {
     bool oldnr = notResponding;
     notResponding = (now - lastupdate) > notRespondingTime;
-    if (!oldnr && notResponding)
+    if (!oldnr && notResponding) {
       startingToNotRespond = true;
+    }
   }
   return startingToNotRespond;
 }
 
-void PlayerInfo::hasSent(char message[]) {
+void PlayerInfo::hasSent() {
   lastmsg = now;
-  DEBUG1("Player %s [%d]: %s\n", callSign, playerIndex, message);
-};
+}
 
 bool PlayerInfo::hasPlayedEarly() {
   bool returnValue = playedEarly;
   playedEarly      = false;
   return returnValue;
-};
+}
 
 void PlayerInfo::setPlayedEarly(bool early) {
   playedEarly = early;
-};
+}
 
 void PlayerInfo::updateIdleTime() {
-  lastupdate = now;
-};
+  if (!paused && (state != PlayerDead)) {
+    lastupdate = now;
+  }
+}
 
-void	PlayerInfo::setReplayState(PlayerReplayState state) {
-  replayState = state;
+void	PlayerInfo::setReplayState(PlayerReplayState _state) {
+  replayState = _state;
 }
 
 PlayerReplayState PlayerInfo::getReplayState()
