@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
@@ -95,9 +95,9 @@ void			ServerListCache::saveCache()
       }
 
       // write out the index of the map
-      memset(&buffer, 0, sizeof(buffer));
+      memset(buffer, 0, sizeof(buffer));
       lenCpy = int((iter->first).size() < MAX_STRING ? (iter->first).size() : MAX_STRING);
-      strncpy(&buffer[0], (iter->first.c_str()), lenCpy);
+      strncpy(buffer, (iter->first.c_str()), lenCpy);
       outFile->write(buffer, sizeof(buffer));
 
       ServerItem x = iter->second;
